@@ -23,10 +23,13 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
-import re
-from apitools.base.py import exceptions as apitools_exceptions
 import enum
+import os
+import re
 
+from apitools.base.py import exceptions as apitools_exceptions
+from googlecloudsdk.api_lib import scheduler
+from googlecloudsdk.api_lib import tasks
 from googlecloudsdk.api_lib.app import build as app_cloud_build
 from googlecloudsdk.api_lib.app import deploy_app_command_util
 from googlecloudsdk.api_lib.app import deploy_command_util
@@ -36,7 +39,9 @@ from googlecloudsdk.api_lib.app import runtime_builders
 from googlecloudsdk.api_lib.app import util
 from googlecloudsdk.api_lib.app import version_util
 from googlecloudsdk.api_lib.app import yaml_parsing
+from googlecloudsdk.api_lib.datastore import index_api
 from googlecloudsdk.api_lib.storage import storage_util
+from googlecloudsdk.api_lib.tasks import app_deploy_migration_util
 from googlecloudsdk.api_lib.util import exceptions as core_api_exceptions
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import base
@@ -55,7 +60,8 @@ from googlecloudsdk.core.configurations import named_configs
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.console import progress_tracker
 from googlecloudsdk.core.util import files
-
+from googlecloudsdk.core.util import times
+import six
 
 _TASK_CONSOLE_LINK = """\
 https://console.cloud.google.com/appengine/taskqueues/cron?project={}
@@ -113,11 +119,15 @@ class InvalidRuntimeNameError(Error):
         'Must match regular expression [{}].'.format(runtime, allowed_regex))
 
 
-# TODO(b/27101941): Remove when commands all rely solely on the property.
-class ServiceManagementOption(enum.Enum):
-  """Enum declaring when to use Service Management for Flexible deployments."""
-  ALWAYS = 1
-  IF_PROPERTY_SET = 2
+class RequiredFileMissingError(Error):
+  """Error for skipped/ignored files that must be uploaded."""
+
+  def __init__(self, filename):
+    super(RequiredFileMissingError, self).__init__(
+        'Required file is not uploaded: [{}]. '
+        'This file should not be added to an ignore list ('
+        'https://cloud.google.com/sdk/gcloud/reference/topic/gcloudignore)'
+        .format(filename))
 
 
 class FlexImageBuildOptions(enum.Enum):
@@ -260,8 +270,9 @@ class ServiceDeployer(object):
       BuildArtifact, a wrapper which contains either the build ID for
         an in-progress build, or the name of the container image for a serial
         build. Possibly None if the service does not require an image.
+    Raises:
+      RequiredFileMissingError: if a required file is not uploaded.
     """
-
     build = None
     if image:
       if service.RequiresImage() and service.parsed.skip_files.regex:
@@ -270,13 +281,19 @@ class ServiceDeployer(object):
                     'already been built.'.format(new_version.service))
       return app_cloud_build.BuildArtifact.MakeImageArtifact(image)
     elif service.RequiresImage():
+      if not _AppYamlInSourceFiles(source_files, service.GetAppYamlBasename()):
+        raise RequiredFileMissingError(service.GetAppYamlBasename())
+
       if flex_image_build_option == FlexImageBuildOptions.ON_SERVER:
         cloud_build_options = {
             'appYamlPath': service.GetAppYamlBasename(),
         }
         timeout = properties.VALUES.app.cloud_build_timeout.Get()
         if timeout:
-          cloud_build_options['cloudBuildTimeout'] = timeout
+          build_timeout = int(
+              times.ParseDuration(timeout, default_suffix='s').total_seconds)
+          cloud_build_options['cloudBuildTimeout'] = six.text_type(
+              build_timeout) + 's'
         build = app_cloud_build.BuildArtifact.MakeBuildOptionsArtifact(
             cloud_build_options)
       else:
@@ -306,7 +323,7 @@ class ServiceDeployer(object):
             all_services, new_version, self.api_client,
             self.deploy_options.stop_previous_version)
       except apitools_exceptions.HttpError as err:
-        err_str = str(core_api_exceptions.HttpException(err))
+        err_str = six.text_type(core_api_exceptions.HttpException(err))
         raise VersionPromotionError(err_str)
     elif self.deploy_options.stop_previous_version:
       log.info('Not stopping previous version because new version was '
@@ -335,6 +352,9 @@ class ServiceDeployer(object):
 
     Returns:
       Dictionary mapping source files to Google Cloud Storage locations.
+
+    Raises:
+      RequiredFileMissingError: if a required file is not uploaded.
     """
     manifest = None
     # "Non-hermetic" services require file upload outside the Docker image
@@ -342,6 +362,10 @@ class ServiceDeployer(object):
     if (not image and
         (flex_image_build_option == FlexImageBuildOptions.ON_SERVER or
          not service_info.is_hermetic)):
+      if (service_info.env == env.FLEX and not _AppYamlInSourceFiles(
+          source_files, service_info.GetAppYamlBasename())):
+        raise RequiredFileMissingError(service_info.GetAppYamlBasename())
+
       limit = None
       if (service_info.env == env.STANDARD and
           service_info.runtime in _RUNTIMES_WITH_FILE_SIZE_LIMITS):
@@ -476,13 +500,24 @@ def ArgsDeploy(parser):
       instances are always running.""")
   parser.add_argument(
       '--image-url',
-      help='Deploy with a specific Docker image.  Docker url must be from one '
-      'of the valid gcr hostnames.')
+      help='(App Engine flexible environment only.) Deploy with a specific '
+      'Docker image. Docker url must be from one of the valid Container '
+      'Registry hostnames.')
+  parser.add_argument(
+      '--appyaml',
+      help='Deploy with a specific app.yaml that will replace '
+      'the one defined in the DEPLOYABLE.')
   parser.add_argument(
       '--promote',
       action=actions.StoreBooleanProperty(
           properties.VALUES.app.promote_by_default),
       help='Promote the deployed version to receive all traffic.')
+  parser.add_argument(
+      '--no-cache',
+      action='store_true',
+      default=False,
+      help='Skip caching mechanisms involved in the deployment process, in '
+      'particular do not use cached dependencies during the build step.')
   staging_group = parser.add_mutually_exclusive_group()
   staging_group.add_argument(
       '--skip-staging',
@@ -532,8 +567,8 @@ def RunDeploy(
     runtime_builder_strategy=runtime_builders.RuntimeBuilderStrategy.NEVER,
     parallel_build=True,
     flex_image_build_option=FlexImageBuildOptions.ON_CLIENT,
-    disable_build_cache=False,
-    dispatch_admin_api=False):
+    dispatch_admin_api=False,
+    use_ct_apis=False):
   """Perform a deployment based on the given args.
 
   Args:
@@ -551,9 +586,11 @@ def RunDeploy(
     flex_image_build_option: FlexImageBuildOptions, whether a flex deployment
       should upload files so that the server can build the image or build the
       image on client.
-    disable_build_cache: bool, disable the build cache.
     dispatch_admin_api: bool, speak to the (new) Admin API rather than the (old)
       Admin Console for config push of dispatch.yaml.
+    use_ct_apis: bool, if true, use the new Cloud Tasks and Cloud Scheduler FE
+      APIs instead of the soon-to-be deprecated admin-console-hr superapp for
+      queue.yaml and cron.yaml uploads
 
   Returns:
     A dict on the form `{'versions': new_versions, 'configs': updated_configs}`
@@ -570,7 +607,7 @@ def RunDeploy(
     stager = _MakeStager(args.skip_staging, use_beta_stager,
                          args.staging_command, staging_area)
     services, configs = deployables.GetDeployables(
-        args.deployables, stager, deployables.GetPathMatchers())
+        args.deployables, stager, deployables.GetPathMatchers(), args.appyaml)
     service_infos = [d.service_info for d in services]
 
     flags.ValidateImageUrl(args.image_url, service_infos)
@@ -636,7 +673,7 @@ def RunDeploy(
           args.image_url,
           all_services,
           app.gcrDomain,
-          disable_build_cache=disable_build_cache,
+          disable_build_cache=args.no_cache,
           flex_image_build_option=flex_image_build_option,
           ignore_file=args.ignore_file)
       new_versions.append(new_version)
@@ -654,6 +691,12 @@ def RunDeploy(
       with progress_tracker.ProgressTracker(message):
         if config.name == 'dispatch' and dispatch_admin_api:
           api_client.UpdateDispatchRules(config.GetRules())
+        elif config.name == yaml_parsing.ConfigYamlInfo.INDEX:
+          index_api.CreateMissingIndexes(project, config.parsed)
+        elif use_ct_apis and config.name == yaml_parsing.ConfigYamlInfo.QUEUE:
+          RunDeployCloudTasks(config)
+        elif use_ct_apis and config.name == yaml_parsing.ConfigYamlInfo.CRON:
+          RunDeployCloudScheduler(config)
         else:
           ac_client.UpdateConfig(config.name, config.parsed)
     metrics.CustomTimedEvent(metric_names.UPDATE_CONFIG)
@@ -669,6 +712,41 @@ def RunDeploy(
   }
 
 
+def RunDeployCloudTasks(config):
+  """Perform a deployment using Cloud Tasks API based on the given args.
+
+  Args:
+    config: A yaml_parsing.ConfigYamlInfo object for the parsed YAML file we
+        are going to process.
+
+  Returns:
+    A list of config file identifiers, see yaml_parsing.ConfigYamlInfo.
+  """
+  # TODO(b/169069379): Upgrade to use GA once the relevant code is promoted
+  tasks_api = tasks.GetApiAdapter(base.ReleaseTrack.BETA)
+  queues_data = app_deploy_migration_util.FetchCurrentQueuesData(tasks_api)
+  app_deploy_migration_util.ValidateQueueYamlFileConfig(config)
+  app_deploy_migration_util.DeployQueuesYamlFile(tasks_api, config, queues_data)
+
+
+def RunDeployCloudScheduler(config):
+  """Perform a deployment using Cloud Scheduler APIs based on the given args.
+
+  Args:
+    config: A yaml_parsing.ConfigYamlInfo object for the parsed YAML file we
+        are going to process.
+
+  Returns:
+    A list of config file identifiers, see yaml_parsing.ConfigYamlInfo.
+  """
+  # TODO(b/169069379): Upgrade to use GA once the relevant code is promoted
+  scheduler_api = scheduler.GetApiAdapter(base.ReleaseTrack.BETA,
+                                          legacy_cron=True)
+  jobs_data = app_deploy_migration_util.FetchCurrentJobsData(scheduler_api)
+  app_deploy_migration_util.ValidateCronYamlFileConfig(config)
+  app_deploy_migration_util.DeployCronYamlFile(scheduler_api, config, jobs_data)
+
+
 # TODO(b/30632016): Move to Epilog() when we have a good way to pass
 # information about the deployed versions
 def PrintPostDeployHints(new_versions, updated_configs):
@@ -752,11 +830,14 @@ def _PossiblyCreateApp(api_client, project):
       return api_client.GetApplication()
     raise exceptions.MissingApplicationError(project)
   except apitools_exceptions.HttpForbiddenError:
+    active_account = properties.VALUES.core.account.Get()
+    # pylint: disable=protected-access
     raise core_api_exceptions.HttpException(
         ('Permissions error fetching application [{}]. Please '
-         'make sure you are using the correct project ID and that '
-         'you have permission to view applications on the project.'.format(
-             api_client._FormatApp())))  # pylint: disable=protected-access
+         'make sure that you have permission to view applications on the '
+         'project and that {} has the App Engine Deployer '
+         '(roles/appengine.deployer) role.'.format(api_client._FormatApp(),
+                                                   active_account)))
 
 
 def _PossiblyRepairApp(api_client, app):
@@ -822,8 +903,18 @@ def GetRuntimeBuilderStrategy(release_track):
       return runtime_builders.RuntimeBuilderStrategy.NEVER
 
   if release_track is base.ReleaseTrack.GA:
-    return runtime_builders.RuntimeBuilderStrategy.WHITELIST_GA
+    return runtime_builders.RuntimeBuilderStrategy.ALLOWLIST_GA
   elif release_track is base.ReleaseTrack.BETA:
-    return runtime_builders.RuntimeBuilderStrategy.WHITELIST_BETA
+    return runtime_builders.RuntimeBuilderStrategy.ALLOWLIST_BETA
   else:
     raise ValueError('Unrecognized release track [{}]'.format(release_track))
+
+
+def _AppYamlInSourceFiles(source_files, app_yaml_path):
+  if not source_files:
+    return False
+
+  # TODO(b/171495697) until the bug is fixed, the app yaml has to be located in
+  #  the root of the app code, hence we're searching only the filename
+  app_yaml_filename = os.path.basename(app_yaml_path)
+  return any([f == app_yaml_filename for f in source_files])
