@@ -38,6 +38,7 @@ from googlecloudsdk.core.updater import installers
 from googlecloudsdk.core.updater import schemas
 from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import files
+from googlecloudsdk.core.util import platforms
 
 import six
 from six.moves import urllib
@@ -197,12 +198,13 @@ class ComponentSnapshot(object):
     """
     extra_repo = url if is_extra_repo else None
     try:
-      response = installers.ComponentInstaller.MakeRequest(url, command_path)
+      response = installers.MakeRequest(url, command_path)
     except (urllib.error.HTTPError, urllib.error.URLError, ssl.SSLError):
       log.debug('Could not fetch [{url}]'.format(url=url), exc_info=True)
       response = None
     except ValueError as e:
-      if not e.message or 'unknown url type' not in e.message:
+      message = str(e)
+      if not message or 'unknown url type' not in message:
         raise
       log.debug('Bad repository url: [{url}]'.format(url=url), exc_info=True)
       raise URLFetchError(malformed=True, extra_repo=extra_repo)
@@ -308,24 +310,19 @@ class ComponentSnapshot(object):
       for dep_id in dep_ids:
         self.__consumers[dep_id].add(component_id)
 
-  def _ClosureFor(self, ids, dependencies=False, consumers=False,
-                  platform_filter=None):
-    """Calculates a dependency closure for the components with the given ids.
+  def _ClosureFor(self, ids, adjacencies, component_filter):
+    """Calculates a connected closure for the components with the given ids.
 
-    This can calculate a dependency closure, consumer closure, or the union of
-    both depending on the flags.  If both dependencies and consumers are set to
-    True, this is basically calculating the set of components that are connected
-    by dependencies to anything in the starting set.  The given ids, are always
-    included in the output if they are valid components.
+    Performs a breadth first search starting with the given component ids, and
+    returns the set of components reachable via the given adjacency map.
 
     Args:
-      ids: list of str, The component ids to get the dependency closure for.
-      dependencies: bool, True to add dependencies of the given components to
-        the closure.
-      consumers: bool, True to add consumers of the given components to the
+      ids: [str], The component ids to get the closure for.
+      adjacencies: {str: set}, Map of component ids to the set of their
+        adjacent component ids.
+      component_filter: schemas.Component -> bool, A function applied to
+        components that determines whether or not to include them in the
         closure.
-      platform_filter: platforms.Platform, A platform that components must
-        match to be pulled into the dependency closure.
 
     Returns:
       set of str, The set of component ids in the closure.
@@ -336,13 +333,10 @@ class ComponentSnapshot(object):
       current = to_process.popleft()
       if current not in self.components or current in closure:
         continue
-      if not self.components[current].platform.Matches(platform_filter):
+      if not component_filter(self.components[current]):
         continue
       closure.add(current)
-      if dependencies:
-        to_process.extend(self.__dependencies[current])
-      if consumers:
-        to_process.extend(self.__consumers[current])
+      to_process.extend(adjacencies[current])
     return closure
 
   def ComponentFromId(self, component_id):
@@ -393,8 +387,9 @@ class ComponentSnapshot(object):
       set of str, All component ids that are in the dependency closure,
       including the given components.
     """
-    return self._ClosureFor(component_ids, dependencies=True, consumers=False,
-                            platform_filter=platform_filter)
+    component_filter = lambda c: c.platform.Matches(platform_filter)
+    return self._ClosureFor(
+        component_ids, self.__dependencies, component_filter)
 
   def ConsumerClosureForComponents(self, component_ids, platform_filter=None):
     """Gets all the components that depend on any of the given ids.
@@ -409,8 +404,8 @@ class ComponentSnapshot(object):
       set of str, All component ids that are in the consumer closure, including
       the given components.
     """
-    return self._ClosureFor(component_ids, dependencies=False, consumers=True,
-                            platform_filter=platform_filter)
+    component_filter = lambda c: c.platform.Matches(platform_filter)
+    return self._ClosureFor(component_ids, self.__consumers, component_filter)
 
   def ConnectedComponents(self, component_ids, platform_filter=None):
     """Gets all the components that are connected to any of the given ids.
@@ -428,8 +423,39 @@ class ComponentSnapshot(object):
       set of str, All component ids that are connected to the given ids,
       including the given components.
     """
-    return self._ClosureFor(component_ids, dependencies=True, consumers=True,
-                            platform_filter=platform_filter)
+    adjacencies = {
+        c_id: self.__dependencies[c_id] | self.__consumers[c_id]
+        for c_id in self.components}
+    component_filter = lambda c: c.platform.Matches(platform_filter)
+    return self._ClosureFor(component_ids, adjacencies, component_filter)
+
+  def StronglyConnectedComponents(self, component_id):
+    """Gets the components strongly connected to the given component id.
+
+    In other words, this functions returns the strongly connected "component" of
+    the dependency graph for the given component id. In this context the
+    strongly connected "component" is the set of ids whose components are both
+    dependencies and consumers of the given component. In practice this can be
+    used to determine the platform-specific subcomponent ID's for a given
+    component ID, since they will always be mutually dependent.
+
+    Args:
+      component_id: str, The id of the component for which to get the strongly
+        connected components.
+
+    Returns:
+      set of str, The ids of the components that are strongly connected to the
+        component with the given id.
+    """
+    # Unlike the other functions above, we don't filter by platform since we
+    # want all platform-specific subcomponents to match.
+    component_filter = lambda c: True
+    dependency_closure = self._ClosureFor(
+        [component_id], self.__dependencies, component_filter)
+    consumer_closure = self._ClosureFor(
+        [component_id], self.__consumers, component_filter)
+
+    return dependency_closure & consumer_closure
 
   def GetEffectiveComponentSize(self, component_id, platform_filter=None):
     """Computes the effective size of the given component.
@@ -482,7 +508,8 @@ class ComponentSnapshot(object):
         size += d.data.size or 0
     return size
 
-  def CreateDiff(self, latest_snapshot, platform_filter=None):
+  def CreateDiff(self, latest_snapshot, platform_filter=None,
+                 enable_fallback=False):
     """Creates a ComponentSnapshotDiff based on this snapshot and the given one.
 
     Args:
@@ -490,12 +517,15 @@ class ComponentSnapshot(object):
         want to compare to.
       platform_filter: platforms.Platform, A platform that components must
         match in order to be considered for any operations.
+      enable_fallback: bool, True to enable fallback from darwin arm64 version
+        to darwin x86_64 version of the component.
 
     Returns:
       A ComponentSnapshotDiff object.
     """
     return ComponentSnapshotDiff(self, latest_snapshot,
-                                 platform_filter=platform_filter)
+                                 platform_filter=platform_filter,
+                                 enable_fallback=enable_fallback)
 
   def CreateComponentInfos(self, platform_filter=None):
     all_components = self.AllComponentIdsMatching(platform_filter)
@@ -537,7 +567,8 @@ class ComponentSnapshot(object):
         if key not in ('components', 'schema_version', 'revision', 'version'):
           del sdk_def_dict[key]
     files.WriteFileContents(
-        path, json.dumps(sdk_def_dict, indent=2, sort_keys=True))
+        path, json.dumps(sdk_def_dict, indent=2, sort_keys=True,
+                         separators=(',', ': ')))
 
 
 class ComponentSnapshotDiff(object):
@@ -553,7 +584,11 @@ class ComponentSnapshotDiff(object):
     latest: CompnentSnapshot, The new snapshot that is being compared.
   """
 
-  def __init__(self, current, latest, platform_filter=None):
+  DARWIN_X86_64 = platforms.Platform(platforms.OperatingSystem.MACOSX,
+                                     platforms.Architecture.x86_64)
+
+  def __init__(self, current, latest, platform_filter=None,
+               enable_fallback=False):
     """Creates a new diff between two ComponentSnapshots.
 
     Args:
@@ -561,17 +596,43 @@ class ComponentSnapshotDiff(object):
       latest: The ComponentSnapshot representing a new state we can move to
       platform_filter: platforms.Platform, A platform that components must
         match in order to be considered for any operations.
+      enable_fallback: bool, True to enable fallback from darwin arm64 version
+        to darwin x86_64 version of the component.
     """
     self.current = current
     self.latest = latest
     self.__platform_filter = platform_filter
+    self.__enable_fallback = (
+        enable_fallback and platform_filter and
+        platform_filter.operating_system == platforms.OperatingSystem.MACOSX and
+        platform_filter.architecture == platforms.Architecture.arm)
 
     self.__all_components = (current.AllComponentIdsMatching(platform_filter) |
                              latest.AllComponentIdsMatching(platform_filter))
-    self.__diffs = [
-        ComponentDiff(component_id, current, latest,
-                      platform_filter=platform_filter)
-        for component_id in self.__all_components]
+
+    if self.__enable_fallback:
+      # Fall back to darwin_x86 version when darwin_arm version is unavailable.
+      self.__all_darwin_x86_64_components = (
+          current.AllComponentIdsMatching(self.DARWIN_X86_64) |
+          latest.AllComponentIdsMatching(self.DARWIN_X86_64))
+      self.__darwin_x86_64_components = (self.__all_darwin_x86_64_components -
+                                         self.__all_components)
+      self.__native_all_components = set(self.__all_components)
+      self.__all_components |= self.__darwin_x86_64_components
+
+      self.__diffs = [
+          ComponentDiff(component_id, current, latest,
+                        platform_filter=platform_filter)
+          for component_id in self.__native_all_components]
+      self.__diffs.extend([
+          ComponentDiff(component_id, current, latest,
+                        platform_filter=self.DARWIN_X86_64)
+          for component_id in self.__darwin_x86_64_components])
+    else:
+      self.__diffs = [
+          ComponentDiff(component_id, current, latest,
+                        platform_filter=platform_filter)
+          for component_id in self.__all_components]
 
     self.__removed_components = set(diff.id for diff in self.__diffs
                                     if diff.state is ComponentState.REMOVED)
@@ -590,6 +651,24 @@ class ComponentSnapshotDiff(object):
     Returns:
       set of str, The component ids that do not exist anywhere.
     """
+    if self.__enable_fallback:
+      native_invalid_ids = set(component_ids) - self.__native_all_components
+      arm_x86_ids = native_invalid_ids & self.__darwin_x86_64_components
+      if arm_x86_ids:
+        rosetta2_installed = os.path.isfile(
+            '/Library/Apple/System/Library/LaunchDaemons/com.apple.oahd.plist')
+        if rosetta2_installed:
+          log.warning('The ARM versions of the following components are not '
+                      'available yet, using x86_64 versions instead: [{}].'
+                      .format(', '.join(arm_x86_ids)))
+        else:
+          log.warning('The ARM versions of the components [{}] are not '
+                      'available yet. To download and execute the x86_64 '
+                      'version of the components, please install Rosetta 2 '
+                      'first by running the command: '
+                      'softwareupdate --install-rosetta.'
+                      .format(', '.join(arm_x86_ids)))
+          return (set(component_ids) - self.__all_components) | arm_x86_ids
     return set(component_ids) - self.__all_components
 
   def AllDiffs(self):
@@ -659,10 +738,24 @@ class ComponentSnapshotDiff(object):
     """
     # Get the full set of everything that needs to be updated together that we
     # currently have installed
-    connected = self.current.ConnectedComponents(
-        update_seed, platform_filter=self.__platform_filter)
-    connected |= self.latest.ConnectedComponents(
-        connected | set(update_seed), platform_filter=self.__platform_filter)
+    if self.__enable_fallback:
+      native_seed = set(update_seed) & self.__native_all_components
+      darwin_x86_64 = set(update_seed) - native_seed
+      connected = self.current.ConnectedComponents(
+          native_seed, platform_filter=self.__platform_filter)
+      connected |= self.latest.ConnectedComponents(
+          connected | set(native_seed), platform_filter=self.__platform_filter)
+      connected_darwin_x86_64 = self.current.ConnectedComponents(
+          darwin_x86_64, platform_filter=self.DARWIN_X86_64)
+      connected_darwin_x86_64 |= self.latest.ConnectedComponents(
+          connected_darwin_x86_64 | set(darwin_x86_64),
+          platform_filter=self.DARWIN_X86_64)
+      connected |= connected_darwin_x86_64
+    else:
+      connected = self.current.ConnectedComponents(
+          update_seed, platform_filter=self.__platform_filter)
+      connected |= self.latest.ConnectedComponents(
+          connected | set(update_seed), platform_filter=self.__platform_filter)
     removal_candidates = connected & set(self.current.components.keys())
     # We need to remove anything that no longer exists or that has been updated
     return (self.__removed_components |
@@ -686,20 +779,44 @@ class ComponentSnapshotDiff(object):
       set of str, The component ids that should be removed.
     """
     installed_components = list(self.current.components.keys())
-    local_connected = self.current.ConnectedComponents(
-        update_seed, platform_filter=self.__platform_filter)
-    all_required = self.latest.DependencyClosureForComponents(
-        local_connected | set(update_seed),
-        platform_filter=self.__platform_filter)
 
-    # Add in anything in the connected graph that we already have installed.
-    # Even though the update seed might not depend on it, if it was already
-    # installed, it might have been removed in ToRemove() if an update was
-    # available so we should put it back.
-    remote_connected = self.latest.ConnectedComponents(
-        local_connected | set(update_seed),
-        platform_filter=self.__platform_filter)
-    all_required |= remote_connected & set(installed_components)
+    if self.__enable_fallback:
+      native_seed = set(update_seed) & self.__native_all_components
+      darwin_x86_64 = set(update_seed) - native_seed
+
+      local_connected = self.current.ConnectedComponents(
+          native_seed, platform_filter=self.__platform_filter)
+      all_required = self.latest.DependencyClosureForComponents(
+          local_connected | set(native_seed),
+          platform_filter=self.__platform_filter)
+      local_connected_darwin_x86_64 = self.current.ConnectedComponents(
+          darwin_x86_64, platform_filter=self.DARWIN_X86_64)
+      all_required |= self.latest.DependencyClosureForComponents(
+          local_connected_darwin_x86_64 | set(darwin_x86_64),
+          platform_filter=self.DARWIN_X86_64)
+
+      remote_connected = self.latest.ConnectedComponents(
+          local_connected | set(native_seed),
+          platform_filter=self.__platform_filter)
+      remote_connected |= self.latest.ConnectedComponents(
+          local_connected_darwin_x86_64 | set(darwin_x86_64),
+          platform_filter=self.__platform_filter)
+      all_required |= remote_connected & set(installed_components)
+    else:
+      local_connected = self.current.ConnectedComponents(
+          update_seed, platform_filter=self.__platform_filter)
+      all_required = self.latest.DependencyClosureForComponents(
+          local_connected | set(update_seed),
+          platform_filter=self.__platform_filter)
+
+      # Add in anything in the connected graph that we already have installed.
+      # Even though the update seed might not depend on it, if it was already
+      # installed, it might have been removed in ToRemove() if an update was
+      # available so we should put it back.
+      remote_connected = self.latest.ConnectedComponents(
+          local_connected | set(update_seed),
+          platform_filter=self.__platform_filter)
+      all_required |= remote_connected & set(installed_components)
 
     different = self.__new_components | self.__updated_components
 
