@@ -94,13 +94,28 @@ _SHELL_RCFILES = [
     'gcfilesys.zsh.inc'
 ]
 
-BUNDLED_PYTHON_COMPONENT = 'bundled-python3'
+BUNDLED_PYTHON_COMPONENTS = ['bundled-python3', 'bundled-python3-unix']
 BUNDLED_PYTHON_REMOVAL_WARNING = (
     'This command is running using a bundled installation of Python. '
     'If you remove it, you may have no way to run this command.\n'
 )
 
 
+def _HaltIfBundledPythonUnix():
+  current_os = platforms.OperatingSystem.Current()
+  if current_os in [platforms.OperatingSystem.MACOSX]:
+    log.error(
+        'To remove bundled python component please re-run this command '
+        'after setting CLOUDSDK_PYTHON env var to preferred system python.'
+    )
+    sys.exit(1)
+
+
+def _ContainsBundledPython(components):
+  """Return true if components list contains 'bundled python' component(s)."""
+  return len(list(set(BUNDLED_PYTHON_COMPONENTS) & set(components))) >= 1
+
+
 class Error(exceptions.Error):
   """Base exception for the update_manager module."""
   pass
@@ -174,6 +189,8 @@ class UpdateManager(object):
 
   BIN_DIR_NAME = 'bin'
   VERSIONED_SNAPSHOT_FORMAT = 'components-v{0}.json'
+  DARWIN_X86_64 = platforms.Platform(platforms.OperatingSystem.MACOSX,
+                                     platforms.Architecture.x86_64)
 
   @staticmethod
   def GetAdditionalRepositories():
@@ -252,7 +269,8 @@ class UpdateManager(object):
     # pylint: disable=protected-access
     manager._PerformUpdateCheck(command_path, force=force)
 
-  def __init__(self, sdk_root=None, url=None, platform_filter=None, warn=True):
+  def __init__(self, sdk_root=None, url=None, platform_filter=None, warn=True,
+               enable_fallback=False):
     """Creates a new UpdateManager.
 
     Args:
@@ -269,6 +287,8 @@ class UpdateManager(object):
         to False when using this class for background operations like checking
         for updates so the user only sees the warnings when they are actually
         dealing directly with the component manager.
+      enable_fallback: bool, True to enable fallback from darwin arm64 version
+        to darwin x86_64 version of the component.
 
     Raises:
       local_state.InvalidSDKRootError: If the Cloud SDK root cannot be found.
@@ -299,6 +319,10 @@ class UpdateManager(object):
     self.__text_wrapper = textwrap.TextWrapper(replace_whitespace=False,
                                                drop_whitespace=False)
     self.__warn = warn
+    self.__enable_fallback = (
+        enable_fallback and platform_filter and
+        platform_filter.operating_system == platforms.OperatingSystem.MACOSX and
+        platform_filter.architecture == platforms.Architecture.arm)
 
     fixed_version = properties.VALUES.component_manager.fixed_sdk_version.Get()
     self.__fixed_version = fixed_version
@@ -615,12 +639,16 @@ version [{1}].  To clear your fixed version setting, run:
     latest_snapshot = self._GetLatestSnapshot(version=version,
                                               command_path=command_path)
     diff = install_state.DiffCurrentState(
-        latest_snapshot, platform_filter=self.__platform_filter)
+        latest_snapshot, platform_filter=self.__platform_filter,
+        enable_fallback=self.__enable_fallback)
     return install_state, diff
 
-  def GetCurrentVersionsInformation(self):
+  def GetCurrentVersionsInformation(self, include_hidden=False):
     """Get the current version for every installed component.
 
+    Args:
+      include_hidden: bool, include hidden components.
+
     Returns:
       {str:str}, A mapping from component id to version string.
     """
@@ -629,7 +657,8 @@ version [{1}].  To clear your fixed version setting, run:
     installed_components = current_state.InstalledComponents()
     for component_id, component in six.iteritems(installed_components):
       component_def = component.ComponentDefinition()
-      if component_def.is_configuration or component_def.is_hidden:
+      if (component_def.is_configuration or
+          component_def.is_hidden and not include_hidden):
         continue
       versions[component_id] = component.VersionString()
     return versions
@@ -922,7 +951,8 @@ version [{1}].  To clear your fixed version setting, run:
     # necessary.
     config.EnsureSDKWriteAccess(self.__sdk_root)
 
-    if self.IsPythonBundled() and BUNDLED_PYTHON_COMPONENT in to_remove:
+    if self.IsPythonBundled() and _ContainsBundledPython(to_remove):
+      _HaltIfBundledPythonUnix()
       log.warning(BUNDLED_PYTHON_REMOVAL_WARNING)
     self._RestartIfUsingBundledPython(args=restart_args)
 
@@ -1195,6 +1225,9 @@ To revert your SDK to the previously installed version, you may run:
 
     to_remove = snapshot.ConsumerClosureForComponents(
         ids, platform_filter=self.__platform_filter)
+    if self.__enable_fallback:
+      to_remove |= snapshot.ConsumerClosureForComponents(
+          ids, platform_filter=self.DARWIN_X86_64)
     if not to_remove:
       self.__Write(log.status, 'No components to remove.\n')
       return
@@ -1211,7 +1244,8 @@ To revert your SDK to the previously installed version, you may run:
     # necessary.
     config.EnsureSDKWriteAccess(self.__sdk_root)
 
-    if self.IsPythonBundled() and BUNDLED_PYTHON_COMPONENT in to_remove:
+    if self.IsPythonBundled() and _ContainsBundledPython(to_remove):
+      _HaltIfBundledPythonUnix()
       log.warning(BUNDLED_PYTHON_REMOVAL_WARNING)
     self._RestartIfUsingBundledPython()
 
@@ -1259,8 +1293,8 @@ To revert your SDK to the previously installed version, you may run:
     config.EnsureSDKWriteAccess(self.__sdk_root)
 
     backup_has_bundled_python = (
-        BUNDLED_PYTHON_COMPONENT in
-        install_state.BackupInstallationState().InstalledComponents())
+        _ContainsBundledPython(
+            install_state.BackupInstallationState().InstalledComponents()))
     if self.IsPythonBundled() and not backup_has_bundled_python:
       log.warning(BUNDLED_PYTHON_REMOVAL_WARNING)
     self._RestartIfUsingBundledPython()
