@@ -33,10 +33,11 @@ class UpdatePerimetersGA(base.UpdateCommand):
 
   @staticmethod
   def Args(parser):
-    UpdatePerimetersGA.ArgsVersioned(parser, version='v1', track='GA')
+    UpdatePerimetersGA.ArgsVersioned(
+        parser, version='v1', track=base.ReleaseTrack.GA)
 
   @staticmethod
-  def ArgsVersioned(parser, version='v1', track='GA'):
+  def ArgsVersioned(parser, version='v1', track=base.ReleaseTrack.GA):
     perimeters.AddResourceArg(parser, 'to update')
     perimeters.AddPerimeterUpdateArgs(parser, version=version, track=track)
 
@@ -59,47 +60,18 @@ class UpdatePerimetersGA(base.UpdateCommand):
         restricted_services=perimeters.ParseRestrictedServices(args, result),
         levels=perimeters.ParseLevels(args, result,
                                       perimeter_ref.accessPoliciesId),
-    )
+        vpc_allowed_services=perimeters.ParseVpcRestriction(
+            args, result, self._API_VERSION),
+        enable_vpc_accessible_services=args.enable_vpc_accessible_services,
+        ingress_policies=perimeters.ParseUpdateDirectionalPoliciesArgs(
+            args, self._release_track, 'ingress-policies'),
+        egress_policies=perimeters.ParseUpdateDirectionalPoliciesArgs(
+            args, self._release_track, 'egress-policies'))
 
   def Patch(self, client, args, result, perimeter_ref, description, title,
-            perimeter_type, resources, restricted_services, levels):
-    return client.Patch(
-        perimeter_ref,
-        description=description,
-        title=title,
-        perimeter_type=perimeter_type,
-        resources=resources,
-        restricted_services=restricted_services,
-        levels=levels)
-
-
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
-class UpdatePerimetersBeta(UpdatePerimetersGA):
-  """Update an existing access zone."""
-  _INCLUDE_UNRESTRICTED = False
-  _API_VERSION = 'v1beta'
-
-  @staticmethod
-  def Args(parser):
-    UpdatePerimetersGA.ArgsVersioned(parser, version='v1beta', track='BETA')
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class UpdatePerimetersAlpha(UpdatePerimetersGA):
-  """Update an existing access zone."""
-  _INCLUDE_UNRESTRICTED = False
-  _API_VERSION = 'v1beta'
-
-  @staticmethod
-  def Args(parser):
-    UpdatePerimetersGA.ArgsVersioned(parser, version='v1beta', track='ALPHA')
-
-  def Patch(self, client, args, perimeter_ref, result, description, title,
-            perimeter_type, resources, restricted_services, levels):
-    vpc_allowed_services = perimeters.ParseVpcRestriction(
-        args, result, self._API_VERSION)
-    enable_vpc_restriction = args.enable_vpc_service_restriction
-
+            perimeter_type, resources, restricted_services, levels,
+            vpc_allowed_services, enable_vpc_accessible_services,
+            ingress_policies, egress_policies):
     return client.Patch(
         perimeter_ref,
         description=description,
@@ -109,5 +81,51 @@ class UpdatePerimetersAlpha(UpdatePerimetersGA):
         restricted_services=restricted_services,
         levels=levels,
         vpc_allowed_services=vpc_allowed_services,
-        enable_vpc_service_restriction=enable_vpc_restriction,
-    )
+        enable_vpc_accessible_services=enable_vpc_accessible_services,
+        ingress_policies=ingress_policies,
+        egress_policies=egress_policies)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class UpdatePerimetersBeta(UpdatePerimetersGA):
+  """Update an existing access zone."""
+  _INCLUDE_UNRESTRICTED = False
+  _API_VERSION = 'v1'
+
+  @staticmethod
+  def Args(parser):
+    UpdatePerimetersGA.ArgsVersioned(
+        parser, version='v1', track=base.ReleaseTrack.BETA)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class UpdatePerimetersAlpha(UpdatePerimetersBeta):
+  """Update an existing access zone."""
+  _INCLUDE_UNRESTRICTED = False
+  _API_VERSION = 'v1alpha'
+
+  @staticmethod
+  def Args(parser):
+    UpdatePerimetersGA.ArgsVersioned(
+        parser, version='v1alpha', track=base.ReleaseTrack.ALPHA)
+
+
+detailed_help = {
+    'brief':
+        'Update the enforced configuration for an existing Service Perimeter.',
+    'DESCRIPTION':
+        ('This command updates the enforced configuration (`status`) of a '
+         'Service Perimeter.'),
+    'EXAMPLES':
+        ('To update the enforced configuration for a Service Perimeter:\n\n'
+         '  $ {command} my-perimeter '
+         '--add-resources="projects/123,projects/456" '
+         '--remove-restricted-services="storage.googleapis.com" '
+         '--add-access-levels="accessPolicies/123/accessLevels/a_level" '
+         '--enable-vpc-accessible-services '
+         '--clear-vpc-allowed-services')
+}
+
+UpdatePerimetersGA.detailed_help = detailed_help
+UpdatePerimetersBeta.detailed_help = detailed_help
+UpdatePerimetersAlpha.detailed_help = detailed_help
