@@ -26,7 +26,7 @@ from googlecloudsdk.core import log
 
 
 @base.Hidden
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
 class Delete(base.Command):
   r"""Delete an organization policy.
 
@@ -57,13 +57,9 @@ class Delete(base.Command):
     Returns:
        If the policy is deleted, then messages.GoogleProtobufEmpty.
     """
-    policy_service = org_policy_service.PolicyService()
-    org_policy_messages = org_policy_service.OrgPolicyMessages()
-
+    org_policy_api = org_policy_service.OrgPolicyApi(self.ReleaseTrack())
     policy_name = utils.GetPolicyNameFromArgs(args)
 
-    delete_request = org_policy_messages.OrgpolicyPoliciesDeleteRequest(
-        name=policy_name)
-    delete_response = policy_service.Delete(delete_request)
+    delete_response = org_policy_api.DeletePolicy(policy_name)
     log.DeletedResource(policy_name, 'policy')
     return delete_response
