@@ -144,7 +144,7 @@ struct ast_custom_function {
 					 * 'dangerous', and should not be run directly
 					 * from external interfaces (AMI, ARI, etc.)
 					 * \since 12 */
-	unsigned int write_escalates:1; /*!< The write function is to be considerd
+	unsigned int write_escalates:1; /*!< The write function is to be considered
 					 * 'dangerous', and should not be run directly
 					 * from external interfaces (AMI, ARI, etc.)
 					 * \since 12 */
@@ -789,7 +789,7 @@ int ast_findlabel_extension2(struct ast_channel *c, struct ast_context *con,
  * \brief Looks for a valid matching extension
  *
  * \param c not really important
- * \param context context to serach within
+ * \param context context to search within
  * \param exten extension to check
  * \param priority priority of extension path
  * \param callerid callerid of extension being searched for
@@ -809,7 +809,7 @@ int ast_canmatch_extension(struct ast_channel *c, const char *context,
  * \brief Looks to see if adding anything to this extension might match something. (exists ^ canmatch)
  *
  * \param c not really important XXX
- * \param context context to serach within
+ * \param context context to search within
  * \param exten extension to check
  * \param priority priority of extension path
  * \param callerid callerid of extension being searched for
@@ -1013,7 +1013,7 @@ int ast_context_remove_extension_callerid2(struct ast_context *con, const char *
 /*!
  * \brief Add an ignorepat
  *
- * \param context which context to add the ignorpattern to
+ * \param context which context to add the ignorepattern to
  * \param ignorepat ignorepattern to set up for the extension
  * \param registrar registrar of the ignore pattern
  *
@@ -1026,7 +1026,7 @@ int ast_context_add_ignorepat(const char *context, const char *ignorepat, const
 
 int ast_context_add_ignorepat2(struct ast_context *con, const char *ignorepat, const char *registrar);
 
-/*
+/*!
  * \brief Remove an ignorepat
  *
  * \param context context from which to remove the pattern
@@ -1401,9 +1401,9 @@ void pbx_substitute_variables_helper(struct ast_channel *c, const char *cp1, cha
 void pbx_substitute_variables_varshead(struct varshead *headp, const char *cp1, char *cp2, int count);
 void pbx_substitute_variables_helper_full(struct ast_channel *c, struct varshead *headp, const char *cp1, char *cp2, int cp2_size, size_t *used);
 /*! @} */
-/*! @} */
 
-/*! @name Substitution routines, using dynamic string buffers */
+/*! @name Substitution routines, using dynamic string buffers
+ * @{ */
 
 /*!
  * \param buf Result will be placed in this buffer.
