@@ -181,7 +181,10 @@
 #define NDPI_JABBER_FT_TIMEOUT				         5
 #define NDPI_SOULSEEK_CONNECTION_IP_TICK_TIMEOUT               600
 
+#ifndef _NDPI_CONFIG_H_
 #include "ndpi_config.h"	/* To have access to NDPI_ENABLE_DEBUG_MESSAGES */
+#define _NDPI_CONFIG_H_
+#endif
 
 #ifdef NDPI_ENABLE_DEBUG_MESSAGES
  #define NDPI_LOG(proto, m, log_level, args...)		                                 \
@@ -257,6 +260,7 @@
   ((x.u6_addr.u6_addr64[0] < y.u6_addr.u6_addr64[0]) || ((x.u6_addr.u6_addr64[0] == y.u6_addr.u6_addr64[0]) && (x.u6_addr.u6_addr64[1] < y.u6_addr.u6_addr64[1])))
 
 #define NDPI_NUM_BITS              512
+#define NDPI_NUM_BITS_MASK         (512-1)
 
 #define NDPI_BITS /* 32 */ (sizeof(ndpi_ndpi_mask) * 8 /* number of bits in a byte */)        /* bits per mask */
 #define howmanybits(x, y)   (((x)+((y)-1))/(y))
@@ -278,18 +282,18 @@
 #define NDPI_BITMASK_SET_ALL(a)   NDPI_ONE(&a)
 #define NDPI_BITMASK_SET(a, b)    { memcpy(&a, &b, sizeof(NDPI_PROTOCOL_BITMASK)); }
 
-#define NDPI_SET_BIT(num, n)    num |= 1UL << ( n )
-#define NDPI_CLR_BIT(num, n)    num &= ~(1UL << ( n ))
-#define NDPI_CLR_BIT(num, n)    num &= ~(1UL << ( n ))
-#define NDPI_ISSET_BIT(num, n)  (num & (1UL << ( n )))
+#define NDPI_SET_BIT(num, n)    num |= 1ULL << ( n )
+#define NDPI_CLR_BIT(num, n)    num &= ~(1ULL << ( n ))
+#define NDPI_CLR_BIT(num, n)    num &= ~(1ULL << ( n ))
+#define NDPI_ISSET_BIT(num, n)  (num & (1ULL << ( n )))
 #define NDPI_ZERO_BIT(num)      num = 0
 
 /* this is a very very tricky macro *g*,
  * the compiler will remove all shifts here if the protocol is static...
  */
-#define NDPI_ADD_PROTOCOL_TO_BITMASK(bmask,value)     NDPI_SET(&bmask,value)
-#define NDPI_DEL_PROTOCOL_FROM_BITMASK(bmask,value)   NDPI_CLR(&bmask,value)
-#define NDPI_COMPARE_PROTOCOL_TO_BITMASK(bmask,value) NDPI_ISSET(&bmask,value)
+#define NDPI_ADD_PROTOCOL_TO_BITMASK(bmask,value)     NDPI_SET(&bmask,   value & NDPI_NUM_BITS_MASK)
+#define NDPI_DEL_PROTOCOL_FROM_BITMASK(bmask,value)   NDPI_CLR(&bmask,   value & NDPI_NUM_BITS_MASK)
+#define NDPI_COMPARE_PROTOCOL_TO_BITMASK(bmask,value) NDPI_ISSET(&bmask, value & NDPI_NUM_BITS_MASK)
 
 #define NDPI_SAVE_AS_BITMASK(bmask,value)  { NDPI_ZERO(&bmask) ; NDPI_ADD_PROTOCOL_TO_BITMASK(bmask, value); }
 
@@ -344,8 +348,8 @@
 #define NDPI_MAX_DNS_REQUESTS                   16
 #define NDPI_MIN_NUM_STUN_DETECTION             8
 
-#define NDPI_MAJOR                              3
-#define NDPI_MINOR                              4
+#define NDPI_MAJOR                              4
+#define NDPI_MINOR                              0
 #define NDPI_PATCH                              0
 
 /* IMPORTANT: order according to its severity */
