@@ -96,9 +96,9 @@ tie %Config, 'Config', {
     inc_version_list => ' ',
     intsize => '4',
     ldlibpthname => 'LD_LIBRARY_PATH',
-    libpth => '/usr/lib /usr/local/lib /usr/lib/clang/8.0.1/lib /usr/lib',
+    libpth => '/usr/lib /usr/local/lib /usr/lib/clang/13.0.0/lib /usr/lib',
     osname => 'freebsd',
-    osvers => '12.1-release-p18-hbsd',
+    osvers => '13.0-stable',
     path_sep => ':',
     privlibexp => '/usr/local/lib/perl5/5.32',
     scriptdir => '/usr/local/bin',
