@@ -19,6 +19,6 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-__version__ = '2.9.7'
+__version__ = '2.11.6'
 __author__ = 'Ansible, Inc.'
-__codename__ = 'Immigrant Song'
+__codename__ = 'Hey Hey, What Can I Do'
