@@ -1,37 +1,29 @@
 #!/usr/local/bin/python3.8
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
-#
+# -*- coding: utf-8 -*-
+# Copyright 2021 Red Hat
+# GNU General Public License v3.0+
+# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
+
 """
 The module file for ios_l3_interfaces
 """
+
 from __future__ import absolute_import, division, print_function
 
 __metaclass__ = type
 
-
 DOCUMENTATION = """
 module: ios_l3_interfaces
 short_description: L3 interfaces resource module
 description:
 - This module provides declarative management of Layer-3 interface on Cisco IOS devices.
 version_added: 1.0.0
-author: Sumit Jaiswal (@justjais)
+author:
+- Sagar Paul (@KB-perByte)
+- Sumit Jaiswal (@justjais)
 notes:
-- Tested against Cisco IOSv Version 15.2 on VIRL.
+- Tested against Cisco IOSv Version 15.6.
+- Using deleted state without config will delete all l3 attributes from all the interfaces.
 options:
   config:
     description: A dictionary of Layer-3 interface options
@@ -64,18 +56,37 @@ options:
             - Configures and specifies client-id to use over DHCP ip. Note, This option
               shall work only when dhcp is configured as IP.
             - GigabitEthernet interface number
-            type: int
+            - This option is DEPRECATED and is replaced with dhcp which
+              accepts dict as input this attribute will be removed after 2023-08-01.
+            type: str
           dhcp_hostname:
             description:
             - Configures and specifies value for hostname option over DHCP ip. Note,
               This option shall work only when dhcp is configured as IP.
+            - This option is DEPRECATED and is replaced with dhcp which
+              accepts dict as input this attribute will be removed after 2023-08-01.
+            type: str
+          dhcp:
+            description: IP Address negotiated via DHCP.
+            type: dict
+            suboptions:
+              enable:
+                description: Enable dhcp.
+                type: bool
+              client_id:
+                description: Specify client-id to use.
+                type: str
+              hostname:
+                description: Specify value for hostname option.
+                type: str
+          pool:
+            description: IP Address auto-configured from a local DHCP pool.
             type: str
       ipv6:
         description:
         - IPv6 address to be set for the Layer-3 interface mentioned in I(name) option.
         - The address format is <ipv6 address>/<mask>, the mask is number in range
           0-128 eg. fd5d:12c9:2201:1::1/64
-        - Can be 'dhcp' for DHCP or 'autoconfig' for SLAAC.
         type: list
         elements: dict
         suboptions:
@@ -83,6 +94,51 @@ options:
             description:
             - Configures the IPv6 address for Interface.
             type: str
+          autoconfig:
+            description: Obtain address using auto-configuration.
+            type: dict
+            suboptions:
+              enable:
+                description: enable auto-configuration.
+                type: bool
+              default:
+                description: Insert default route.
+                type: bool
+          dhcp:
+            description: Obtain a ipv6 address using DHCP.
+            type: dict
+            suboptions:
+              enable:
+                description: Enable dhcp.
+                type: bool
+              rapid_commit:
+                description: Enable Rapid-Commit.
+                type: bool
+          anycast:
+            description: Configure as an anycast
+            type: bool
+          cga:
+            description: Use CGA interface identifier
+            type: bool
+          eui:
+            description: Use eui-64 interface identifier
+            type: bool
+          link_local:
+            description: Use link-local address
+            type: bool
+          segment_routing:
+            description: Segment Routing submode
+            type: dict
+            suboptions:
+              enable:
+                description: Enable segmented routing.
+                type: bool
+              default:
+                description: Set a command to its defaults.
+                type: bool
+              ipv6_sr:
+                description: Set ipv6_sr.
+                type: bool
   running_config:
     description:
       - This option is used only with state I(parsed).
@@ -120,16 +176,15 @@ options:
         I(show running-config | section ^interface) executed on device. For state I(parsed) active
         connection to remote host is not required.
     type: str
-
 """
 
 EXAMPLES = """
-# Using merged
-#
+# Using state merged
+
 # Before state:
 # -------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  description Configured by Ansible
 #  ip address 10.1.1.1 255.255.255.0
@@ -164,10 +219,24 @@ EXAMPLES = """
       - address: 192.168.0.3/24
     state: merged
 
+# Commands Fired:
+# ---------------
+
+# "commands": [
+#       "interface GigabitEthernet0/1",
+#       "ip address 192.168.0.1 255.255.255.0 secondary",
+#       "interface GigabitEthernet0/2",
+#       "ip address 192.168.0.2 255.255.255.0",
+#       "interface GigabitEthernet0/3",
+#       "ipv6 address fd5d:12c9:2201:1::1/64",
+#       "GigabitEthernet0/3.100",
+#       "ip address 192.168.0.3 255.255.255.0",
+#     ],
+
 # After state:
 # ------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  description Configured by Ansible
 #  ip address 10.1.1.1 255.255.255.0
@@ -186,12 +255,12 @@ EXAMPLES = """
 #  encapsulation dot1Q 20
 #  ip address 192.168.0.3 255.255.255.0
 
-# Using replaced
-#
+# Using state replaced
+
 # Before state:
 # -------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  description Configured by Ansible
 #  ip address 10.1.1.1 255.255.255.0
@@ -217,19 +286,35 @@ EXAMPLES = """
       - address: 192.168.2.0/24
     - name: GigabitEthernet0/3
       ipv4:
-      - address: dhcp
-        dhcp_client: 2
-        dhcp_hostname: test.com
+      - dhcp:
+          client_id: GigabitEthernet0/2
+          hostname: test.com
     - name: GigabitEthernet0/3.100
       ipv4:
       - address: 192.168.0.3/24
         secondary: true
     state: replaced
 
+# Commands Fired:
+# ---------------
+
+# "commands": [
+#       "interface GigabitEthernet0/1",
+#       "ip address 192.168.0.1 255.255.255.0 secondary",
+#       "interface GigabitEthernet0/2",
+#       "ip address 192.168.0.2 255.255.255.0",
+#       "interface GigabitEthernet0/3",
+#       "no ip address 192.168.2.0 255.255.255.0",
+#       "ip address dhcp client-id GigabitEthernet0/2 hostname test.com",
+#       "GigabitEthernet0/3.100",
+#       "no ip address 192.168.0.2 255.255.255.0",
+#       "ip address 192.168.0.3 255.255.255.0 secondary",
+#     ],
+
 # After state:
 # ------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  description Configured by Ansible
 #  ip address 10.1.1.1 255.255.255.0
@@ -245,15 +330,14 @@ EXAMPLES = """
 #  ip address dhcp client-id GigabitEthernet0/2 hostname test.com
 # interface GigabitEthernet0/3.100
 #  encapsulation dot1Q 20
-#  ip address 192.168.0.2 255.255.255.0
 #  ip address 192.168.0.3 255.255.255.0 secondary
 
-# Using overridden
-#
+# Using state overridden
+
 # Before state:
 # -------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  description Configured by Ansible
 #  ip address 10.1.1.1 255.255.255.0
@@ -279,13 +363,29 @@ EXAMPLES = """
       - address: 192.168.0.1/24
     - name: GigabitEthernet0/3.100
       ipv6:
-      - address: autoconfig
+      - autoconfig: true
     state: overridden
 
+# Commands Fired:
+# ---------------
+
+# "commands": [
+#       "interface GigabitEthernet0/1",
+#       "no ip address 10.1.1.1 255.255.255.0",
+#       "interface GigabitEthernet0/2",
+#       "no ip address 192.168.2.1 255.255.255.0",
+#       "ip address 192.168.0.1 255.255.255.0",
+#       "interface GigabitEthernet0/3",
+#       "no ipv6 address FD5D:12C9:2201:1::1/64",
+#       "GigabitEthernet0/3.100",
+#       "no ip address 192.168.0.2 255.255.255.0",
+#       "ipv6 address autoconfig",
+#     ],
+
 # After state:
 # ------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  description Configured by Ansible
 #  no ip address
@@ -302,12 +402,12 @@ EXAMPLES = """
 #  encapsulation dot1Q 20
 #  ipv6 address autoconfig
 
-# Using Deleted
-#
+# Using state Deleted
+
 # Before state:
 # -------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  ip address 192.0.2.10 255.255.255.0
 #  shutdown
@@ -327,19 +427,26 @@ EXAMPLES = """
 #  encapsulation dot1Q 20
 #  ip address 192.168.0.2 255.255.255.0
 
-- name: "Delete attributes of given interfaces (NOTE: This won't delete the interface sitself)"
+- name: "Delete attributes of given interfaces (NOTE: This won't delete the interfaces itself)"
   cisco.ios.ios_l3_interfaces:
     config:
     - name: GigabitEthernet0/2
     - name: GigabitEthernet0/3.100
     state: deleted
 
+# "commands": [
+#       "interface GigabitEthernet0/2",
+#       "no ip address 192.168.1.0 255.255.255.0",
+#       "GigabitEthernet0/3.100",
+#       "no ip address 192.168.0.2 255.255.255.0",
+#     ],
+
 # After state:
 # -------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
-#  no ip address
+#  ip address 192.0.2.10 255.255.255.0
 #  shutdown
 #  duplex auto
 #  speed auto
@@ -356,14 +463,13 @@ EXAMPLES = """
 # interface GigabitEthernet0/3.100
 #  encapsulation dot1Q 20
 
-# Using Deleted without any config passed
+# Using state Deleted without any config passed
 #"(NOTE: This will delete all of configured L3 resource module attributes from each configured interface)"
 
-#
 # Before state:
 # -------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  ip address 192.0.2.10 255.255.255.0
 #  shutdown
@@ -387,10 +493,22 @@ EXAMPLES = """
   cisco.ios.ios_l3_interfaces:
     state: deleted
 
+# "commands": [
+#       "interface GigabitEthernet0/1",
+#       "no ip address 192.0.2.10 255.255.255.0",
+#       "interface GigabitEthernet0/2",
+#       "no ip address 192.168.1.0 255.255.255.0",
+#       "interface GigabitEthernet0/3",
+#       "no ip address 192.168.0.1 255.255.255.0",
+#       "no ipv6 address FD5D:12C9:2201:1::1/64",
+#       "GigabitEthernet0/3.100",
+#       "no ip address 192.168.0.2 255.255.255.0",
+#     ],
+
 # After state:
 # -------------
-#
-# vios#show running-config | section ^interface
+
+# router-ios#show running-config | section ^interface
 # interface GigabitEthernet0/1
 #  no ip address
 #  shutdown
@@ -407,12 +525,12 @@ EXAMPLES = """
 # interface GigabitEthernet0/3.100
 #  encapsulation dot1Q 20
 
-# Using Gathered
+# Using state Gathered
 
 # Before state:
 # -------------
-#
-# vios#sh running-config | section ^interface
+
+# router-ios#sh running-config | section ^interface
 # interface GigabitEthernet0/1
 #  ip address 203.0.113.27 255.255.255.0
 # interface GigabitEthernet0/2
@@ -422,12 +540,11 @@ EXAMPLES = """
 
 - name: Gather listed l3 interfaces with provided configurations
   cisco.ios.ios_l3_interfaces:
-    config:
     state: gathered
 
 # Module Execution Result:
 # ------------------------
-#
+
 # "gathered": [
 #         {
 #             "ipv4": [
@@ -458,8 +575,8 @@ EXAMPLES = """
 
 # After state:
 # ------------
-#
-# vios#sh running-config | section ^interface
+
+# router-ios#sh running-config | section ^interface
 # interface GigabitEthernet0/1
 #  ip address 203.0.113.27 255.255.255.0
 # interface GigabitEthernet0/2
@@ -467,16 +584,16 @@ EXAMPLES = """
 #  ip address 192.0.2.2 255.255.255.0
 #  ipv6 address 2001:DB8:0:3::/64
 
-# Using Rendered
+# Using state Rendered
 
-- name: Render the commands for provided  configuration
+- name: Render the commands for provided configuration
   cisco.ios.ios_l3_interfaces:
     config:
     - name: GigabitEthernet0/1
       ipv4:
-      - address: dhcp
-        dhcp_client: 0
-        dhcp_hostname: test.com
+      - dhcp:
+          client_id: GigabitEthernet0/0
+          hostname: test.com
     - name: GigabitEthernet0/2
       ipv4:
       - address: 198.51.100.1/24
@@ -488,7 +605,7 @@ EXAMPLES = """
 
 # Module Execution Result:
 # ------------------------
-#
+
 # "rendered": [
 #         "interface GigabitEthernet0/1",
 #         "ip address dhcp client-id GigabitEthernet 0/0 hostname test.com",
@@ -498,18 +615,17 @@ EXAMPLES = """
 #         "ipv6 address 2001:db8:0:3::/64"
 #     ]
 
-# Using Parsed
+# Using state Parsed
 
 # File: parsed.cfg
 # ----------------
 #
 # interface GigabitEthernet0/1
-# ip address dhcp client-id
-# GigabitEthernet 0/0 hostname test.com
+#  ip address dhcp client-id GigabitEthernet 0/0 hostname test.com
 # interface GigabitEthernet0/2
-# ip address 198.51.100.1 255.255.255.0
-# secondary ip address 198.51.100.2 255.255.255.0
-# ipv6 address 2001:db8:0:3::/64
+#  ip address 198.51.100.1 255.255.255.0
+#  ip address 198.51.100.2 255.255.255.0 secondary
+#  ipv6 address 2001:db8:0:3::/64
 
 - name: Parse the commands for provided configuration
   cisco.ios.ios_l3_interfaces:
@@ -518,14 +634,15 @@ EXAMPLES = """
 
 # Module Execution Result:
 # ------------------------
-#
+
 # "parsed": [
 #         {
 #             "ipv4": [
 #                 {
-#                     "address": "dhcp",
-#                     "dhcp_client": 0,
-#                     "dhcp_hostname": "test.com"
+#                     "dhcp": {
+#                         "client_id": GigabitEthernet0/0,
+#                         "hostname": "test.com"
+#                     }
 #                 }
 #             ],
 #             "name": "GigabitEthernet0/1"
@@ -533,11 +650,11 @@ EXAMPLES = """
 #         {
 #             "ipv4": [
 #                 {
-#                     "address": "198.51.100.1 255.255.255.0",
+#                     "address": "198.51.100.1/24",
 #                     "secondary": true
 #                 },
 #                 {
-#                     "address": "198.51.100.2 255.255.255.0"
+#                     "address": "198.51.100.2/24"
 #                 }
 #             ],
 #             "ipv6": [
@@ -553,52 +670,80 @@ EXAMPLES = """
 
 RETURN = """
 before:
-  description: The configuration as structured data prior to module invocation.
-  returned: always
-  type: list
-  sample: The configuration returned will always be in the same format of the parameters above.
+  description: The configuration prior to the module execution.
+  returned: when state is I(merged), I(replaced), I(overridden), I(deleted) or I(purged)
+  type: dict
+  sample: >
+    This output will always be in the same format as the
+    module argspec.
 after:
-  description: The configuration as structured data after module completion.
+  description: The resulting configuration after module execution.
   returned: when changed
-  type: list
-  sample: The configuration returned will always be in the same format of the parameters above.
+  type: dict
+  sample: >
+    This output will always be in the same format as the
+    module argspec.
 commands:
-  description: The set of commands pushed to the remote device
-  returned: always
+  description: The set of commands pushed to the remote device.
+  returned: when state is I(merged), I(replaced), I(overridden), I(deleted) or I(purged)
   type: list
-  sample: ['interface GigabitEthernet0/1', 'ip address 192.168.0.2 255.255.255.0']
+  sample:
+    - "ip address 192.168.0.3 255.255.255.0"
+    - "ipv6 address dhcp rapid-commit"
+    - "ipv6 address fd5d:12c9:2201:1::1/64 anycast"
+rendered:
+  description: The provided configuration in the task rendered in device-native format (offline).
+  returned: when state is I(rendered)
+  type: list
+  sample:
+    - "ipv6 address FD5D:12C9:2201:1::1/64"
+    - "ip address 192.168.0.3 255.255.255.0"
+    - "ip address autoconfig"
+gathered:
+  description: Facts about the network resource gathered from the remote device as structured data.
+  returned: when state is I(gathered)
+  type: list
+  sample: >
+    This output will always be in the same format as the
+    module argspec.
+parsed:
+  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
+  returned: when state is I(parsed)
+  type: list
+  sample: >
+    This output will always be in the same format as the
+    module argspec.
 """
 
 from ansible.module_utils.basic import AnsibleModule
 from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.l3_interfaces.l3_interfaces import (
-    L3_InterfacesArgs,
+    L3_interfacesArgs,
 )
 from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.l3_interfaces.l3_interfaces import (
-    L3_Interfaces,
+    L3_interfaces,
 )
 
 
 def main():
     """
     Main entry point for module execution
+
     :returns: the result form module invocation
     """
-    required_if = [
-        ("state", "merged", ("config",)),
-        ("state", "replaced", ("config",)),
-        ("state", "overridden", ("config",)),
-        ("state", "rendered", ("config",)),
-        ("state", "parsed", ("running_config",)),
-    ]
-    mutually_exclusive = [("config", "running_config")]
-
     module = AnsibleModule(
-        argument_spec=L3_InterfacesArgs.argument_spec,
-        required_if=required_if,
-        mutually_exclusive=mutually_exclusive,
+        argument_spec=L3_interfacesArgs.argument_spec,
+        mutually_exclusive=[["config", "running_config"]],
+        required_if=[
+            ["state", "merged", ["config"]],
+            ["state", "replaced", ["config"]],
+            ["state", "overridden", ["config"]],
+            ["state", "rendered", ["config"]],
+            ["state", "parsed", ["running_config"]],
+        ],
         supports_check_mode=True,
     )
-    result = L3_Interfaces(module).execute_module()
+
+    result = L3_interfaces(module).execute_module()
     module.exit_json(**result)
 
 
