@@ -21,7 +21,7 @@ author: Sagar Paul (@KB-perByte)
 notes:
   - Tested against Cisco IOSv Version 15.6
   - This module works with connection C(network_cli).
-    See L(IOS Platform Options,../network/user_guide/platform_ios.html)
+    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
   - The Configuration defaults of the Cisco IOS network devices
     are supposed to hinder idempotent behavior of plays
 options:
@@ -206,7 +206,7 @@ options:
                   stream: *stream
                   filtered: *filtered
                   port:
-                    description: Specify the TCP port number (default=601) (1 - 65535)
+                    description: Specify the UDP port number (default=514) (1 - 65535)
                     type: int
                   sequence_num_session: *sequence_num_session
                   session_id: *session_id
@@ -485,70 +485,6 @@ EXAMPLES = """
 # Before state:
 # -------------
 
-# router-ios#show running-config | section logging
-# logging exception 4099
-# logging message-counter log
-# logging userinfo
-# logging buffered xml 5099 notifications
-# no logging reload
-# no logging rate-limit
-# logging console xml critical
-# logging monitor warnings
-# logging cns-events warnings
-# logging policy-firewall rate-limit 10
-# logging dmvpn rate-limit 10
-# logging trap errors
-# logging facility local5
-# logging snmp-trap errors
-# logging snmp-trap warnings
-# logging host 172.16.1.13 transport tcp port 514
-# logging host 172.16.1.11 xml
-# logging host 172.16.1.12
-# logging host 172.16.1.10 filtered stream 10
-
-- name: Remove as per the provided configuration
-  cisco.ios.ios_logging_global:
-    config:
-      snmp_trap:
-        - errors
-        - warnings
-      hosts:
-        - hostname: 172.16.1.11
-    state: deleted
-
-# Commands Fired:
-# ---------------
-
-# "commands": [
-#         "no logging snmp-trap errors",
-#         "no logging snmp-trap warnings",
-#         "no logging host 172.16.1.11"
-#     ],
-
-# After state:
-# ------------
-
-# router-ios#show running-config | section logging
-# logging exception 4099
-# logging message-counter log
-# logging userinfo
-# logging buffered xml 5099 notifications
-# no logging reload
-# no logging rate-limit
-# logging console xml critical
-# logging monitor warnings
-# logging cns-events warnings
-# logging policy-firewall rate-limit 10
-# logging dmvpn rate-limit 10
-# logging trap errors
-# logging facility local5
-# logging host 172.16.1.13 transport tcp port 514
-# logging host 172.16.1.12
-# logging host 172.16.1.10 filtered stream 10
-
-# Before state:
-# -------------
-
 # router-ios#show running-config | section logging
 # logging exception 4099
 # logging message-counter log
