@@ -5,10 +5,11 @@
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 from __future__ import absolute_import, division, print_function
+
 __metaclass__ = type
 
 
-DOCUMENTATION = '''
+DOCUMENTATION = """
 ---
 module: digital_ocean_firewall
 short_description: Manage cloud firewalls within DigitalOcean
@@ -145,9 +146,9 @@ options:
                 which the Firewall will allow traffic
             required: false
 extends_documentation_fragment: digital_ocean.documentation
-'''
+"""
 
-EXAMPLES = '''
+EXAMPLES = """
 # Allows tcp connections to port 22 (SSH) from specific sources
 # Allows tcp connections to ports 80 and 443 from any source
 # Allows outbound access to any destination for protocols tcp, udp and icmp
@@ -187,9 +188,9 @@ EXAMPLES = '''
           addresses: ["0.0.0.0/0", "::/0"]
     droplet_ids: []
     tags: ["sample"]
-'''
+"""
 
-RETURN = '''
+RETURN = """
 data:
     description: DigitalOcean firewall resource
     returned: success
@@ -278,30 +279,32 @@ data:
         "status": "succeeded",
         "tags": ["sample"]
     }
-'''
+"""
 
 from traceback import format_exc
 from ansible.module_utils.basic import AnsibleModule
-from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import DigitalOceanHelper
+from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
+    DigitalOceanHelper,
+)
 from ansible.module_utils._text import to_native
 
 address_spec = dict(
-    addresses=dict(type='list', elements='str', required=False),
-    droplet_ids=dict(type='list', elements='str', required=False),
-    load_balancer_uids=dict(type='list', elements='str', required=False),
-    tags=dict(type='list', elements='str', required=False),
+    addresses=dict(type="list", elements="str", required=False),
+    droplet_ids=dict(type="list", elements="str", required=False),
+    load_balancer_uids=dict(type="list", elements="str", required=False),
+    tags=dict(type="list", elements="str", required=False),
 )
 
 inbound_spec = dict(
-    protocol=dict(type='str', choices=['udp', 'tcp', 'icmp'], default='tcp'),
-    ports=dict(type='str', required=True),
-    sources=dict(type='dict', required=True, options=address_spec),
+    protocol=dict(type="str", choices=["udp", "tcp", "icmp"], default="tcp"),
+    ports=dict(type="str", required=True),
+    sources=dict(type="dict", required=True, options=address_spec),
 )
 
 outbound_spec = dict(
-    protocol=dict(type='str', choices=['udp', 'tcp', 'icmp'], default='tcp'),
-    ports=dict(type='str', required=True),
-    destinations=dict(type='dict', required=True, options=address_spec),
+    protocol=dict(type="str", choices=["udp", "tcp", "icmp"], default="tcp"),
+    ports=dict(type="str", required=True),
+    destinations=dict(type="dict", required=True, options=address_spec),
 )
 
 
@@ -309,22 +312,50 @@ class DOFirewall(object):
     def __init__(self, module):
         self.rest = DigitalOceanHelper(module)
         self.module = module
-        self.name = self.module.params.get('name')
-        self.baseurl = 'firewalls'
+        self.name = self.module.params.get("name")
+        self.baseurl = "firewalls"
         self.firewalls = self.get_firewalls()
 
     def get_firewalls(self):
         base_url = self.baseurl + "?"
         response = self.rest.get("%s" % base_url)
         status_code = response.status_code
-        if status_code != 200:
-            self.module.fail_json(msg="Failed to retrieve firewalls from DigitalOcean")
-        return self.rest.get_paginated_data(base_url=base_url, data_key_name='firewalls')
+        status_code_success = 200
+
+        if status_code != status_code_success:
+            error = response.json
+            info = response.info
+
+            if error:
+                error.update({"status_code": status_code})
+                error.update({"status_code_success": status_code_success})
+                self.module.fail_json(msg=error)
+            elif info:
+                info.update({"status_code_success": status_code_success})
+                self.module.fail_json(msg=info)
+            else:
+                msg_error = "Failed to retrieve firewalls from DigitalOcean"
+                self.module.fail_json(
+                    msg=msg_error
+                    + " (url="
+                    + self.rest.baseurl
+                    + "/"
+                    + self.baseurl
+                    + ", status="
+                    + str(status_code or "")
+                    + " - expected:"
+                    + str(status_code_success)
+                    + ")"
+                )
+
+        return self.rest.get_paginated_data(
+            base_url=base_url, data_key_name="firewalls"
+        )
 
     def get_firewall_by_name(self):
         rule = {}
         for firewall in self.firewalls:
-            if firewall['name'] == self.name:
+            if firewall["name"] == self.name:
                 rule.update(firewall)
                 return rule
         return None
@@ -338,37 +369,27 @@ class DOFirewall(object):
             return obj
 
     def fill_protocol_defaults(self, obj):
-        if obj.get('protocol') is None:
-            obj['protocol'] = 'tcp'
+        if obj.get("protocol") is None:
+            obj["protocol"] = "tcp"
 
         return obj
 
     def fill_source_and_destination_defaults_inner(self, obj):
-        addresses = obj.get('addresses')
+        addresses = obj.get("addresses") or []
 
-        if addresses is None:
-            addresses = []
+        droplet_ids = obj.get("droplet_ids") or []
+        droplet_ids = [str(droplet_id) for droplet_id in droplet_ids]
 
-        droplet_ids = obj.get('droplet_ids')
+        load_balancer_uids = obj.get("load_balancer_uids") or []
+        load_balancer_uids = [str(uid) for uid in load_balancer_uids]
 
-        if droplet_ids is None:
-            droplet_ids = []
-
-        load_balancer_uids = obj.get('load_balancer_uids')
-
-        if load_balancer_uids is None:
-            load_balancer_uids = []
-
-        tags = obj.get('tags')
-
-        if tags is None:
-            tags = []
+        tags = obj.get("tags") or []
 
         data = {
             "addresses": addresses,
             "droplet_ids": droplet_ids,
             "load_balancer_uids": load_balancer_uids,
-            "tags": tags
+            "tags": tags,
         }
 
         return data
@@ -386,38 +407,39 @@ class DOFirewall(object):
         return obj
 
     def fill_data_defaults(self, obj):
-        inbound_rules = obj.get('inbound_rules')
+        inbound_rules = obj.get("inbound_rules")
 
         if inbound_rules is None:
             inbound_rules = []
         else:
             inbound_rules = [self.fill_protocol_defaults(x) for x in inbound_rules]
-            inbound_rules = [self.fill_sources_and_destinations_defaults(x, 'sources') for x in inbound_rules]
+            inbound_rules = [
+                self.fill_sources_and_destinations_defaults(x, "sources")
+                for x in inbound_rules
+            ]
 
-        outbound_rules = obj.get('outbound_rules')
+        outbound_rules = obj.get("outbound_rules")
 
         if outbound_rules is None:
             outbound_rules = []
         else:
             outbound_rules = [self.fill_protocol_defaults(x) for x in outbound_rules]
-            outbound_rules = [self.fill_sources_and_destinations_defaults(x, 'destinations') for x in outbound_rules]
+            outbound_rules = [
+                self.fill_sources_and_destinations_defaults(x, "destinations")
+                for x in outbound_rules
+            ]
 
-        droplet_ids = obj.get('droplet_ids')
+        droplet_ids = obj.get("droplet_ids") or []
+        droplet_ids = [str(droplet_id) for droplet_id in droplet_ids]
 
-        if droplet_ids is None:
-            droplet_ids = []
-
-        tags = obj.get('tags')
-
-        if tags is None:
-            tags = []
+        tags = obj.get("tags") or []
 
         data = {
-            "name": obj.get('name'),
+            "name": obj.get("name"),
             "inbound_rules": inbound_rules,
             "outbound_rules": outbound_rules,
             "droplet_ids": droplet_ids,
-            "tags": tags
+            "tags": tags,
         }
 
         return data
@@ -431,80 +453,96 @@ class DOFirewall(object):
             resp = self.rest.post(path=self.baseurl, data=obj)
         else:
             status_code_success = 200
-            resp = self.rest.put(path=self.baseurl + '/' + id, data=obj)
+            resp = self.rest.put(path=self.baseurl + "/" + id, data=obj)
         status_code = resp.status_code
         if status_code != status_code_success:
             error = resp.json
-            error.update({'status_code': status_code})
-            error.update({'status_code_success': status_code_success})
+            error.update(
+                {
+                    "context": "error when trying to "
+                    + ("create" if (id is None) else "update")
+                    + " firewalls"
+                }
+            )
+            error.update({"status_code": status_code})
+            error.update({"status_code_success": status_code_success})
             self.module.fail_json(msg=error)
-        self.module.exit_json(changed=True, data=resp.json['firewall'])
+        self.module.exit_json(changed=True, data=resp.json["firewall"])
 
     def create(self):
         rule = self.get_firewall_by_name()
         data = {
-            "name": self.module.params.get('name'),
-            "inbound_rules": self.module.params.get('inbound_rules'),
-            "outbound_rules": self.module.params.get('outbound_rules'),
-            "droplet_ids": self.module.params.get('droplet_ids'),
-            "tags": self.module.params.get('tags')
+            "name": self.module.params.get("name"),
+            "inbound_rules": self.module.params.get("inbound_rules"),
+            "outbound_rules": self.module.params.get("outbound_rules"),
+            "droplet_ids": self.module.params.get("droplet_ids"),
+            "tags": self.module.params.get("tags"),
         }
         if rule is None:
             self.update(data, None)
         else:
             rule_data = {
-                "name": rule.get('name'),
-                "inbound_rules": rule.get('inbound_rules'),
-                "outbound_rules": rule.get('outbound_rules'),
-                "droplet_ids": rule.get('droplet_ids'),
-                "tags": rule.get('tags')
+                "name": rule.get("name"),
+                "inbound_rules": rule.get("inbound_rules"),
+                "outbound_rules": rule.get("outbound_rules"),
+                "droplet_ids": rule.get("droplet_ids"),
+                "tags": rule.get("tags"),
             }
 
             user_data = {
-                "name": data.get('name'),
-                "inbound_rules": data.get('inbound_rules'),
-                "outbound_rules": data.get('outbound_rules'),
-                "droplet_ids": data.get('droplet_ids'),
-                "tags": data.get('tags')
+                "name": data.get("name"),
+                "inbound_rules": data.get("inbound_rules"),
+                "outbound_rules": data.get("outbound_rules"),
+                "droplet_ids": data.get("droplet_ids"),
+                "tags": data.get("tags"),
             }
 
             if self.data_to_compare(user_data) == self.data_to_compare(rule_data):
                 self.module.exit_json(changed=False, data=rule)
             else:
-                self.update(data, rule.get('id'))
+                self.update(data, rule.get("id"))
 
     def destroy(self):
         rule = self.get_firewall_by_name()
         if rule is None:
             self.module.exit_json(changed=False, data="Firewall does not exist")
         else:
-            endpoint = self.baseurl + '/' + rule['id']
+            endpoint = self.baseurl + "/" + rule["id"]
             resp = self.rest.delete(path=endpoint)
             status_code = resp.status_code
             if status_code != 204:
                 self.module.fail_json(msg="Failed to delete firewall")
-            self.module.exit_json(changed=True, data="Deleted firewall rule: {0} - {1}".format(rule['name'], rule['id']))
+            self.module.exit_json(
+                changed=True,
+                data="Deleted firewall rule: {0} - {1}".format(
+                    rule["name"], rule["id"]
+                ),
+            )
 
 
 def core(module):
-    state = module.params.get('state')
+    state = module.params.get("state")
     firewall = DOFirewall(module)
 
-    if state == 'present':
+    if state == "present":
         firewall.create()
-    elif state == 'absent':
+    elif state == "absent":
         firewall.destroy()
 
 
 def main():
     argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
     argument_spec.update(
-        name=dict(type='str', required=True),
-        state=dict(type='str', choices=['present', 'absent'], default='present'),
-        droplet_ids=dict(type='list', elements='str', required=False),
-        tags=dict(type='list', elements='str', required=False),
-        inbound_rules=dict(type='list', elements='dict', options=inbound_spec, required=True),
-        outbound_rules=dict(type='list', elements='dict', options=outbound_spec, required=True),
+        name=dict(type="str", required=True),
+        state=dict(type="str", choices=["present", "absent"], default="present"),
+        droplet_ids=dict(type="list", elements="str", required=False),
+        tags=dict(type="list", elements="str", required=False),
+        inbound_rules=dict(
+            type="list", elements="dict", options=inbound_spec, required=True
+        ),
+        outbound_rules=dict(
+            type="list", elements="dict", options=outbound_spec, required=True
+        ),
     )
     module = AnsibleModule(argument_spec=argument_spec)
 
@@ -514,5 +552,5 @@ def main():
         module.fail_json(msg=to_native(e), exception=format_exc())
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
