@@ -340,10 +340,7 @@ def grafana_create_dashboard(module, data):
     if grafana_version >= 5:
         folder_exists, folder_id = grafana_folder_exists(module, data['grafana_url'], data['folder'], headers)
         if folder_exists is False:
-            result['msg'] = "Dashboard folder '%s' does not exist." % data['folder']
-            result['uid'] = uid
-            result['changed'] = False
-            return result
+            raise GrafanaAPIException("Dashboard folder '%s' does not exist." % data['folder'])
 
         payload['folderId'] = folder_id
 
@@ -378,17 +375,13 @@ def grafana_create_dashboard(module, data):
             else:
                 body = json.loads(info['body'])
                 raise GrafanaAPIException('Unable to update the dashboard %s : %s (HTTP: %d)' %
-                                          (uid, body['commit_message'], info['status']))
+                                          (uid, body['message'], info['status']))
         else:
             # unchanged
             result['uid'] = uid
             result['msg'] = "Dashboard %s unchanged." % payload['dashboard']['title']
             result['changed'] = False
     else:
-        # create
-        if folder_exists is True:
-            payload['folderId'] = folder_id
-
         # Ensure there is no id in payload
         if 'id' in payload['dashboard']:
             del payload['dashboard']['id']
