@@ -291,6 +291,66 @@ options:
     default: False
     aliases:
       - restart
+  generate_systemd:
+    description:
+      - Generate systemd unit file for container.
+    type: dict
+    default: {}
+    suboptions:
+      path:
+        description:
+          - Specify a path to the directory where unit files will be generated.
+            If it doesn't exist, the directory will be created.
+        type: str
+        required: false
+      restart_policy:
+        description:
+          - Specify a restart policy for the service.  The restart-policy must be one of
+            "no", "on-success", "on-failure", "on-abnormal", "on-watchdog", "on-abort", or "always".
+            The default policy is "on-failure".
+        type: str
+        required: false
+        choices:
+            - 'no'
+            - 'on-success'
+            - 'on-failure'
+            - 'on-abnormal'
+            - 'on-watchdog'
+            - 'on-abort'
+            - 'always'
+      time:
+        description:
+          - Override the default stop timeout for the container with the given value.
+        type: int
+        required: false
+      no_header:
+        description:
+          - Do not generate the header including meta data such as the Podman version and the timestamp.
+            From podman version 3.1.0.
+        type: bool
+        default: false
+      names:
+        description:
+          - Use names of the containers for the start, stop, and description in the unit file.
+            Default is true.
+        type: bool
+        default: true
+      container_prefix:
+        description:
+          - Set the systemd unit name prefix for containers. The default is "container".
+        type: str
+        required: false
+      pod_prefix:
+        description:
+          - Set the systemd unit name prefix for pods. The default is "pod".
+        type: str
+        required: false
+      separator:
+        description:
+          - Set the systemd unit name separator between the name/id of a
+            container/pod and the prefix. The default is "-" (dash).
+        type: str
+        required: false
   gidmap:
     description:
       - Run the container in a new user namespace using the supplied mapping.
@@ -595,6 +655,11 @@ options:
       - If true, the first argument refers to an exploded container on the file
         system. The default is false.
     type: bool
+  secrets:
+    description:
+      - Add the named secrets into the container at /run/secrets.
+    type: list
+    elements: str
   security_opt:
     description:
       - Security Options. For example security_opt "seccomp=unconfined"
@@ -724,7 +789,7 @@ EXAMPLES = r"""
     volume:
       - /tmp/data
 
-- name: Re-create a redis container
+- name: Re-create a redis container with systemd service file generated in /tmp/
   containers.podman.podman_container:
     name: myredis
     image: redis
@@ -735,6 +800,12 @@ EXAMPLES = r"""
       - 6379
     volumes_from:
       - mydata
+    generate_systemd:
+      path: /tmp/
+      restart_policy: always
+      time: 120
+      names: true
+      container_prefix: ainer
 
 - name: Restart a container
   containers.podman.podman_container:
@@ -921,7 +992,7 @@ def main():
     )
 
     # work on input vars
-    if (module.params['state'] in ['started', 'present', 'created']
+    if (module.params['state'] in ['present', 'created']
             and not module.params['force_restart']
             and not module.params['image']):
         module.fail_json(msg="State '%s' required image to be configured!" %
