@@ -12,7 +12,7 @@ DOCUMENTATION = r'''
 module: bigip_asm_policy_manage
 short_description: Manage BIG-IP ASM policies
 description:
-   - Manage BIG-IP ASM policies, create from templates, and manage global policy settings.
+   - Manage BIG-IP ASM policies, create policies from templates, and manage global policy settings.
 version_added: "1.0.0"
 options:
   active:
@@ -26,7 +26,7 @@ options:
   apply:
     description:
       - If C(yes) will apply the policy if the policy has pending changes.
-      - Parameter supported on TMOS C(v14.x) and above.
+      - This parameter supported on TMOS C(v14.x) and above.
     type: bool
     version_added: "1.4.0"
   name:
@@ -158,12 +158,12 @@ EXAMPLES = r'''
 
 RETURN = r'''
 active:
-  description: Set when activating/deactivating an ASM policy
+  description: Set when activating/deactivating an ASM policy.
   returned: changed
   type: bool
   sample: yes
 apply:
-  description: Set when applying pending changes to an ASM policy
+  description: Set when applying pending changes to an ASM policy.
   returned: changed when target policy has changes pending
   type: bool
   sample: yes
@@ -611,7 +611,7 @@ class BaseManager(object):
             self.client.provider['server_port'],
         )
 
-        query = "?$filter=name+eq+{0}+and+partition+eq+{1}&$select=name,partition".format(
+        query = "?$filter=contains(name,'{0}')+and+contains(partition,'{1}')&$select=name,partition".format(
             self.want.name, self.want.partition
         )
         resp = self.client.api.get(uri + query)
@@ -628,9 +628,11 @@ class BaseManager(object):
                 raise F5ModuleError(response['message'])
             else:
                 raise F5ModuleError(resp.content)
-
         if 'items' in response and response['items'] != []:
-            return True
+            # because api filter on ASM is broken when names that contain numbers at the end we need to work around it
+            for policy in response['items']:
+                if policy['name'] == self.want.name and policy['partition'] == self.want.partition:
+                    return True
         return False
 
     def wait_for_policy_apply(self):
@@ -682,7 +684,7 @@ class BaseManager(object):
             self.client.provider['server'],
             self.client.provider['server_port'],
         )
-        query = "?$filter=name+eq+{0}+and+partition+eq+{1}&$select=name,id".format(
+        query = "?$filter=contains(name,'{0}')+and+contains(partition,'{1}')&$select=name,id".format(
             self.want.name, self.want.partition
         )
         resp = self.client.api.get(uri + query)
@@ -695,16 +697,18 @@ class BaseManager(object):
             raise F5ModuleError(resp.content)
 
         if 'items' in response and response['items'] != []:
+            # because api filter on ASM is broken when names that contain numbers at the end we need to work around it
             if len(response['items']) == 1:
                 policy_id = response['items'][0]['id']
             else:
                 for item in response['items']:
-                    if item['name'] == self.want.name:
+                    if item['name'] == self.want.name and item['partition'] == self.want.partition:
                         policy_id = item['id']
 
         if not policy_id:
-            raise F5ModuleError("The policy was not found")
-
+            raise F5ModuleError(
+                "The policy with the name {0} was not found.".format(self.want.name)
+            )
         return policy_id
 
     def update_on_device(self):
