@@ -21,7 +21,7 @@ version_added: '21.4.0'
 author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
 
 description:
-- Create or delete Cloud Manager CVO/working environment in single or HA mode for Azure.
+- Create, delete, or manage Cloud Manager CVO/working environment in single or HA mode for Azure.
 
 options:
 
@@ -57,13 +57,32 @@ options:
   license_type:
     description:
     - The type of license to use.
-    - For single node ['azure-cot-explore-paygo', 'azure-cot-standard-paygo', 'azure-cot-premium-paygo', 'azure-cot-premium-byol']
-    - For HA ['azure-ha-cot-standard-paygo', 'azure-ha-cot-premium-paygo', 'azure-ha-cot-premium-byol']
-    choices: ['azure-cot-standard-paygo', 'azure-cot-premium-paygo', 'azure-cot-premium-byol', 'azure-cot-explore-paygo',
-    'azure-ha-cot-standard-paygo', 'azure-ha-cot-premium-paygo', 'azure-ha-cot-premium-byol']
+    - For single node ['azure-cot-explore-paygo', 'azure-cot-standard-paygo', 'azure-cot-premium-paygo', \
+     'azure-cot-premium-byol', 'capacity-paygo']
+    - For HA ['azure-ha-cot-standard-paygo', 'azure-ha-cot-premium-paygo', 'azure-ha-cot-premium-byol', \
+     'ha-capacity-paygo']
+    - Use capacity-paygo or ha-capacity-paygo for HA on selecting Bring Your Own License type Capacity-Based or Freemium.
+    - Use azure-cot-premium-boyl or azure-ha-cot-permium-boyl for HA on selecting Bring Your Own License type Node-Based.
+    choices: ['azure-cot-standard-paygo', 'azure-cot-premium-paygo', 'azure-cot-premium-byol', \
+     'azure-cot-explore-paygo', 'azure-ha-cot-standard-paygo', 'azure-ha-cot-premium-paygo', \
+    'azure-ha-cot-premium-byol', 'capacity-paygo', 'ha-capacity-paygo']
     default: 'azure-cot-standard-paygo'
     type: str
 
+  provided_license:
+    description:
+    - Using a NLF license file for BYOL deployment.
+    type: str
+
+  capacity_package_name:
+    description:
+    - Capacity package name is required when selecting a capacity based license.
+    - Essential only available with Bring Your Own License Capacity-Based.
+    - Professional available as an annual contract from a cloud provider or Bring Your Own License Capacity-Based.
+    choices: ['Professional', 'Essential', 'Freemium']
+    type: str
+    version_added: 21.12.0
+
   workspace_id:
     description:
     - The ID of the Cloud Manager workspace where you want to deploy Cloud Volumes ONTAP.
@@ -120,6 +139,12 @@ options:
     default: 'AZURE'
     type: str
 
+  azure_encryption_parameters:
+    description:
+    - AZURE encryption parameters. It is required if using AZURE encryption.
+    type: str
+    version_added: 21.10.0
+
   storage_type:
     description:
     - The type of storage for the first data aggregate.
@@ -158,6 +183,7 @@ options:
     required: true
     description:
     - The admin password for Cloud Volumes ONTAP.
+    - It will be updated on each run.
     type: str
 
   ontap_version:
@@ -258,11 +284,11 @@ options:
     - For HA BYOL, the serial number for the second node.
     type: str
 
-  refresh_token:
-    required: true
+  ha_enable_https:
     description:
-    - The refresh token for NetApp Cloud Manager API operations.
-    type: str
+    - For HA, enable the HTTPS connection from CVO to storage accounts. This can impact write performance. The default is false.
+    type: bool
+    version_added: 21.10.0
 
 '''
 
@@ -271,7 +297,7 @@ EXAMPLES = """
   netapp.cloudmanager.na_cloudmanager_cvo_azure:
     state: present
     refresh_token: "{{ xxxxxxxxxxxxxxx }}"
-    name: TerraformCVO
+    name: AnsibleCVO
     location: westus
     subnet_id: subnet-xxxxxxx
     vnet_id: vnetxxxxxxxx
@@ -286,7 +312,7 @@ EXAMPLES = """
   netapp.cloudmanager.na_cloudmanager_cvo_azure:
     state: present
     refresh_token: "{{ xxxxxxxxxxxxxxx }}"
-    name: TerraformCVO
+    name: AnsibleCVO
     location: westus
     subnet_id: subnet-xxxxxxx
     vnet_id: vnetxxxxxxxx
@@ -302,6 +328,7 @@ EXAMPLES = """
   netapp.cloudmanager.na_cloudmanager_cvo_azure:
     state: absent
     name: ansible
+    refresh_token: "{{ xxxxxxxxxxxxxxx }}"
     location: westus
     subnet_id: subnet-xxxxxxx
     vnet_id: vnetxxxxxxxx
@@ -323,13 +350,12 @@ from ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp import
 
 IMPORT_EXCEPTION = None
 
-Cloud_Manager_Host = "cloudmanager.cloud.netapp.com"
 AZURE_License_Types = ['azure-cot-standard-paygo', 'azure-cot-premium-paygo', 'azure-cot-premium-byol', 'azure-cot-explore-paygo',
-                       'azure-ha-cot-standard-paygo', 'azure-ha-cot-premium-paygo', 'azure-ha-cot-premium-byol']
+                       'azure-ha-cot-standard-paygo', 'azure-ha-cot-premium-paygo', 'azure-ha-cot-premium-byol', 'capacity-paygo', 'ha-capacity-paygo']
 
 
 class NetAppCloudManagerCVOAZURE:
-    ''' object initialize and class methods '''
+    """ object initialize and class methods """
 
     def __init__(self):
         self.use_rest = False
@@ -340,6 +366,8 @@ class NetAppCloudManagerCVOAZURE:
             instance_type=dict(required=False, type='str', default='Standard_DS4_v2'),
             license_type=dict(required=False, type='str', choices=AZURE_License_Types, default='azure-cot-standard-paygo'),
             workspace_id=dict(required=False, type='str'),
+            capacity_package_name=dict(required=False, type='str', choices=['Professional', 'Essential', 'Freemium']),
+            provided_license=dict(required=False, type='str'),
             subnet_id=dict(required=True, type='str'),
             vnet_id=dict(required=True, type='str'),
             vnet_resource_group=dict(required=False, type='str'),
@@ -348,6 +376,7 @@ class NetAppCloudManagerCVOAZURE:
             location=dict(required=True, type='str'),
             subscription_id=dict(required=True, type='str'),
             data_encryption_type=dict(required=False, type='str', choices=['AZURE', 'NONE'], default='AZURE'),
+            azure_encryption_parameters=dict(required=False, type='str', no_log=True),
             storage_type=dict(required=False, type='str', choices=['Premium_LRS', 'Standard_LRS', 'StandardSSD_LRS'], default='Premium_LRS'),
             disk_size=dict(required=False, type='int', default=1),
             disk_size_unit=dict(required=False, type='str', choices=['GB', 'TB'], default='TB'),
@@ -373,85 +402,55 @@ class NetAppCloudManagerCVOAZURE:
             is_ha=dict(required=False, type='bool', default=False),
             platform_serial_number_node1=dict(required=False, type='str'),
             platform_serial_number_node2=dict(required=False, type='str'),
+            ha_enable_https=dict(required=False, type='bool'),
         ))
 
         self.module = AnsibleModule(
             argument_spec=self.argument_spec,
+            required_one_of=[['refresh_token', 'sa_client_id']],
+            required_together=[['sa_client_id', 'sa_secret_key']],
+            required_if=[
+                ['license_type', 'capacity-paygo', ['capacity_package_name']],
+                ['license_type', 'ha-capacity-paygo', ['capacity_package_name']],
+            ],
             supports_check_mode=True
         )
 
         self.na_helper = NetAppModule()
         self.parameters = self.na_helper.set_parameters(self.module.params)
-
+        self.changeable_params = ['svm_password', 'azure_tag', 'tier_level']
         self.rest_api = CloudManagerRestAPI(self.module)
+        self.rest_api.url += self.rest_api.environment_data['CLOUD_MANAGER_HOST']
+        self.rest_api.api_root_path = '/occm/api/azure/%s' % ('ha' if self.parameters['is_ha'] else 'vsa')
         self.headers = {
-            'X-Agent-Id': self.parameters['client_id'] + "clients"
+            'X-Agent-Id': self.rest_api.format_cliend_id(self.parameters['client_id'])
         }
 
-    def get_working_environment(self):
-        """
-        Get working environment details including:
-        name: working environment name
-        """
-
-        api_url = '%s/occm/api/working-environments' % Cloud_Manager_Host
-        response, error, dummy = self.rest_api.get(api_url, header=self.headers)
-        if error is not None:
-            self.module.fail_json(
-                msg="Error: unexpected response on getting azure cvo: %s, %s" % (str(error), str(response)))
-
-        for each in response['azureVsaWorkingEnvironments']:
-            if each['name'] == self.parameters['name']:
-                return each
-
-        return None
-
-    def get_tenant(self):
-        """
-        Get workspace ID (tenant)
-        """
-
-        api_url = '%s/occm/api/tenants' % Cloud_Manager_Host
-        response, error, dummy = self.rest_api.get(api_url, header=self.headers)
-        if error is not None:
-            self.module.fail_json(
-                msg="Error: unexpected response on getting tenant for azure cvo: %s, %s" % (str(error), str(response)))
-
-        return response[0]['publicId']
-
-    def get_nss(self):
-        """
-        Get nss account
-        """
-
-        api_url = '%s/occm/api/accounts' % Cloud_Manager_Host
-        response, error, dummy = self.rest_api.get(api_url, header=self.headers)
-        if error is not None:
-            self.module.fail_json(
-                msg="Error: unexpected response on getting nss for azure cvo: %s, %s" % (str(error), str(response)))
-
-        if len(response['nssAccounts']) == 0:
-            self.module.fail_json(msg="could not find any NSS account")
-
-        return response['nssAccounts'][0]['publicId']
-
     def create_cvo_azure(self):
         """
         Create AZURE CVO
         """
-
         if self.parameters.get('workspace_id') is None:
-            self.parameters['workspace_id'] = self.get_tenant()
+            response, msg = self.na_helper.get_tenant(self.rest_api, self.headers)
+            if response is None:
+                self.module.fail_json(msg)
+            self.parameters['workspace_id'] = response
 
         if self.parameters.get('nss_account') is None:
             if self.parameters.get('serial_number') is not None:
                 if not self.parameters['serial_number'].startswith('Eval-') and self.parameters['license_type'] == 'azure-cot-premium-byol':
-                    self.parameters['nss_account'] = self.get_nss()
+                    response, msg = self.na_helper.get_nss(self.rest_api, self.headers)
+                    if response is None:
+                        self.module.fail_json(msg)
+                    self.parameters['nss_account'] = response
             elif self.parameters.get('platform_serial_number_node1') is not None and self.parameters.get('platform_serial_number_node2') is not None:
                 if not self.parameters['platform_serial_number_node1'].startswith('Eval-')\
                         and not self.parameters['platform_serial_number_node2'].startswith('Eval-')\
                         and self.parameters['license_type'] == 'azure-ha-cot-premium-byol':
-                    self.parameters['nss_account'] = self.get_nss()
+                    response, msg = self.na_helper.get_nss(self.rest_api, self.headers)
+                    if response is None:
+                        self.module.fail_json(msg)
+                    self.parameters['nss_account'] = response
 
         json = {"name": self.parameters['name'],
                 "region": self.parameters['location'],
@@ -469,8 +468,8 @@ class NetAppCloudManagerCVOAZURE:
                 "enableMonitoring": self.parameters['enable_monitoring'],
                 "vsaMetadata": {
                     "ontapVersion": self.parameters['ontap_version'],
-                    "useLatestVersion": self.parameters['use_latest_version'],
                     "licenseType": self.parameters['license_type'],
+                    "useLatestVersion": self.parameters['use_latest_version'],
                     "instanceType": self.parameters['instance_type']}
                 }
 
@@ -478,6 +477,12 @@ class NetAppCloudManagerCVOAZURE:
             json.update({"capacityTier": self.parameters['capacity_tier'],
                          "tierLevel": self.parameters['tier_level']})
 
+        if self.parameters.get('provided_license') is not None:
+            json['vsaMetadata'].update({"providedLicense": self.parameters['provided_license']})
+
+        if self.parameters.get('capacity_package_name') is not None:
+            json['vsaMetadata'].update({"capacityPackageName": self.parameters['capacity_package_name']})
+
         if self.parameters.get('cidr') is not None:
             json.update({"cidr": self.parameters['cidr']})
 
@@ -505,6 +510,10 @@ class NetAppCloudManagerCVOAZURE:
         if self.parameters.get('nss_account') is not None:
             json.update({"nssAccount": self.parameters['nss_account']})
 
+        if self.parameters['data_encryption_type'] == "AZURE":
+            if self.parameters.get('azure_encryption_parameters') is not None:
+                json.update({"azureEncryptionParameters": {"key": self.parameters['azure_encryption_parameters']}})
+
         if self.parameters.get('azure_tag') is not None:
             tags = []
             for each_tag in self.parameters['azure_tag']:
@@ -525,28 +534,27 @@ class NetAppCloudManagerCVOAZURE:
             if self.parameters.get('platform_serial_number_node2'):
                 ha_params["platformSerialNumberNode2"] = self.parameters['platform_serial_number_node2']
 
+            if self.parameters.get('ha_enable_https') is not None:
+                ha_params['enableHttps'] = self.parameters['ha_enable_https']
+
             json["haParams"] = ha_params
 
-        if self.parameters.get('vnet_resource_group') is not None:
-            json.update({"vnetId": ('/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
-                self.parameters['subscription_id'], self.parameters['vnet_resource_group'], self.parameters['vnet_id']))})
-            json.update({"subnetId": ('/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s/subnets/%s' % (
-                self.parameters['subscription_id'], self.parameters['vnet_resource_group'], self.parameters['vnet_id'], self.parameters['subnet_id']))})
-        else:
-            json.update({"vnetId": ('/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
-                self.parameters['subscription_id'], self.parameters['resource_group'], self.parameters['vnet_id']))})
-            json.update({"subnetId": ('/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s/subnets/%s' % (
-                self.parameters['subscription_id'], self.parameters['resource_group'], self.parameters['vnet_id'], self.parameters['subnet_id']))})
+        resource_group = self.parameters['vnet_resource_group'] if self.parameters.get(
+            'vnet_resource_group') is not None else self.parameters['resource_group']
 
-        base_url = '/occm/api/azure/%s/working-environments' % ('ha' if self.parameters['is_ha'] else 'vsa')
+        resource_group_path = 'subscriptions/%s/resourceGroups/%s' % (self.parameters['subscription_id'], resource_group)
+        vnet_format = '%s/%s' if self.rest_api.simulator else '/%s/providers/Microsoft.Network/virtualNetworks/%s'
+        vnet = vnet_format % (resource_group_path, self.parameters['vnet_id'])
+        json.update({"vnetId": vnet})
+        json.update({"subnetId": '%s/subnets/%s' % (vnet, self.parameters['subnet_id'])})
 
-        api_url = '%s%s' % (Cloud_Manager_Host, base_url)
+        api_url = '%s/working-environments' % self.rest_api.api_root_path
         response, error, on_cloud_request_id = self.rest_api.post(api_url, json, header=self.headers)
         if error is not None:
             self.module.fail_json(
                 msg="Error: unexpected response on creating cvo azure: %s, %s" % (str(error), str(response)))
         working_environment_id = response['publicId']
-        wait_on_completion_api_url = '%s/occm/api/audit/activeTask/%s' % (Cloud_Manager_Host, str(on_cloud_request_id))
+        wait_on_completion_api_url = '/occm/api/audit/activeTask/%s' % str(on_cloud_request_id)
         err = self.rest_api.wait_on_completion(wait_on_completion_api_url, "CVO", "create", 60, 60)
 
         if err is not None:
@@ -554,18 +562,36 @@ class NetAppCloudManagerCVOAZURE:
 
         return working_environment_id
 
+    def update_cvo_azure(self, working_environment_id, modify):
+        base_url = '%s/working-environments/%s/' % (self.rest_api.api_root_path, working_environment_id)
+        for item in modify:
+            if item == 'svm_password':
+                response, error = self.na_helper.update_svm_password(base_url, self.rest_api, self.headers, self.parameters['svm_password'])
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
+            if item == 'azure_tag':
+                tag_list = None
+                if 'azure_tag' in self.parameters:
+                    tag_list = self.parameters['azure_tag']
+                response, error = self.na_helper.update_cvo_tags(base_url, self.rest_api, self.headers, 'azure_tag', tag_list)
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
+            if item == 'tier_level':
+                response, error = self.na_helper.update_tier_level(base_url, self.rest_api, self.headers, self.parameters['tier_level'])
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
+
     def delete_cvo_azure(self, we_id):
         """
         Delete AZURE CVO
         """
-        base_url = '/occm/api/azure/%s/working-environments' % ('ha' if self.parameters['is_ha'] else 'vsa')
 
-        api_url = '%s%s/%s' % (Cloud_Manager_Host, base_url, we_id)
+        api_url = '%s/working-environments/%s' % (self.rest_api.api_root_path, we_id)
         response, error, on_cloud_request_id = self.rest_api.delete(api_url, None, header=self.headers)
         if error is not None:
             self.module.fail_json(msg="Error: unexpected response on deleting cvo azure: %s, %s" % (str(error), str(response)))
 
-        wait_on_completion_api_url = '%s/occm/api/audit/activeTask/%s' % (Cloud_Manager_Host, str(on_cloud_request_id))
+        wait_on_completion_api_url = '/occm/api/audit/activeTask/%s' % str(on_cloud_request_id)
         err = self.rest_api.wait_on_completion(wait_on_completion_api_url, "CVO", "delete", 40, 60)
 
         if err is not None:
@@ -583,22 +609,40 @@ class NetAppCloudManagerCVOAZURE:
                 self.module.fail_json(msg="both platform_serial_number_node1 and platform_serial_number_node2 parameters are required"
                                           "when having ha type as true and license_type as azure-ha-cot-premium-byol")
 
+        if self.parameters.get('capacity_package_name') is not None:
+            if self.parameters['is_ha'] is True and self.parameters['license_type'] != 'ha-capacity-paygo':
+                self.module.fail_json(msg="license_type value must be ha-capacity-paygo")
+            if self.parameters['is_ha'] is False and self.parameters['license_type'] != 'capacity-paygo':
+                self.module.fail_json(msg="license_type value must be capacity-paygo")
+
     def apply(self):
         """
         Apply action to the Cloud Manager CVO for AZURE
         :return: None
         """
         working_environment_id = None
-        current = self.get_working_environment()
+        modify = None
+        current, dummy = self.na_helper.get_working_environment_details_by_name(self.rest_api, self.headers,
+                                                                                self.parameters['name'], "azure")
+        if current:
+            self.parameters['working_environment_id'] = current['publicId']
         # check the action whether to create, delete, or not
         cd_action = self.na_helper.get_cd_action(current, self.parameters)
 
+        if current and self.parameters['state'] != 'absent':
+            working_environment_id = current['publicId']
+            modify, error = self.na_helper.is_cvo_update_needed(self.rest_api, self.headers, self.parameters, self.changeable_params, 'azure')
+            if error is not None:
+                self.module.fail_json(changed=False, msg=error)
+
         if self.na_helper.changed and not self.module.check_mode:
             if cd_action == "create":
                 self.validate_cvo_params()
                 working_environment_id = self.create_cvo_azure()
             elif cd_action == "delete":
                 self.delete_cvo_azure(current['publicId'])
+            else:
+                self.update_cvo_azure(current['publicId'], modify)
 
         self.module.exit_json(changed=self.na_helper.changed, working_environment_id=working_environment_id)
 
