@@ -154,7 +154,7 @@ options:
 EXAMPLES = """
 
     - name: Create FlexCache
-      na_ontap_FlexCache:
+      netapp.ontap.na_ontap_flexcache:
         state: present
         origin_volume: test_src
         name: test_dest
@@ -165,7 +165,7 @@ EXAMPLES = """
         password: "{{ netapp_password }}"
 
     - name: Delete FlexCache
-      na_ontap_FlexCache:
+      netapp.ontap.na_ontap_flexcache:
         state: absent
         name: test_dest
         vserver: ansible_dest
@@ -184,8 +184,8 @@ from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils._text import to_native
 import ansible_collections.netapp.ontap.plugins.module_utils.netapp as netapp_utils
 from ansible_collections.netapp.ontap.plugins.module_utils.netapp_module import NetAppModule
-import ansible_collections.netapp.ontap.plugins.module_utils.rest_flexcache as rest_flexcache
-import ansible_collections.netapp.ontap.plugins.module_utils.rest_volume as rest_volume
+from ansible_collections.netapp.ontap.plugins.module_utils import rest_flexcache
+from ansible_collections.netapp.ontap.plugins.module_utils import rest_volume
 
 
 class NetAppONTAPFlexCache(object):
