@@ -570,7 +570,7 @@ class AnsibleCloudStackInstance(AnsibleCloudStack):
         ids = self.get_network_ids(network_names)
         res = []
         for i, data in enumerate(network_mappings):
-            res.append({'networkid': ids[i], 'ip': data['ip']})
+            res.append(dict(networkid=ids[i], **data))
         return res
 
     def get_ssh_keypair(self, key=None, name=None, fail_on_missing=True):
@@ -695,7 +695,10 @@ class AnsibleCloudStackInstance(AnsibleCloudStack):
         cpu_speed = self.module.params.get('cpu_speed')
         memory = self.module.params.get('memory')
         if all([cpu, cpu_speed, memory]):
-            details.extends({
+            if details is None:
+                details = {}
+
+            details.update({
                 'cpuNumber': cpu,
                 'cpuSpeed': cpu_speed,
                 'memory': memory,
