@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 """
 Command line tool to bisect failing CPython tests.
 
@@ -16,7 +16,6 @@ Load an existing list of tests from a file using -i option:
     ./python -m test --list-cases -m FileTests test_os > tests
     ./python -m test.bisect_cmd -i tests test_os
 """
-from __future__ import print_function
 
 import argparse
 import datetime
@@ -39,7 +38,7 @@ def write_tests(filename, tests):
 def write_output(filename, tests):
     if not filename:
         return
-    print("Write %s tests into %s" % (len(tests), filename))
+    print("Writing %s tests into %s" % (len(tests), filename))
     write_tests(filename, tests)
     return filename
 
@@ -51,23 +50,16 @@ def format_shell_args(args):
 def list_cases(args):
     cmd = [sys.executable, '-m', 'test', '--list-cases']
     cmd.extend(args.test_args)
-    proc = subprocess.Popen(cmd,
-                            stdout=subprocess.PIPE,
-                            universal_newlines=True)
-    try:
-        stdout = proc.communicate()[0]
-    except:
-        proc.stdout.close()
-        proc.kill()
-        proc.wait()
-        raise
-    exitcode = proc.wait()
+    proc = subprocess.run(cmd,
+                          stdout=subprocess.PIPE,
+                          universal_newlines=True)
+    exitcode = proc.returncode
     if exitcode:
         cmd = format_shell_args(cmd)
         print("Failed to list tests: %s failed with exit code %s"
               % (cmd, exitcode))
         sys.exit(exitcode)
-    tests = stdout.splitlines()
+    tests = proc.stdout.splitlines()
     return tests
 
 
@@ -79,14 +71,8 @@ def run_tests(args, tests, huntrleaks=None):
         cmd = [sys.executable, '-m', 'test', '--matchfile', tmp]
         cmd.extend(args.test_args)
         print("+ %s" % format_shell_args(cmd))
-        proc = subprocess.Popen(cmd)
-        try:
-            exitcode = proc.wait()
-        except:
-            proc.kill()
-            proc.wait()
-            raise
-        return exitcode
+        proc = subprocess.run(cmd)
+        return proc.returncode
     finally:
         if os.path.exists(tmp):
             os.unlink(tmp)
@@ -130,7 +116,7 @@ def main():
     output = write_output(args.output, tests)
     print()
 
-    start_time = time.time()
+    start_time = time.monotonic()
     iteration = 1
     try:
         while len(tests) > args.max_tests and iteration <= args.max_iter:
@@ -147,11 +133,11 @@ def main():
             print("ran %s tests/%s" % (ntest, len(tests)))
             print("exit", exitcode)
             if exitcode:
-                print("Tests failed: use this new subtest")
+                print("Tests failed: continuing with this subtest")
                 tests = subtests
                 output = write_output(args.output, tests)
             else:
-                print("Tests succeeded: skip this subtest, try a new subbset")
+                print("Tests succeeded: skipping this subtest, trying a new subset")
             print()
             iteration += 1
     except KeyboardInterrupt:
@@ -167,7 +153,7 @@ def main():
     if output:
         print("Output written into %s" % output)
 
-    dt = math.ceil(time.time() - start_time)
+    dt = math.ceil(time.monotonic() - start_time)
     if len(tests) <= args.max_tests:
         print("Bisection completed in %s iterations and %s"
               % (iteration, datetime.timedelta(seconds=dt)))
