@@ -1,7 +1,7 @@
 <?php
 
 /**
- *    Copyright (C) 2017 Frank Wall
+ *    Copyright (C) 2017-2021 Frank Wall
  *    Copyright (C) 2015 Deciso B.V.
  *
  *    All rights reserved.
@@ -28,6 +28,7 @@
  *    POSSIBILITY OF SUCH DAMAGE.
  *
  */
+
 namespace OPNsense\AcmeClient;
 
 /**
@@ -40,6 +41,9 @@ class ActionsController extends \OPNsense\Base\IndexController
     {
         // include form definitions
         $this->view->formDialogAction = $this->getForm("dialogAction");
+        // set additional view parameters
+        $mdlAcme = new \OPNsense\AcmeClient\AcmeClient();
+        $this->view->showIntro = (string)$mdlAcme ->settings->showIntro;
         // choose template
         $this->view->pick('OPNsense/AcmeClient/actions');
     }
