@@ -1,4 +1,5 @@
 <?php
+
 /**
  *    Copyright (C) 2017-2019 Frank Wall
  *    Copyright (C) 2015 Deciso B.V.
@@ -27,12 +28,13 @@
  *    POSSIBILITY OF SUCH DAMAGE.
  *
  */
+
 namespace OPNsense\AcmeClient\Api;
 
-use \OPNsense\Base\ApiMutableModelControllerBase;
-use \OPNsense\Base\UIModelGrid;
-use \OPNsense\Core\Config;
-use \OPNsense\AcmeClient\AcmeClient;
+use OPNsense\Base\ApiMutableModelControllerBase;
+use OPNsense\Base\UIModelGrid;
+use OPNsense\Core\Config;
+use OPNsense\AcmeClient\AcmeClient;
 
 /**
  * Class ValidationsController
@@ -71,6 +73,6 @@ class ValidationsController extends ApiMutableModelControllerBase
 
     public function searchAction()
     {
-        return $this->searchBase('validations.validation', array('enabled', 'name', 'description'), 'name');
+        return $this->searchBase('validations.validation', array('enabled', 'name', 'method', 'description'), 'name');
     }
 }
