@@ -39,7 +39,6 @@ use OPNsense\Core\Config;
  */
 class FirewallController extends ApiControllerBase
 {
-
     /**
      * retrieve firewall log
      * @return array
@@ -348,4 +347,13 @@ class FirewallController extends ApiControllerBase
         }
         return ["result" => "failed"];
     }
+
+    /**
+     * retrieve various pf statistics
+     * @return mixed
+     */
+    public function pfStatistcsAction($section = null)
+    {
+        return json_decode((new Backend())->configdpRun('filter diag info', [$section]), true);
+    }
 }
