@@ -37,7 +37,7 @@ use OPNsense\AcmeClient\LeAutomationFactory;
 use OPNsense\AcmeClient\LeValidationFactory;
 
 /**
- * Manage Let's Encrypt certificates with acme.sh
+ * Manage ACME certificates with acme.sh
  * @package OPNsense\AcmeClient
  */
 class LeCertificate extends LeCommon
@@ -74,13 +74,16 @@ class LeCertificate extends LeCommon
         // Set log level
         $this->setLoglevel();
 
-        // Set Let's Encrypt environment
-        $this->setEnvironment();
+        // Set ACME CA
+        $this->setCa((string)$this->config->account);
 
         // Handle special key types
         if ($this->config->keyLength == 'key_ec256' || $this->config->keyLength == 'key_ec384') {
             // Pass --ecc to acme client to locate the correct cert directory
             $this->acme_args[] = '--ecc';
+            $this->cert_ecc = true;
+        } else {
+            $this->cert_ecc = false;
         }
 
         // Store cert filenames
@@ -159,7 +162,7 @@ class LeCertificate extends LeCommon
         foreach (Config::getInstance()->object()->ca as $cacrt) {
             $cacrt_subject = cert_get_subject($cacrt->crt, true);
             $cacrt_issuer = cert_get_issuer($cacrt->crt, true);
-            if (($ca_subject == $cacrt_subject) and ($ca_issuer == $cacrt_issuer)) {
+            if (($ca_subject === $cacrt_subject) and ($ca_issuer === $cacrt_issuer)) {
                 // Use old refid instead of generating a new one
                 $ca['refid'] = (string)$cacrt->refid;
                 $ca_found = true;
@@ -169,7 +172,7 @@ class LeCertificate extends LeCommon
 
         // Collect required CA information
         $ca_cn = LeUtils::local_cert_get_cn($ca_content, false);
-        $ca['descr'] = (string)$ca_cn . ' (Let\'s Encrypt)';
+        $ca['descr'] = (string)$ca_cn . ' (ACME Client)';
 
         // Prepare CA for import
         LeUtils::local_ca_import($ca, $ca_content);
@@ -186,7 +189,7 @@ class LeCertificate extends LeCommon
             }
         } else {
             // Create new CA
-            LeUtils::log("importing Let's Encrypt CA: ${ca_cn}");
+            LeUtils::log("importing ACME CA: ${ca_cn}");
             $newca = Config::getInstance()->object()->addChild('ca');
             foreach (array_keys($ca) as $cacfg) {
                 $newca->addChild($cacfg, (string)$ca[$cacfg]);
@@ -251,12 +254,18 @@ class LeCertificate extends LeCommon
 
         // Collect required cert information
         $cert_cn = LeUtils::local_cert_get_cn($cert_content, false);
-        $cert['descr'] = (string)$cert_cn . ' (Let\'s Encrypt)';
+        $cert['descr'] = (string)$cert_cn . ' (ACME Client)';
         $cert['refid'] = $cert_refid;
 
         // Prepare certificate for import
         cert_import($cert, $cert_content, $key_content);
 
+        // Overwrite caref in order to use the correct CA (GH #2550).
+        // This is required because cert_import() uses lookup_ca_by_subject()
+        // to find a matching CA. If multiple CAs are using the same name, the
+        // first CA wins, but it may still be the wrong CA.
+        $cert['caref'] = (string)$ca['refid'];
+
         // Check if cert was found in config
         if ($cert_found == true) {
             // Update existing cert
@@ -277,7 +286,7 @@ class LeCertificate extends LeCommon
                 $newcert->addChild($certcfg, (string)$cert[$certcfg]);
             }
         }
-        LeUtils::log("${import_log_message} Let's Encrypt X.509 certificate: ${cert_cn}");
+        LeUtils::log("${import_log_message} ACME X.509 certificate: ${cert_cn}");
 
         /**
          * Step 3: update configuration
@@ -300,7 +309,7 @@ class LeCertificate extends LeCommon
     }
 
     /**
-     * check if certificate is already issued by Let's Encrypt
+     * check if certificate is already issued by ACME CA
      * @return bool
      */
     public function isIssued()
@@ -351,6 +360,7 @@ class LeCertificate extends LeCommon
             return false;
         }
         LeUtils::log("${acme_action} certificate: " . (string)$this->config->name);
+        LeUtils::log('using CA: ' . $this->ca);
 
         // Ensure that account is registered.
         if (!($this->setAccount())) {
@@ -407,8 +417,31 @@ class LeCertificate extends LeCommon
     {
         $return = false;
 
+        // Try to get issue date from certificate
+        if (is_file($this->cert_file)) {
+            // Read contents from certificate file
+            $cert_content = @file_get_contents($this->cert_file);
+            if ($cert_content != false) {
+                $cert_info = @openssl_x509_parse($cert_content);
+                if (!empty($cert_info['validFrom_time_t'])) {
+                    $last_update = $cert_info['validFrom_time_t'];
+                } else {
+                    LeUtils::log_error('unable to get expiration time from certificate for ' . (string)$this->config->name);
+                    $last_update = 0; // Just assume the cert requires renewal.
+                }
+            } else {
+                LeUtils::log_error('unable to read certificate content from file for ' . (string)$this->config->name);
+                $last_update = 0; // Just assume the cert requires renewal.
+            }
+        } elseif (!empty((string)$this->config->lastUpdate)) {
+            // Fallback to lastUpdate() state, although it may not be correct
+            // if the cert was imported manually after issue/renewal.
+            $last_update = (string)$this->config->lastUpdate;
+        } else {
+            $last_update = 0; // Just assume the cert requires renewal.
+        }
+
         // Collect required information
-        $last_update = !empty((string)$this->config->lastUpdate) ? (string)$this->config->lastUpdate : 0;
         $current_time = new \DateTime();
         $last_update_time = new \DateTime();
         $last_update_time->setTimestamp($last_update);
@@ -528,7 +561,7 @@ class LeCertificate extends LeCommon
         LeUtils::log('revoking certificate: ' . (string)$this->config->name);
 
         // Collect account information
-        $account_conf_dir = self::ACME_BASE_ACCOUNT_DIR . '/' . $this->account_id . '_' . $this->environment;
+        $account_conf_dir = self::ACME_BASE_ACCOUNT_DIR . '/' . $this->account_id . '_' . $this->ca_compat;
         $account_conf_file = $account_conf_dir . '/account.conf';
 
         // Preparation to run acme client
@@ -598,7 +631,7 @@ class LeCertificate extends LeCommon
         foreach ($automations as $auto_uuid) {
             $autoFactory = new LeAutomationFactory();
             $automation = $autoFactory->getAutomation($auto_uuid);
-            $automation->init($this->getId(), (string)$this->config->account);
+            $automation->init($this->getId(), (string)$this->config->name, (string)$this->config->account, $this->cert_ecc);
             // Ignore invalid automations.
             if ($automation->prepare()) {
                 $automation->run();
@@ -647,7 +680,7 @@ class LeCertificate extends LeCommon
                 LeUtils::log_error('invalid challenge type for certificate: ' . (string)$this->config->name);
                 return false;
             }
-            if (!$val->init((string)$this->config->id, (string)$this->config->account)) {
+            if (!$val->init((string)$this->config->id, (string)$this->config->account, $this->cert_ecc)) {
                 LeUtils::log_error('failed to initialize validation for certificate: ' . (string)$this->config->name);
                 return false;
             }
