@@ -1,7 +1,7 @@
 <?php
 
 /*
- * Copyright (C) 2015 Deciso B.V.
+ * Copyright (C) 2015-2021 Deciso B.V.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
 namespace OPNsense\Core;
 
 use Phalcon\DI\FactoryDefault;
+use Phalcon\Logger;
 use Phalcon\Logger\Adapter\Syslog;
 
 /**
@@ -37,7 +38,6 @@ use Phalcon\Logger\Adapter\Syslog;
  */
 class Config extends Singleton
 {
-
     /**
      * config file location ( path + name )
      * @var string
@@ -79,7 +79,7 @@ class Config extends Singleton
      */
     private function isArraySequential(&$arrayData)
     {
-        return ctype_digit(implode('', array_keys($arrayData)));
+        return is_array($arrayData) && ctype_digit(implode('', array_keys($arrayData)));
     }
 
     /**
@@ -101,9 +101,9 @@ class Config extends Singleton
         // copy attributes to @attribute key item
         foreach ($node->attributes() as $AttrKey => $AttrValue) {
             if (!isset($result['@attributes'])) {
-                $result['@attributes'] = array();
+                $result['@attributes'] = [];
             }
-            $result['@attributes'][$AttrKey] = $AttrValue->__toString();
+            $result['@attributes'][$AttrKey] = (string)$AttrValue;
         }
         // iterate xml children
         foreach ($node->children() as $xmlNode) {
@@ -135,16 +135,23 @@ class Config extends Singleton
                         $result[$xmlNodeName] = array();
                         $result[$xmlNodeName][] = $tmp;
                     }
-                    $result[$xmlNodeName][] = $xmlNode->__toString();
+                    $result[$xmlNodeName][] = (string)$xmlNode;
                 } else {
                     // single content item
                     if (isset($forceList[$xmlNodeName])) {
                         $result[$xmlNodeName] = array();
-                        if ($xmlNode->__toString() != null && trim($xmlNode->__toString()) !== "") {
-                            $result[$xmlNodeName][] = $xmlNode->__toString();
+                        if ((string)$xmlNode != null && trim((string)$xmlNode) !== '') {
+                            $result[$xmlNodeName][] = (string)$xmlNode;
                         }
                     } else {
-                        $result[$xmlNodeName] = $xmlNode->__toString();
+                        $result[$xmlNodeName] = (string)$xmlNode;
+                    }
+                    // copy attributes to xzy@attribute key item
+                    foreach ($xmlNode->attributes() as $AttrKey => $AttrValue) {
+                        if (!isset($result["${xmlNodeName}@attributes"])) {
+                            $result["${xmlNodeName}@attributes"] = [];
+                        }
+                        $result["${xmlNodeName}@attributes"][$AttrKey] = (string)$AttrValue;
                     }
                 }
             }
@@ -205,10 +212,19 @@ class Config extends Singleton
                     $node->addAttribute($attrKey, $attrValue);
                 }
                 continue;
+            } elseif (strstr($itemKey, '@attributes') !== false) {
+                $origname = str_replace('@attributes', '', $itemKey);
+                if (count($node->$origname)) {
+                    // copy xml attributes
+                    foreach ($itemValue as $attrKey => $attrValue) {
+                        $node->$origname->addAttribute($attrKey, $attrValue);
+                    }
+                }
+                continue;
             } elseif (is_numeric($itemKey)) {
                 // recurring tag (content), use parent tagname.
                 $childNode = $node->addChild($parentTagName);
-            } elseif (is_array($itemValue) && $this->isArraySequential($itemValue)) {
+            } elseif ($this->isArraySequential($itemValue)) {
                 // recurring tag, skip placeholder.
                 $childNode = $node;
             } else {
@@ -285,7 +301,15 @@ class Config extends Singleton
             $this->simplexml = null;
             // there was an issue with loading the config, try to restore the last backup
             $backups = $this->getBackups();
-            $logger = new Syslog("config", array('option' => LOG_PID, 'facility' => LOG_LOCAL4));
+            $logger = new Logger(
+                'messages',
+                [
+                    'main' => new Syslog("config", array(
+                        'option' => LOG_PID,
+                        'facility' => LOG_LOCAL4
+                    ))
+                ]
+            );
             if (count($backups) > 0) {
                 // load last backup
                 $logger->error(gettext('No valid config.xml found, attempting last known config restore.'));
@@ -455,25 +479,50 @@ class Config extends Singleton
     }
 
     /**
-     * backup current (running) config
-     * @return float timestamp
+     * send config change to audit log including the context we currently know of.
+     */
+    private function auditLogChange($backup_filename, $revision = null)
+    {
+        openlog("audit", LOG_ODELAY, LOG_AUTH);
+        $append_message = "";
+        if (is_array($revision) && !empty($revision['description'])) {
+            $append_message = sprintf(" [%s]", $revision['description']);
+        }
+        syslog(LOG_NOTICE, sprintf(
+            "user %s%s changed configuration to %s in %s%s",
+            !empty($_SESSION["Username"]) ? $_SESSION["Username"] : "(system)",
+            !empty($_SERVER['REMOTE_ADDR']) ? "@" . $_SERVER['REMOTE_ADDR'] : "",
+            $backup_filename,
+            !empty($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : $_SERVER['SCRIPT_NAME'],
+            $append_message
+        ));
+        closelog();
+    }
+
+    /**
+     * backup current config
+     * @return string target filename
      */
     public function backup()
     {
         $timestamp = microtime(true);
         $target_dir = dirname($this->config_file) . "/backup/";
-        $target_filename = "config-" . $timestamp . ".xml";
 
         if (!file_exists($target_dir)) {
             // create backup directory if it is missing
             mkdir($target_dir);
         }
-        // The new target backup filename shouldn't exists, because of the use of microtime.
-        // But if for some reason a script keeps calling this backup very often, it should not crash.
-        if (!file_exists($target_dir . $target_filename)) {
-            copy($this->config_file, $target_dir . $target_filename);
+        if (file_exists($target_dir . "config-" . $timestamp . ".xml")) {
+            // The new target backup filename shouldn't exists, because of the use of microtime.
+            // in the unlikely event that we can process events too fast for microtime(), suffix with a more
+            // precise tiestamp to ensure we can't miss a backup
+            $target_filename = "config-" . $timestamp . "_" .  hrtime()[1] . ".xml";
+        } else {
+            $target_filename = "config-" . $timestamp . ".xml";
         }
-        return $timestamp;
+        copy($this->config_file, $target_dir . $target_filename);
+
+        return $target_dir . $target_filename;
     }
 
     /**
@@ -498,7 +547,7 @@ class Config extends Singleton
                     $xmlNode = @simplexml_load_file($filename, "SimpleXMLElement", LIBXML_NOERROR | LIBXML_ERR_NONE);
                     if (isset($xmlNode->revision)) {
                         $result[$filename] = $this->toArray(null, $xmlNode->revision);
-                        $result[$filename]['version'] = $xmlNode->version->__toString();
+                        $result[$filename]['version'] = (string)$xmlNode->version;
                         $result[$filename]['filesize'] = filesize($filename);
                     }
                 }
@@ -555,7 +604,7 @@ class Config extends Singleton
         ) {
             return intval($this->simplexml->system->backupcount);
         } else {
-            return 60;
+            return 100;
         }
     }
 
@@ -601,25 +650,33 @@ class Config extends Singleton
     {
         $this->checkvalid();
 
-        if ($backup) {
-            $timestamp = $this->backup();
-        } else {
-            $timestamp = microtime(true);
-        }
-
         // update revision information ROOT.revision tag, align timestamp to backup output
-        $this->updateRevision($revision, null, $timestamp);
-
-        // serialize to text
-        $xml_text = $this->__toString();
+        $this->updateRevision($revision, null, microtime(true));
 
         if ($this->config_file_handle !== null) {
             if (flock($this->config_file_handle, LOCK_EX)) {
                 fseek($this->config_file_handle, 0);
                 ftruncate($this->config_file_handle, 0);
-                fwrite($this->config_file_handle, $xml_text);
+                fwrite($this->config_file_handle, (string)$this);
                 // flush, unlock, but keep the handle open
                 fflush($this->config_file_handle);
+                $backup_filename = $backup ? $this->backup() : null;
+                if ($backup_filename) {
+                    $this->auditLogChange($backup_filename, $revision);
+                    // use syslog to trigger a new configd event, which should signal a syshook config (in batch).
+                    // Althought we include the backup filename, the event handler is responsible to determine the
+                    // last processed event itself. (it's merely added for debug purposes)
+                    $logger = new Logger(
+                        'messages',
+                        [
+                            'main' => new Syslog("config", array(
+                                'option' => LOG_PID,
+                                'facility' => LOG_LOCAL5
+                            ))
+                        ]
+                    );
+                    $logger->info("config-event: new_config " . $backup_filename);
+                }
                 flock($this->config_file_handle, LOCK_UN);
             } else {
                 throw new ConfigException("Unable to lock config");
