@@ -2,7 +2,8 @@
 
 /*
 
-    Copyright (C) 2018 Fabian Franz
+    Copyright (C) 2018-2020 Fabian Franz
+    Copyright (C) 2020 Manuel Faux
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -29,32 +30,11 @@
 
 namespace OPNsense\Nginx;
 
-class AccessLogParser
+class AccessLogParser extends LogParserBase
 {
-    private $file_name;
-    private $result;
-
     private const LogLineRegex = '/(\S+) - (\S+) \[([\d\sa-z\:\-\/\+]+)\] "([^"]+?)" (\d+) (\d+) "([^"]*?)" "([^"]*?)" "([^"]*?)"/i';
 
-    function __construct($file_name)
-    {
-        $this->file_name = $file_name;
-        $this->result = array();
-        $this->parse_file();
-    }
-
-    private function parse_file()
-    {
-        $handle = @fopen($this->file_name, 'r');
-        if ($handle) {
-            while (($buffer = fgets($handle)) !== false) {
-                $this->result[] = $this->parse_line($buffer);
-            }
-            fclose($handle);
-        }
-    }
-
-    private function parse_line($line)
+    protected function parse_line($line)
     {
         $container = new AccessLogLine();
         if (preg_match(self::LogLineRegex, $line, $data)) {
@@ -70,9 +50,4 @@ class AccessLogParser
         }
         return $container;
     }
-
-    public function get_result()
-    {
-        return $this->result;
-    }
 }
