@@ -7,6 +7,8 @@ chmod 750 /var/run/c-icap
 mkdir -p /var/log/c-icap
 chown -R c_icap:c_icap /var/log/c-icap
 chmod 750 /var/log/c-icap
+(cd /var/log && ln -s c-icap cicap)
+chown -R c_icap:c_icap /var/log/cicap
 
 mkdir -p /tmp/c-icap/templates/virus_scan/en
 chmod -R 755 /tmp/c-icap/
