@@ -7,42 +7,57 @@ let LogCategoryList = Backbone.View.extend({
     className: "nav nav-tabs",
 
     initialize: function(data) {
-        this.listenTo(this.collection, "sync",   this.render);
         this.listenTo(this.collection, "update", this.render);
         this.logview = data.logview;
+        this.logType = data.logType;
     },
 
     render: function() {
         this.$el.attr('role', 'tablist');
         this.$el.html('');
-        this.collection.forEach((element) => this.render_one(element));
-        this.render_single_tabs();
+
+        if (this.logType == 'global') {
+            this.render_global_error_tab();
+        }
+        else {
+            this.collection.forEach((element) => this.render_one_server(element));
+        }
     },
 
-    render_one: function(element) {
-        const servers = new LogCollection(
+    render_one_server: function(element) {
+        const files = new LogCollection(
             {
-                uuid: element.get('url'),
-                logType: element.get('logType')
+                uuid: element.get('id'),
+                logType: this.logType
             }
         );
         const logList = new TabLogList({
-            collection: servers,
+            collection: files,
             model: element,
+            logType: this.logType,
             logview: this.logview
         });
         this.$el.append(logList.$el);
-        servers.fetch();
     },
-    render_single_tabs: function () {
-        const single_tab = new SingleTab({
-            logview: this.logview,
-            log_name: 'global',
-            visible_name: 'Global Error Log',
-            log_type: 'errors'});
-        single_tab.render();
-        this.$el.append(single_tab.$el);
 
+    render_global_error_tab: function () {
+        const files = new LogCollection(
+            {
+                uuid: 'global',
+                logType: 'errors'
+            }
+        );
+        const logList = new TabLogList({
+            collection: files,
+            model: new Backbone.Model({
+                server_name: 'Global Error Log',
+                id: 'global'
+            }),
+            logType: 'errors',
+            logview: this.logview
+        });
+        this.$el.append(logList.$el);
+        files.fetch();
     }
 });
 
