@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -16,6 +16,6 @@ def plugin(event_tuple, packet=None):
     if packet:
         localtime = time.strftime(TIME_FORMAT, time.localtime(int(sec)))
         output = "\n[%s] %s:%s -> %s:%s:\n" % (localtime, src_ip, src_port, dst_ip, dst_port)
-        output += "%s\n" % "".join(_ if _ in string.printable else '.' for _ in packet)
+        output += "%s\n" % "".join(chr(_) if chr(_) in string.printable else '.' for _ in packet)
         sys.stderr.write(output)
         sys.stderr.flush()
