@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -25,6 +25,7 @@ import struct
 import threading
 import time
 import traceback
+import warnings
 
 from core.addr import inet_ntoa6
 from core.addr import addr_port
@@ -36,6 +37,7 @@ from core.common import get_ex_message
 from core.common import get_text
 from core.common import is_local
 from core.common import load_trails
+from core.common import patch_parser
 from core.compat import xrange
 from core.datatype import LRUDict
 from core.enums import BLOCK_MARKER
@@ -49,24 +51,24 @@ from core.log import log_error
 from core.log import log_event
 from core.parallel import worker
 from core.parallel import write_block
-from core.settings import check_memory
 from core.settings import config
 from core.settings import CAPTURE_TIMEOUT
 from core.settings import CHECK_CONNECTION_MAX_RETRIES
 from core.settings import CONFIG_FILE
 from core.settings import CONSONANTS
-from core.settings import DAILY_SECS
 from core.settings import DLT_OFFSETS
 from core.settings import DNS_EXHAUSTION_THRESHOLD
 from core.settings import GENERIC_SINKHOLE_REGEX
+from core.settings import HOMEPAGE
+from core.settings import HOURLY_SECS
 from core.settings import HTTP_TIME_FORMAT
 from core.settings import IGNORE_DNS_QUERY_SUFFIXES
 from core.settings import IPPROTO_LUT
 from core.settings import IS_WIN
 from core.settings import LOCALHOST_IP
 from core.settings import LOCAL_SUBDOMAIN_LOOKUPS
+from core.settings import MAX_CACHE_ENTRIES
 from core.settings import MMAP_ZFILL_CHUNK_LENGTH
-from core.settings import MAX_RESULT_CACHE_ENTRIES
 from core.settings import NAME
 from core.settings import NO_SUCH_NAME_COUNTERS
 from core.settings import NO_SUCH_NAME_PER_HOUR_THRESHOLD
@@ -91,7 +93,7 @@ from core.settings import SUSPICIOUS_UA_REGEX
 from core.settings import VALID_DNS_NAME_REGEX
 from core.settings import trails
 from core.settings import VERSION
-from core.settings import WEB_SHELLS
+from core.settings import WEB_SCANNING_THRESHOLD
 from core.settings import WHITELIST
 from core.settings import WHITELIST_DIRECT_DOWNLOAD_KEYWORDS
 from core.settings import WHITELIST_LONG_DOMAIN_NAME_KEYWORDS
@@ -102,35 +104,41 @@ from core.update import update_trails
 from thirdparty import six
 from thirdparty.six.moves import urllib as _urllib
 
+warnings.filterwarnings(action="ignore", category=DeprecationWarning)       # NOTE: https://github.com/helpsystems/pcapy/pull/67/files
+
 _buffer = None
 _caps = []
 _connect_sec = 0
 _connect_src_dst = {}
 _connect_src_details = {}
+_path_src_dst = {}
+_path_src_dst_details = {}
 _count = 0
 _locks = AttribDict()
 _multiprocessing = None
 _n = None
-_result_cache = LRUDict(MAX_RESULT_CACHE_ENTRIES)
-_local_cache = {}
+_result_cache = LRUDict(MAX_CACHE_ENTRIES)
+_local_cache = LRUDict(MAX_CACHE_ENTRIES)
 _last_syn = None
 _last_logged_syn = None
 _last_udp = None
 _last_logged_udp = None
-_last_dns_exhaustion = None
 _done_count = 0
 _done_lock = threading.Lock()
 _subdomains = {}
 _subdomains_sec = None
 _dns_exhausted_domains = set()
 
+class _set(set):
+    pass
+
 try:
     import pcapy
 except ImportError:
     if IS_WIN:
         exit("[!] please install 'WinPcap' (e.g. 'http://www.winpcap.org/install/') and Pcapy (e.g. 'https://breakingcode.wordpress.com/?s=pcapy')")
     else:
-        msg = "[!] please install 'Pcapy' (e.g. 'sudo pip%s install pcapy')" % ('3' if six.PY3 else '2')
+        msg = "[!] please install 'Pcapy' (e.g. 'sudo pip%s install pcapy-ng')" % ('3' if six.PY3 else '2')
 
         exit(msg)
 
@@ -162,21 +170,14 @@ def _check_domain(query, sec, usec, src_ip, src_port, dst_ip, dst_port, proto, p
     if query.replace('.', "").isdigit():  # IP address
         return
 
-    if _result_cache.get((CACHE_TYPE.DOMAIN, query)) == False:
+    if _result_cache.get((CACHE_TYPE.DOMAIN, query)) is False:
         return
 
     result = False
     if re.search(VALID_DNS_NAME_REGEX, query) is not None and not _check_domain_whitelisted(query):
         parts = query.split('.')
 
-        if ".onion." in query:
-            trail = re.sub(r"(\.onion)(\..*)", r"\1(\2)", query)
-            _ = trail.split('(')[0]
-            if _ in trails:
-                result = True
-                log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, trails[_][0], trails[_][1]), packet)
-
-        elif query.endswith(".ip-adress.com"):  # Reference: https://www.virustotal.com/gui/domain/ip-adress.com/relations
+        if query.endswith(".ip-adress.com"):  # Reference: https://www.virustotal.com/gui/domain/ip-adress.com/relations
             _ = '.'.join(parts[:-2])
             trail = "%s(.ip-adress.com)" % _
             if _ in trails:
@@ -234,7 +235,14 @@ def _check_domain(query, sec, usec, src_ip, src_port, dst_ip, dst_port, proto, p
 
                     log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, trails[candidate][0], trails[candidate][1]), packet)
 
-    if result == False:
+        if not result and ".onion." in query:
+            trail = re.sub(r"(\.onion)(\..*)", r"\1(\2)", query)
+            _ = trail.split('(')[0]
+            if _ in trails:
+                result = True
+                log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, trails[_][0], trails[_][1]), packet)
+
+    if result is False:
         _result_cache[(CACHE_TYPE.DOMAIN, query)] = False
 
 def _get_local_prefix():
@@ -260,7 +268,6 @@ def _process_packet(packet, sec, usec, ip_offset):
     global _last_logged_syn
     global _last_udp
     global _last_logged_udp
-    global _last_dns_exhaustion
     global _subdomains_sec
 
     try:
@@ -294,6 +301,15 @@ def _process_packet(packet, sec, usec, ip_offset):
                 _connect_src_dst.clear()
                 _connect_src_details.clear()
 
+                for key in _path_src_dst:
+                    if len(_path_src_dst[key]) > WEB_SCANNING_THRESHOLD:
+                        _src_ip, _dst_ip = key.split('~')
+                        _sec, _usec, _src_port, _dst_port, _path = _path_src_dst_details[key].pop()
+                        log_event((_sec, _usec, _src_ip, _src_port, _dst_ip, _dst_port, PROTO.TCP, TRAIL.PATH, "*", "potential web scanning", "(heuristic)"), packet)
+
+                _path_src_dst.clear()
+                _path_src_dst_details.clear()
+
         ip_data = packet[ip_offset:]
         ip_version = ord(ip_data[0:1]) >> 4
         localhost_ip = LOCALHOST_IP[ip_version]
@@ -318,7 +334,7 @@ def _process_packet(packet, sec, usec, ip_offset):
             return
 
         if protocol == socket.IPPROTO_TCP:  # TCP
-            src_port, dst_port, _, _, doff_reserved, flags = struct.unpack("!HHLLBB", ip_data[iph_length:iph_length+14])
+            src_port, dst_port, _, _, doff_reserved, flags = struct.unpack("!HHLLBB", ip_data[iph_length:iph_length + 14])
 
             if flags != 2 and config.plugin_functions:
                 if dst_ip in trails:
@@ -382,7 +398,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                         index = tcp_data.find("<title>")
                         if index >= 0:
                             title = tcp_data[index + len("<title>"):tcp_data.find("</title>", index)]
-                            if all(_ in title.lower() for _ in ("this domain", "has been seized")):
+                            if re.search(r"domain name has been seized by|Domain Seized|Domain Seizure", title):
                                 log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.HTTP, title, "seized domain (suspicious)", "(heuristic)"), packet)
 
                     content_type = None
@@ -422,7 +438,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                             if host and host[0].isalpha() and dst_ip in trails:
                                 log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.IP, "%s (%s)" % (dst_ip, host.split(':')[0]), trails[dst_ip][0], trails[dst_ip][1]), packet)
                             elif re.search(r"\A\d+\.[0-9.]+\Z", host or "") and re.search(SUSPICIOUS_DIRECT_IP_URL_REGEX, "%s%s" % (host, path)):
-                                if not _dst_ip.startswith(_get_local_prefix()):
+                                if not dst_ip.startswith(_get_local_prefix()):
                                     trail = "(%s)%s" % (host, path)
                                     log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.HTTP, trail, "potential iot-malware download (suspicious)", "(heuristic)"), packet)
                                     return
@@ -435,7 +451,20 @@ def _process_packet(packet, sec, usec, ip_offset):
                     if index >= 0:
                         post_data = tcp_data[index + 4:]
 
-                    if config.USE_HEURISTICS and dst_port == 80 and path.startswith("http://") and any(_ in path for _ in SUSPICIOUS_PROXY_PROBE_PRE_CONDITION) and not _check_domain_whitelisted(path.split('/')[2]):
+                    url = None
+                    if config.USE_HEURISTICS and path.startswith('/'):
+                        _path = path.split('/')[1]
+
+                        key = "%s~%s" % (src_ip, dst_ip)
+                        if key not in _path_src_dst:
+                            _path_src_dst[key] = set()
+                        _path_src_dst[key].add(_path)
+
+                        if key not in _path_src_dst_details:
+                            _path_src_dst_details[key] = set()
+                        _path_src_dst_details[key].add((sec, usec, src_port, dst_port, path))
+
+                    elif config.USE_HEURISTICS and dst_port == 80 and path.startswith("http://") and any(_ in path for _ in SUSPICIOUS_PROXY_PROBE_PRE_CONDITION) and not _check_domain_whitelisted(path.split('/')[2]):
                         trail = re.sub(r"(http://[^/]+/)(.+)", r"\g<1>(\g<2>)", path)
                         trail = re.sub(r"(http://)([^/(]+)", lambda match: "%s%s" % (match.group(1), match.group(2).split(':')[0].rstrip('.')), trail)
                         log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.HTTP, trail, "potential proxy probe (suspicious)", "(heuristic)"), packet)
@@ -470,7 +499,8 @@ def _process_packet(packet, sec, usec, ip_offset):
                         url = "%s%s" % (host, path)
                         proxy_domain = host.split(':')[0]
                         _check_domain(proxy_domain, sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, packet)
-                    else:
+
+                    if url is None:
                         url = "%s%s" % (host, path)
 
                     if config.USE_HEURISTICS:
@@ -506,22 +536,27 @@ def _process_packet(packet, sec, usec, ip_offset):
                                 log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.UA, result, "user agent (suspicious)", "(heuristic)"), packet)
 
                     if not _check_domain_whitelisted(host):
+                        path = path.replace("//", '/')
+
                         unquoted_path = _urllib.parse.unquote(path)
                         unquoted_post_data = _urllib.parse.unquote(post_data or "")
 
                         checks = [path.rstrip('/')]
+
                         if '?' in path:
                             checks.append(path.split('?')[0].rstrip('/'))
 
                             if '=' in path:
                                 checks.append(path[:path.index('=') + 1])
+
+                            _ = re.sub(r"(\w+=)[^&=]+", r"\g<1>", path)
+                            if _ not in checks:
+                                checks.append(_)
+                                if _.count('/') > 1:
+                                    checks.append("/%s" % _.split('/')[-1])
                         elif post_data:
                             checks.append("%s?%s" % (path, unquoted_post_data.lower()))
 
-                        #_ = os.path.splitext(checks[-1])       # causing FPs in cases like elf_mirai - /juno if legit /juno.php is accessed
-                        #if _[1]:
-                            #checks.append(_[0])
-
                         if checks[-1].count('/') > 1:
                             checks.append(checks[-1][:checks[-1].rfind('/')])
                             checks.append(checks[0][checks[0].rfind('/'):].split('?')[0])
@@ -547,7 +582,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                             return
 
                         if config.USE_HEURISTICS:
-                            match = re.search(r"\bX-Forwarded-For:\s*([0-9.]+)".encode(), packet, re.I)
+                            match = re.search(r"\b(CF-Connecting-IP|True-Client-IP|X-Forwarded-For):\s*([0-9.]+)".encode(), packet, re.I)
                             if match:
                                 src_ip = "%s,%s" % (src_ip, match.group(1))
 
@@ -566,7 +601,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                                                 found = desc
                                                 break
                                         _result_cache[(CACHE_TYPE.PATH, unquoted_path)] = found or ""
-                                    if found:
+                                    if found and not ("data leakage" in found and is_local(dst_ip)):
                                         trail = "%s(%s)" % (host, path)
                                         log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.URL, trail, "%s (suspicious)" % found, "(heuristic)"), packet)
                                         return
@@ -590,10 +625,8 @@ def _process_packet(packet, sec, usec, ip_offset):
                                 filename = _.path.split('/')[-1]
                                 name, extension = os.path.splitext(filename)
                                 trail = "%s(%s)" % (host, path)
-                                if extension and extension in SUSPICIOUS_DIRECT_DOWNLOAD_EXTENSIONS and not any(_ in path for _ in WHITELIST_DIRECT_DOWNLOAD_KEYWORDS) and '=' not in _.query and len(name) < 10:
+                                if extension in SUSPICIOUS_DIRECT_DOWNLOAD_EXTENSIONS and not is_local(dst_ip) and not any(_ in path for _ in WHITELIST_DIRECT_DOWNLOAD_KEYWORDS) and '=' not in _.query and len(name) < 10:
                                     log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.URL, trail, "direct %s download (suspicious)" % extension, "(heuristic)"), packet)
-                                elif filename in WEB_SHELLS:
-                                    log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.URL, trail, "potential web shell (suspicious)", "(heuristic)"), packet)
                                 else:
                                     for desc, regex in SUSPICIOUS_HTTP_PATH_REGEXES:
                                         if re.search(regex, filename, re.I):
@@ -662,7 +695,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                                     domain = '.'.join(parts[-2:])
 
                                 if not _check_domain_whitelisted(domain):  # e.g. <hash>.hashserver.cs.trendmicro.com
-                                    if (sec - (_subdomains_sec or 0)) > DAILY_SECS:
+                                    if (sec - (_subdomains_sec or 0)) > HOURLY_SECS:
                                         _subdomains.clear()
                                         _dns_exhausted_domains.clear()
                                         _subdomains_sec = sec
@@ -670,19 +703,21 @@ def _process_packet(packet, sec, usec, ip_offset):
                                     subdomains = _subdomains.get(domain)
 
                                     if not subdomains:
-                                        subdomains = _subdomains[domain] = set()
+                                        subdomains = _subdomains[domain] = _set()
+                                        subdomains._start = sec
 
                                     if not re.search(r"\A\d+\-\d+\-\d+\-\d+\Z", parts[0]):
-                                        if len(subdomains) < DNS_EXHAUSTION_THRESHOLD:
+                                        if sec - subdomains._start > 60:
+                                            subdomains._start = sec
+                                            subdomains.clear()
+                                        elif len(subdomains) < DNS_EXHAUSTION_THRESHOLD:
                                             subdomains.add('.'.join(parts[:-2]))
                                         else:
-                                            if (sec - (_last_dns_exhaustion or 0)) > 60:
-                                                trail = "(%s).%s" % ('.'.join(parts[:-2]), '.'.join(parts[-2:]))
-                                                if re.search(r"bl\b", trail) is None:                                               # generic check for DNSBLs
-                                                    if not any(_ in subdomains for _ in LOCAL_SUBDOMAIN_LOOKUPS):                   # generic check for local DNS resolutions
-                                                        log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.UDP, TRAIL.DNS, trail, "potential dns exhaustion (suspicious)", "(heuristic)"), packet)
-                                                        _dns_exhausted_domains.add(domain)
-                                                        _last_dns_exhaustion = sec
+                                            trail = "(%s).%s" % ('.'.join(parts[:-2]), '.'.join(parts[-2:]))
+                                            if re.search(r"bl\b", trail) is None:                                               # generic check for DNSBLs
+                                                if not any(_ in subdomains for _ in LOCAL_SUBDOMAIN_LOOKUPS):                   # generic check for local DNS resolutions
+                                                    log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.UDP, TRAIL.DNS, trail, "potential dns exhaustion (suspicious)", "(heuristic)"), packet)
+                                                    _dns_exhausted_domains.add(domain)
 
                                             return
 
@@ -816,7 +851,7 @@ def init():
 
     def update_timer():
         retries = 0
-        if not config.no_updates:
+        if not config.offline:
             while retries < CHECK_CONNECTION_MAX_RETRIES and not check_connection():
                 sys.stdout.write("[!] can't update because of lack of Internet connection (waiting..." if not retries else '.')
                 sys.stdout.flush()
@@ -826,7 +861,7 @@ def init():
             if retries:
                 print(")")
 
-        if config.no_updates or retries == CHECK_CONNECTION_MAX_RETRIES:
+        if config.offline or retries == CHECK_CONNECTION_MAX_RETRIES:
             if retries == CHECK_CONNECTION_MAX_RETRIES:
                 print("[x] going to continue without online update")
             _ = update_trails(offline=True)
@@ -847,7 +882,7 @@ def init():
                 if re.search(r"[\].][*+]|\[[a-z0-9_.\-]+\]", trail, re.I):
                     try:
                         re.compile(trail)
-                    except:
+                    except re.error:
                         pass
                     else:
                         if re.escape(trail) != trail:
@@ -864,8 +899,6 @@ def init():
     create_log_directory()
     get_error_log_handle()
 
-    check_memory()
-
     msg = "[i] using '%s' for trail storage" % config.TRAILS_FILE
     if os.path.isfile(config.TRAILS_FILE):
         mtime = time.gmtime(os.path.getmtime(config.TRAILS_FILE))
@@ -876,7 +909,7 @@ def init():
     update_timer()
 
     if not config.DISABLE_CHECK_SUDO and check_sudo() is False:
-        exit("[!] please run '%s' with sudo/Administrator privileges" % __file__)
+        exit("[!] please run '%s' with root privileges" % __file__)
 
     if config.plugins:
         config.plugin_functions = []
@@ -905,7 +938,7 @@ def init():
                     sys.path.insert(0, dirname)
 
                 try:
-                    module = __import__(filename[:-3].encode(sys.getfilesystemencoding()))
+                    module = __import__(filename[:-3])
                 except (ImportError, SyntaxError) as msg:
                     exit("[!] unable to import plugin script '%s' (%s)" % (filename, msg))
 
@@ -954,6 +987,15 @@ def init():
     if config.SYSLOG_SERVER and not len(config.SYSLOG_SERVER.split(':')) == 2:
         exit("[!] invalid configuration value for 'SYSLOG_SERVER' ('%s')" % config.SYSLOG_SERVER)
 
+    if config.LOGSTASH_SERVER and not len(config.LOGSTASH_SERVER.split(':')) == 2:
+        exit("[!] invalid configuration value for 'LOGSTASH_SERVER' ('%s')" % config.LOGSTASH_SERVER)
+
+    if config.REMOTE_SEVERITY_REGEX:
+        try:
+            re.compile(config.REMOTE_SEVERITY_REGEX)
+        except re.error:
+            exit("[!] invalid configuration value for 'REMOTE_SEVERITY_REGEX' ('%s')" % config.REMOTE_SEVERITY_REGEX)
+
     if config.CAPTURE_FILTER:
         print("[i] setting capture filter '%s'" % config.CAPTURE_FILTER)
         for _cap in _caps:
@@ -992,6 +1034,7 @@ def _init_multiprocessing():
     """
 
     global _buffer
+    global _multiprocessing
     global _n
 
     if _multiprocessing:
@@ -1008,20 +1051,25 @@ def _init_multiprocessing():
         except:
             exit("[!] unable to allocate network capture buffer. Please adjust value of 'CAPTURE_BUFFER'")
 
-        print("[i] creating %d more processes (out of total %d)" % (config.PROCESS_COUNT - 1, config.PROCESS_COUNT))
         _n = _multiprocessing.Value('L', lock=False)
 
-        for i in xrange(config.PROCESS_COUNT - 1):
-            process = _multiprocessing.Process(target=worker, name=str(i), args=(_buffer, _n, i, config.PROCESS_COUNT - 1, _process_packet))
-            process.daemon = True
-            process.start()
+        try:
+            for i in xrange(config.PROCESS_COUNT - 1):
+                process = _multiprocessing.Process(target=worker, name=str(i), args=(_buffer, _n, i, config.PROCESS_COUNT - 1, _process_packet))
+                process.daemon = True
+                process.start()
+        except TypeError:   # Note: https://github.com/stamparm/maltrail/issues/11823
+            _buffer = None
+            _multiprocessing = None
+        else:
+            print("[i] created %d more processes (out of total %d)" % (config.PROCESS_COUNT - 1, config.PROCESS_COUNT))
 
 def monitor():
     """
     Sniffs/monitors given capturing interface
     """
 
-    print("[o] running...")
+    print("[^] running...")
 
     def packet_handler(datalink, header, packet):
         global _count
@@ -1086,28 +1134,33 @@ def monitor():
 
             datalink = _cap.datalink()
 
-            if six.PY3 and not config.pcap_file:  # https://github.com/helpsystems/pcapy/issues/37#issuecomment-530795813
-                def _loop_handler(header, packet):
-                    packet_handler(datalink, header, packet)
 
-                _cap.loop(-1, _loop_handler)
-            else:
-                while True:
-                    success = False
-                    try:
-                        (header, packet) = _cap.next()
-                        if header is not None:
-                            success = True
-                            packet_handler(datalink, header, packet)
-                        elif config.pcap_file:
-                            with _done_lock:
-                                _done_count += 1
-                            break
-                    except (pcapy.PcapError, socket.timeout):
-                        pass
+#
+# NOTE: currently an issue with pcapy-png and loop()
+#
+#            if six.PY3 and not config.pcap_file:  # https://github.com/helpsystems/pcapy/issues/37#issuecomment-530795813
+#                def _loop_handler(header, packet):
+#                    packet_handler(datalink, header, packet)
+#
+#                _cap.loop(-1, _loop_handler)
+#            else:
 
-                    if not success:
-                        time.sleep(REGULAR_SENSOR_SLEEP_TIME)
+            while True:
+                success = False
+                try:
+                    (header, packet) = _cap.next()
+                    if header is not None:
+                        success = True
+                        packet_handler(datalink, header, packet)
+                    elif config.pcap_file:
+                        with _done_lock:
+                            _done_count += 1
+                        break
+                except (pcapy.PcapError, socket.timeout):
+                    pass
+
+                if not success:
+                    time.sleep(REGULAR_SENSOR_SLEEP_TIME)
 
         if config.profile and len(_caps) == 1:
             print("[=] will store profiling results to '%s'..." % config.profile)
@@ -1124,7 +1177,8 @@ def monitor():
             while _caps and not _done_count == (config.pcap_file or "").count(',') + 1:
                 time.sleep(1)
 
-        print("[i] all capturing interfaces closed")
+        if not config.pcap_file:
+            print("[i] all capturing interfaces closed")
     except SystemError as ex:
         if "error return without" in str(ex):
             print("\r[x] stopping (Ctrl-C pressed)")
@@ -1133,7 +1187,7 @@ def monitor():
     except KeyboardInterrupt:
         print("\r[x] stopping (Ctrl-C pressed)")
     finally:
-        print("\r[i] please wait...")
+        print("\r[i] cleaning up...")
 
         if _multiprocessing:
             try:
@@ -1161,19 +1215,27 @@ def main():
                 else:
                     break
 
-    print("%s (sensor) #v%s\n" % (NAME, VERSION))
+    print("%s (sensor) #v%s {%s}\n" % (NAME, VERSION, HOMEPAGE))
+
+    if "--version" in sys.argv:
+        raise SystemExit
 
     parser = optparse.OptionParser(version=VERSION)
     parser.add_option("-c", dest="config_file", default=CONFIG_FILE, help="configuration file (default: '%s')" % os.path.split(CONFIG_FILE)[-1])
-    parser.add_option("-i", dest="pcap_file", help="open pcap file for offline analysis")
+    parser.add_option("-r", dest="pcap_file", help="pcap file for offline analysis")
     parser.add_option("-p", dest="plugins", help="plugin(s) to be used per event")
-    parser.add_option("-q", dest="quiet", action="store_true", help="turn off regular output")
-    parser.add_option("--console", dest="console", action="store_true", help="print events to console (Note: switch '-q' might be useful)")
-    parser.add_option("--no-updates", dest="no_updates", action="store_true", help="disable (online) trail updates")
+    parser.add_option("-q", "--quiet", dest="quiet", action="store_true", help="turn off regular output")
+    parser.add_option("--console", dest="console", action="store_true", help="print events to console")
+    parser.add_option("--offline", dest="offline", action="store_true", help="disable (online) trail updates")
     parser.add_option("--debug", dest="debug", action="store_true", help=optparse.SUPPRESS_HELP)
     parser.add_option("--profile", dest="profile", help=optparse.SUPPRESS_HELP)
+
+    patch_parser(parser)
+
     options, _ = parser.parse_args()
 
+    print("[*] starting @ %s\n" % time.strftime("%X /%Y-%m-%d/"))
+
     read_config(options.config_file)
 
     for option in dir(options):
@@ -1196,7 +1258,7 @@ def main():
             print("[i] using pcap file(s) '%s'" % options.pcap_file)
 
     if not config.DISABLE_CHECK_SUDO and not check_sudo():
-        exit("[!] please run '%s' with sudo/Administrator privileges" % __file__)
+        exit("[!] please run '%s' with root privileges" % __file__)
 
     try:
         init()
@@ -1209,27 +1271,26 @@ def main():
         print("\r[x] stopping (Ctrl-C pressed)")
 
 if __name__ == "__main__":
-    show_final = True
+    code = 0
 
     try:
         main()
     except SystemExit as ex:
-        show_final = False
-
-        if isinstance(get_ex_message(ex), six.string_types):
+        if isinstance(get_ex_message(ex), six.string_types) and get_ex_message(ex).strip('0'):
             print(get_ex_message(ex))
-            os._exit(1)
+            code = 1
     except IOError:
-        show_final = False
         log_error("\n\n[!] session abruptly terminated\n[?] (hint: \"https://stackoverflow.com/a/20997655\")")
+        code = 1
     except Exception:
         msg = "\r[!] unhandled exception occurred ('%s')" % sys.exc_info()[1]
         msg += "\n[x] please report the following details at 'https://github.com/stamparm/maltrail/issues':\n---\n'%s'\n---" % traceback.format_exc()
         log_error("\n\n%s" % msg.replace("\r", ""))
 
         print(msg)
+        code = 1
     finally:
-        if show_final:
-            print("[i] finished")
+        if not any(_ in sys.argv for _ in ("--version", "-h", "--help")):
+            print("\n[*] ending @ %s" % time.strftime("%X /%Y-%m-%d/"))
 
-        os._exit(0)
+        os._exit(code)
