@@ -1,18 +1,17 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 
 from core.common import retrieve_content
 
-__url__ = "https://www.talosintelligence.com/feeds/ip-filter.blf"
+__url__ = "https://www.talosintelligence.com/documents/ip-blacklist"
 __check__ = ".1"
 __info__ = "bad reputation"
 __reference__ = "talosintelligence.com"
 
-# Reference: http://blog.snort.org/2015/09/ip-blacklist-feed-has-moved-locations.html
 def fetch():
     retval = {}
     content = retrieve_content(__url__)
