@@ -1,18 +1,16 @@
 --
--- (C) 2013-20 - ntop.org
+-- (C) 2013-21 - ntop.org
 --
 
 local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
+package.path = dirs.installdir .. "/scripts/lua/modules/timeseries/?.lua;" .. package.path
 
-require "lua_utils"
-require "alert_utils"
-
-local callback_utils = require "callback_utils"
-local db_utils = require "db_utils"
-local ts_utils = require "ts_utils"
+local scripts_triggers = require "scripts_triggers"
+local ts_utils         = require "ts_utils"
 local data_retention_utils = require "data_retention_utils"
-local user_scripts = require("user_scripts")
+
+local prefs            = ntop.getPrefs() or nil
 
 if(ntop.isPro()) then
    package.path = dirs.installdir .. "/pro/scripts/callbacks/interface/?.lua;" .. package.path
@@ -29,16 +27,22 @@ local _ifname = ifstats.name
 
 local interface_id = getInterfaceId(_ifname)
 
-user_scripts.runPeriodicScripts("day")
+-- Setting up periodic checks
+local k = string.format("ntopng.cache.ifid_%i.checks.request.granularity_day", interface.getId())
+ntop.setCache(k, "1")
 
 local data_retention = data_retention_utils.getDataRetentionDays()
 
-ntop.deleteMinuteStatsOlderThan(interface_id, data_retention)
+-- ###########################################
+
+if scripts_triggers.isDumbFlowToSQLEnabled(ifstats) then
+   local db_utils = require "db_utils"
 
-if ntop.getPrefs()["is_dump_flows_to_mysql_enabled"] and not ifstats.isViewed then
    local mysql_retention = os.time() - 86400 * data_retention
    db_utils.harverstExpiredMySQLFlows(_ifname, mysql_retention, verbose)
 end
 
+-- ###########################################
+
+ntop.deleteMinuteStatsOlderThan(interface_id, data_retention)
 ts_utils.deleteOldData(interface_id)
-optimizeAlerts()
