@@ -1,5 +1,5 @@
 --
--- (C) 2017-20 - ntop.org
+-- (C) 2017-21 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -10,12 +10,12 @@ local json = require("dkjson")
 
 sendHTTPHeader('application/json')
 
-if not haveAdminPrivileges() then
+if not isAdministratorOrPrintErr() then
   return
 end
 
 ntop.serviceRestart()
 
-res = { csrf = ntop.getRandomCSRFValue() }
+res = { }
 
 print(json.encode(res, nil, 1))
