@@ -1,5 +1,5 @@
 --
--- (C) 2013-20 - ntop.org
+-- (C) 2013-21 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -396,9 +396,7 @@ else
             interface.select(ifname)
             rtp_host = interface.getHostInfo(address_ip)
             if(rtp_host ~= nil) then
-              print('<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/host_details.lua?host='..address_ip.. '\\\">')
-              print(address_ip)
-              print('</A>')
+              print(hostinfo2detailshref(rtp_host, nil, address_ip))
             end
           else
             print(address_ip)
@@ -406,7 +404,7 @@ else
         end
         if((getFlowValue(info, "SIP_RTP_L4_SRC_PORT")~=nil) and (getFlowValue(info, "SIP_RTP_L4_SRC_PORT")~="") and (sip_rtp_src_addr == 1)) then
           --print(':'..getFlowValue(info, "SIP_RTP_L4_SRC_PORT"))
-          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/port_details.lua?port='..getFlowValue(info, "SIP_RTP_L4_SRC_PORT").. '\\\">')
+          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/flows_stats.lua?port='..getFlowValue(info, "SIP_RTP_L4_SRC_PORT").. '\\\">')
           print(getFlowValue(info, "SIP_RTP_L4_SRC_PORT"))
           print('</A>')
         end
@@ -416,13 +414,14 @@ else
         if((getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~="")) then
           sip_rtp_dst_addr = 1
           local address_ip = getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")
-          if (address_ip ~= "0.0.0.0") then
+
+	  if (address_ip ~= "0.0.0.0") then
             interface.select(ifname)
+	    
             rtp_host = interface.getHostInfo(address_ip)
+	    
             if(rtp_host ~= nil) then
-              print('<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/host_details.lua?host='..address_ip.. '\\\">')
-              print(address_ip)
-              print('</A>')
+	      print(hostinfo2detailshref(rtp_host, nil, address_ip))
             end
           else
             print(address_ip)
@@ -430,7 +429,7 @@ else
         end
         if((getFlowValue(info, "SIP_RTP_L4_DST_PORT")~=nil) and (getFlowValue(info, "SIP_RTP_L4_DST_PORT")~="") and (sip_rtp_dst_addr == 1)) then
           --print(':'..getFlowValue(info, "SIP_RTP_L4_DST_PORT"))
-          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/port_details.lua?port='..getFlowValue(info, "SIP_RTP_L4_DST_PORT").. '\\\">')
+          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/flows_stats.lua?port='..getFlowValue(info, "SIP_RTP_L4_DST_PORT").. '\\\">')
           print(getFlowValue(info, "SIP_RTP_L4_DST_PORT"))
           print('</A>')
         end
