@@ -1,5 +1,5 @@
 --
--- (C) 2013-20 - ntop.org
+-- (C) 2013-21 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -41,7 +41,7 @@ for key, value in pairs(hosts_stats) do
    if(num > 0) then print(",\n") else print("\n") end
 
    local hinfo = hostkey2hostinfo(key)
-   name = host2name(hinfo["host"], hinfo["vlan"])
+   name = hostinfo2label(hinfo)
 
    --if(res == nil) then res = "AAA" end
    print("{ \"name\": \"" .. res .. "\", \"size\": " .. (hosts_stats[key]["bytes.sent"]+hosts_stats[key]["bytes.rcvd"]).. "} ")
