@@ -1,5 +1,5 @@
 --
--- (C) 2013-20 - ntop.org
+-- (C) 2013-21 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -20,6 +20,22 @@ local custom_column_utils = require("custom_column_utils")
 local discover = require("discover_utils")
 local have_nedge = ntop.isnEdge()
 
+local function generate_map_url(map, map_url, query, icon)
+
+   local url = ""
+
+   if (ntop.isPro()) then
+
+      local map_available = table.len(map) > 0
+
+      if (map_available) then
+         url = "<a class='ms-1' href='"..ntop.getHttpPrefix().."/lua/pro/enterprise/"..map_url.."?" .. query .. "'><i class='".. icon .."'></i></a>"
+      end
+   end
+
+   return url
+end
+
 sendHTTPContentTypeHeader('text/html')
 
 
@@ -55,7 +71,7 @@ if ((mode ~= "all") or (not isEmptyString(pool))) then
    hosts_filter = '<span class="fas fa-filter"></span>'
 end
 
-function getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipver_title, os_, country, asninfo, mac, pool_, vlan_title)
+function getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipver_title, os_, country, asninfo, mac, pool_, vlan_title, vlan_alias)
    local mode_label = ""
 
    if mode == "remote" then
@@ -70,6 +86,14 @@ function getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipv
       mode_label = i18n("nedge.network_conf_dhcp")
    end
 
+   if(network == nil) then
+      wheel = ""
+      charts_icon = ""
+   else
+      wheel = '<A HREF="'.. ntop.getHttpPrefix().. '/lua/network_details.lua?network='.. network ..'&page=config' ..'"><i class="fas fa-cog fa-sm"></i></A>'
+      charts_icon = charts_icon.."&nbsp; <a href='".. ntop.getHttpPrefix() .."/lua/network_details.lua?network=".. network .. "&page=historical'><i class='fas fa-sm fa-chart-area'></i></a>"
+   end
+   
    -- Note: we must use the empty string as fallback. Multiple spaces will be collapsed into one automatically.
    return i18n("hosts_stats.hosts_page_title", {
         all = isEmptyString(mode_label) and i18n("hosts_stats.all") or "",
@@ -82,7 +106,10 @@ function getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipv
         ["os"] = discover.getOsName(os_),
         country_asn_or_mac = country or asninfo or mac or pool_ or "",
         vlan = vlan_title or "",
-   }) .. charts_icon
+        vlan_name = vlan_alias or "",
+        charts_icon = charts_icon,
+	wheel = wheel
+   })
 end
 
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
@@ -100,6 +127,7 @@ local country_title = nil
 local mac_title = nil
 local vlan_title = nil
 local pool_title = nil
+local vlan_alias = nil
 
 if((protocol ~= nil) and (protocol ~= "")) then
    protocol_name = interface.getnDPIProtoName(tonumber(protocol))
@@ -139,9 +167,10 @@ end
 
 if(asn ~= nil) then
    asninfo = " " .. i18n("hosts_stats.asn_title",{asn=asn}) ..
-      "<small>&nbsp;<i class='fas fa-info-circle fa-sm' aria-hidden='true'></i> <A HREF='https://stat.ripe.net/AS"..
+      "<small>&nbsp;<i class='fas fa-info-circle fa-sm' aria-hidden='true'></i> <A class='ntopng-external-link' href='https://stat.ripe.net/AS"..
       asn .. "'><i class='fas fa-external-link-alt fa-sm' title=\\\"".. i18n("hosts_stats.more_info_about_as_popup_msg") ..
-      "\\\"></i></A></small>"
+      "\\\"></i></A> "..charts_icon.."&nbsp; <a href='".. ntop.getHttpPrefix() .."/lua/as_details.lua?asn="..
+      asn .. "&page=historical'><i class='fas fa-sm fa-chart-area'></i></a> </small>"
 end
 
 if(os_ ~= nil) then
@@ -157,11 +186,29 @@ if(mac ~= nil) then
 end
 
 if(vlan ~= nil) then
-   vlan_title = " [".. i18n("hosts_stats.vlan_title", {vlan=vlan}) .."]"
+   
+   local link_service_map = generate_map_url(interface.serviceMap(nil, tonumber(vlan)), "service_map.lua", "vlan=" .. vlan, "fas fa-concierge-bell")
+   local link_periodicity_map = generate_map_url(interface.periodicityMap(nil, tonumber(vlan)), "periodicity_map.lua", "vlan=" .. vlan, "fas fa-clock")
+
+   vlan_title = " [".. i18n("hosts_stats.vlan_title", {vlan=vlan}) .."] <A HREF='".. ntop.getHttpPrefix().. "/lua/vlan_details.lua?vlan=".. vlan .."&page=config" .."'><i class='fas fa-cog fa-sm'></i></A>" .. link_service_map .. " " .. link_periodicity_map
+   if(vlan==getVlanAlias(vlan)) then
+      vlan_alias=""
+   else
+      vlan_alias = getVlanAlias(vlan)
+   end
 end
 
 if(pool ~= nil) then
+
+   local link_service_map = ""
+   local link_periodicity_map = ""
    local charts_available = areHostPoolsTimeseriesEnabled(ifstats.id)
+
+   if (tonumber(pool) ~= host_pools_instance.DEFAULT_POOL_ID) then
+      link_service_map = generate_map_url(interface.serviceMap(nil, nil, tonumber(pool)), "service_map.lua", "host_pool_id=".. pool, "fas fa-concierge-bell")
+      link_periodicity_map = generate_map_url(interface.periodicityMap(nil, nil, tonumber(pool)), "periodicity_map.lua", "host_pool_id=".. pool, "fas fa-clock")
+   end
+
    local pool_edit = ""
    local pool_link
    local title
@@ -182,35 +229,41 @@ if(pool ~= nil) then
    pool_title = " "..i18n(ternary(have_nedge, "hosts_stats.user_title", "hosts_stats.pool_title"),
 		     {poolname=host_pools_instance:get_pool_name(pool)})
       .."<small>".. pool_edit ..
-      ternary(charts_available, "&nbsp; <A HREF='"..ntop.getHttpPrefix().."/lua/pool_details.lua?page=historical&pool="..pool.."'><i class='fas fa-chart-area fa-sm' title='"..i18n("chart") .. "'></i></A>", "")..
+      ternary(charts_available, " <a href='"..ntop.getHttpPrefix().."/lua/pool_details.lua?page=historical&pool="..pool.."'><i class='fas fa-chart-area fa-sm' title='"..i18n("chart") .. "'></i></a>", "")
+      ..link_service_map .. link_periodicity_map..
       "</small>"
 end
 
-page_utils.print_page_title(getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipver_title, os_title, country_title, asninfo, mac_title, pool_title, vlan_title))
+page_utils.print_page_title(getPageTitle(protocol_name, traffic_type_title, network_name, cidr, ipver_title, os_title, country_title, asninfo, mac_title, pool_title, vlan_title,vlan_alias))
 
 if (_GET["page"] ~= "historical") then
    if(asn ~= nil) then
       print [[
-<div class="container-fluid">
-  <ul class="nav nav-tabs">
-    <li class="nav-item" class="active"><a class="nav-link active" data-toggle="tab" href="#home">]] print(i18n("hosts_stats.hosts")) print[[</a></li>
+   <div class='card'><div class='card-header'>
+  <ul class="nav nav-tabs card-header-tabs">
+    <li class="nav-item" class="active"><a class="nav-link active" data-bs-toggle="tab" href="#home">]] print(i18n("hosts_stats.hosts")) print[[</a></li>
 ]]
 
       if(asn ~= "0") then
 	 print [[
-    <li class="nav-item"><a class="nav-link" data-toggle="tab" href="#asinfo">]] print(i18n("hosts_stats.as_info")) print[[</a></li>
-    <li class="nav-item"><a class="nav-link" data-toggle="tab" href="#aspath">]] print(i18n("hosts_stats.as_path")) print[[</a></li>
-    <li class="nav-item"><a class="nav-link" data-toggle="tab" href="#geoloc">]] print(i18n("hosts_stats.as_geolocation")) print[[</a></li>
-    <li class="nav-item"><a class="nav-link" data-toggle="tab" href="#prefix">]] print(i18n("hosts_stats.as_prefixes")) print[[</a></li>
-    <li class="nav-item"><a class="nav-link" data-toggle="tab" href="#bgp">]] print(i18n("hosts_stats.bgp_updates")) print[[</a></li>
+    <li class="nav-item"><a class="nav-link" data-bs-toggle="tab" href="#asinfo">]] print(i18n("hosts_stats.as_info")) print[[</a></li>
+    <li class="nav-item"><a class="nav-link" data-bs-toggle="tab" href="#aspath">]] print(i18n("hosts_stats.as_path")) print[[</a></li>
+    <li class="nav-item"><a class="nav-link" data-bs-toggle="tab" href="#geoloc">]] print(i18n("hosts_stats.as_geolocation")) print[[</a></li>
+    <li class="nav-item"><a class="nav-link" data-bs-toggle="tab" href="#prefix">]] print(i18n("hosts_stats.as_prefixes")) print[[</a></li>
+    <li class="nav-item"><a class="nav-link" data-bs-toggle="tab" href="#bgp">]] print(i18n("hosts_stats.bgp_updates")) print[[</a></li>
 ]]
       end
    end
 
-   print("</ul>")
+   print("</ul>") -- close .card-header
 
+   if (asn ~= nil) then
+      print("</div>")
+   end
+   
    if(asn ~= nil) then
       print [[
+   <div class='card-body'>
   <div class="tab-content">
 <div id="home" class="tab-pane in active">
 ]]
@@ -264,7 +317,7 @@ if (_GET["page"] ~= "historical") then
    local custom_name, custom_key, custom_align = custom_column_utils.getCustomColumnName()
 
    print [[
-      <div id="table-hosts"></div>
+      <div class='table-responsive'><div id="table-hosts"></div></div>
 	 <script>
 	 var url_update = "]] print(getPageUrl(ntop.getHttpPrefix() .. "/lua/get_hosts_data.lua", page_params)) print[[";]]
 
@@ -336,7 +389,7 @@ if (_GET["page"] ~= "historical") then
    -- table.clone needed to modify some parameters while keeping the original unchanged
    local hosts_filter_params = table.clone(page_params)
 
-   print(', \'<div class="btn-group"><button class="btn btn-link dropdown-toggle" data-toggle="dropdown">'..i18n("hosts_stats.filter_hosts")..hosts_filter..'<span class="caret"></span></button> <ul class="dropdown-menu scrollable-dropdown" role="menu" style="min-width: 90px;"><li"><a class="dropdown-item" href="')
+   print(', \'<div class="btn-group"><button class="btn btn-link dropdown-toggle" data-bs-toggle="dropdown">'..i18n("hosts_stats.filter_hosts")..hosts_filter..'<span class="caret"></span></button> <ul class="dropdown-menu scrollable-dropdown" role="menu" style="min-width: 90px;"><li"><a class="dropdown-item" href="')
 
    hosts_filter_params.mode = nil
    hosts_filter_params.pool = nil
@@ -443,15 +496,6 @@ if (_GET["page"] ~= "historical") then
 
    print [[
 			     {
-			     title: "]] print(i18n("hosts_stats.location")) print[[",
-				 field: "column_location",
-				 sortable: false,
-	 	             css: {
-			        textAlign: 'center'
-			     }
-
-				 },
-			     {
 			     title: "]] print(i18n("flows")) print[[",
 				 field: "column_num_flows",
 				 sortable: true,
@@ -574,6 +618,7 @@ if (_GET["page"] ~= "historical") then
       print [[
 </div>
 
+
 <script src="/js/ripe_widget_api.js"></script>
 
 <div id="asinfo" class="tab-pane"></div>
@@ -581,9 +626,15 @@ if (_GET["page"] ~= "historical") then
 <div id="geoloc" class="tab-pane"></div>
 <div id="prefix" class="tab-pane"></div>
 <div id="bgp" class="tab-pane"></div>
+]]
 
-</div>
+if (asn ~= nil) then
+   print("</div>")
+   print("</div>")
+   print("</div>")
+end
 
+print[[
 <script>
    $(document).ready(function() {
       var tab_id_to_widget = {
@@ -611,7 +662,7 @@ if (_GET["page"] ~= "historical") then
          loaded_widgets[widget] = true;
       }
 
-      $('a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
+      $('a[data-bs-toggle="tab"]').on('shown.bs.tab', function (e) {
          var target = $(e.target).attr("href") // activated tab
          load_widget(target);
       });
