@@ -1,5 +1,5 @@
 --
--- (C) 2017-20 - ntop.org
+-- (C) 2017-21 - ntop.org
 --
 
 local json = require "dkjson"
@@ -379,7 +379,7 @@ end
 -- ################################################################################
 
 function discover.printDeviceTypeSelector(device_type, field_name)
-   print [[<div class="form-group"><select name="]] print(field_name) print[[" class="form-control">\
+   print [[<div class="form-group mb-3"><select name="]] print(field_name) print[[" class="form-select">\
    <option value="0"></option>]]
 
    discover.printDeviceTypeSelectorOptions(device_type, true)
@@ -1176,7 +1176,7 @@ local function discoverARP()
       status = discoverStatus("ERROR", i18n("discover.err_unable_to_arp_discovery"))
    else
       -- Add the known macs to the list
-      local known_macs = interface.getMacsInfo(nil, 999, 0, false, true, nil) or {}
+      local known_macs = interface.getMacsInfo(nil, 999, 0, false, true, nil, nil, nil, nil, nil) or {}
 
       for _,hmac in pairs(known_macs.macs) do
 	 if((hmac["bytes.sent"] > 0)
