@@ -1,12 +1,11 @@
 --
--- (C) 2014-20 - ntop.org
+-- (C) 2014-21 - ntop.org
 --
 
 require "lua_utils"
-require "flow_aggregation_utils"
 
 local json = require ("dkjson")
-
+local graph_common = require "graph_common"
 local favourites_url = ntop.getHttpPrefix().."/lua/get_historical_favourites.lua"
 local flows_download_url = ntop.getHttpPrefix().."/lua/get_db_flows.lua"
 
@@ -94,8 +93,8 @@ function buildRequestData(source_div_id){
 var zoom_vals = {
 ]]
 
-if zoom_vals ~= nil then
-  for _, zv in pairs(zoom_vals) do
+if graph_common.zoom_vals ~= nil then
+  for _, zv in pairs(graph_common.zoom_vals) do
      print('"'..zv[1]..'": '..zv[3]..', ')
   end
 end
@@ -223,9 +222,6 @@ function printFlowsCountColumn()
    -- hides the flows count column when aggregated database flows are being used
    local col = '{title: "'..i18n("flows")
    local hide_count = ""
-   if useAggregatedFlows() == true then
-      hide_count = " hidden :true, "
-   end
    col = col..'", field: "column_flows", '..hide_count..' sortable: true, css: {textAlign:\'right\'}}'
    return col
 end
@@ -380,9 +376,9 @@ var populateInterfaceTopTalkersTable = function(){
 
 
   if ($('#historical-interface-top-talkers-table').attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $('#historical-interface-top-talkers-table').attr("loaded", 1);
     $('#historical-interface-top-talkers-table').datatable({
 	title: "",]]
@@ -394,7 +390,7 @@ var populateInterfaceTopTalkersTable = function(){
 	post: {totalRows: function(){ return $('#historical-interface-top-talkers-table').attr("total_rows");} },
 	showFilter: true,
 	showPagination: true,
-	tableCallback: function(){$('#historical-interface-top-talkers-table').attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+	tableCallback: function(){$('#historical-interface-top-talkers-table').attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	rowCallback: function(row){
 	  var addr_td = $("td:eq(1)", row[0]);
 	  var label_td = $("td:eq(0)", row[0]);
@@ -436,9 +432,9 @@ var populateHostTopTalkersTable = function(host){
   div_id='#'+div_id;
 
   if ($(div_id).attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $(div_id).attr("loaded", 1);
     $(div_id).attr("host", host);
     $(div_id).datatable({
@@ -451,7 +447,7 @@ var populateHostTopTalkersTable = function(host){
 	post: {totalRows: function(){ return $(div_id).attr("total_rows");} },
 	showFilter: true,
 	showPagination: true,
-	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	rowCallback: function(row){
 	  var addr_td = $("td:eq(1)", row[0]);
 	  var label_td = $("td:eq(0)", row[0]);
@@ -500,9 +496,9 @@ var populateAppsPerHostsPairTable = function(peer1, peer2){
 
   // if the table has already been loaded, we just show up all the dropdowns
   if ($(div_id).attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {   // load the table only if it is the first time we've been called
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $(div_id).attr("loaded", 1);
     $(div_id).attr("peer1", peer1);
     $(div_id).attr("peer2", peer2);
@@ -516,7 +512,7 @@ var populateAppsPerHostsPairTable = function(peer1, peer2){
 	post: {totalRows: function(){ return $(div_id).attr("total_rows");} },
 	showFilter: true,
 	showPagination: true,
-	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	rowCallback: function(row){
 	  var l7_proto_id_td = $("td:eq(0)", row[0]);
 	  var label_td = $("td:eq(1)", row[0]);
@@ -568,9 +564,9 @@ var populateFlowsPerHostsPairTable = function(peer1, peer2, l7_proto_id, num_flo
 
   // if the table has already been loaded, we just show up all the dropdowns
   if ($(div_id).attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {   // load the table only if it is the first time we've been called
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $(div_id).attr("loaded", 1);
     $(div_id).attr("peer1", peer1);
     $(div_id).attr("peer2", peer2);
@@ -579,7 +575,7 @@ var populateFlowsPerHostsPairTable = function(peer1, peer2, l7_proto_id, num_flo
 	title: "",]]
 	print("url: '"..ntop.getHttpPrefix().."/lua/get_db_flows.lua?ifid="..tostring(ifId)..interface_talkers_url_params.."&peer1=' + peer1 + '&peer2=' + peer2 + '&l7_proto_id=' + l7_proto_id")
 
-	if not useAggregatedFlows() and not allowedNetworksRestrictions() then
+	if not allowedNetworksRestrictions() then
 	   -- speed up by passing the number of flows that is already calculated when browsing raw flows
 	   print("+ '&limit=' + num_flows")
 	end
@@ -593,7 +589,7 @@ var populateFlowsPerHostsPairTable = function(peer1, peer2, l7_proto_id, num_flo
 	showPagination: true,
 totalRows: 100,
 	sort: [ [ "BYTES","desc"] ],
-	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	columns:
 	[
 	  {title: "]] print(i18n("key")) print[[",         field: "idx",            hidden: true},
@@ -616,7 +612,7 @@ totalRows: 100,
 $('a[href="#historical-top-talkers"]').on('shown.bs.tab', function (e) {
   if ($('a[href="#historical-top-talkers"]').attr("loaded") == 1){
     // do nothing if the tabs have already been computed and populated
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
     return;
   }
 
@@ -663,7 +659,7 @@ function historicalTopApplicationsTable(ifid, epoch_begin, epoch_end, host, vlan
 
    top_apps_url_params = top_apps_url_params.."&epoch_begin="..epoch_begin
    top_apps_url_params = top_apps_url_params.."&epoch_end="..epoch_end
-   
+
    if vlan ~= "" and vlan ~= nil and tonumber(vlan) ~= nil then
       top_apps_url_params = top_apps_url_params.."&vlan="..tostring(vlan)
       container_params = container_params..' vlan="'..(tostring(vlan) or '')..'"'
@@ -773,9 +769,9 @@ var populateInterfaceTopAppsTable = function(){
   showOne('historical-interface-apps', 'historical-interface-top-apps-table');
 
   if ($('#historical-interface-top-apps-table').attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $('#historical-interface-top-apps-table').attr("loaded", 1);
     $('#historical-interface-top-apps-table').datatable({
       title: "",]]
@@ -787,7 +783,7 @@ print [[
       post: {totalRows: function(){ return $('#historical-interface-top-apps-table').attr("total_rows");} },
       showFilter: true,
       showPagination: true,
-      tableCallback: function(){$('#historical-interface-top-apps-table').attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+      tableCallback: function(){$('#historical-interface-top-apps-table').attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
       rowCallback: function(row){
 	var proto_id_td = $("td:eq(0)", row[0]);
 	var proto_label_td = $("td:eq(1)", row[0]);
@@ -838,9 +834,9 @@ var populateAppTopTalkersTable = function(proto_id){
   div_id='#'+div_id;
 
   if ($(div_id).attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $(div_id).attr("loaded", 1);
     $(div_id).attr("l7_proto", proto_id);
     $(div_id).datatable({
@@ -853,7 +849,7 @@ var populateAppTopTalkersTable = function(proto_id){
 	post: {totalRows: function(){ return $(div_id).attr("total_rows");} },
 	showFilter: true,
 	showPagination: true,
-	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	rowCallback: function(row){
 	  var addr_td = $("td:eq(1)", row[0]);
 	  var label_td = $("td:eq(0)", row[0]);
@@ -897,9 +893,9 @@ var populatePeersPerHostByApplication = function(host, proto_id){
   div_id='#'+div_id;
 
   if ($(div_id).attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $(div_id).attr("loaded", 1);
     $(div_id).attr("l7_proto", proto_id);
     $(div_id).attr("host", host);
@@ -913,7 +909,7 @@ var populatePeersPerHostByApplication = function(host, proto_id){
 	post: {totalRows: function(){ return $(div_id).attr("total_rows");} },
 	showFilter: true,
 	showPagination: true,
-	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	rowCallback: function(row){
 	  var addr_td = $("td:eq(1)", row[0]);
 	  var label_td = $("td:eq(0)", row[0]);
@@ -966,9 +962,9 @@ var populateFlowsPerHostPairByApplicationTable = function(peer1, peer2, l7_proto
 
   // if the table has already been loaded, we just show up all the dropdowns
   if ($(div_id).attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {   // load the table only if it is the first time we've been called
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $(div_id).attr("loaded", 1);
     $(div_id).attr("peer1", peer1);
     $(div_id).attr("peer2", peer2);
@@ -976,8 +972,8 @@ var populateFlowsPerHostPairByApplicationTable = function(peer1, peer2, l7_proto
     $(div_id).datatable({
 	title: "",]]
 	print("url: '"..ntop.getHttpPrefix().."/lua/get_db_flows.lua?ifid="..tostring(ifId)..top_apps_url_params.."&peer1=' + peer1 + '&peer2=' + peer2 + '&l7_proto_id=' + l7_proto_id")
-	
-	if not useAggregatedFlows() and not allowedNetworksRestrictions() then
+
+	if not allowedNetworksRestrictions() then
 	   -- speed up by passing the number of flows that is already calculated when browsing raw flows
 	   print("+ '&limit=' + num_flows")
 	end
@@ -992,7 +988,7 @@ var populateFlowsPerHostPairByApplicationTable = function(peer1, peer2, l7_proto
 	showPagination: true,
 totalRows: 100,
 	sort: [ [ "BYTES","desc"] ],
-	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+	tableCallback: function(){$(div_id).attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	columns:
 	[
 	  {title: "]] print(i18n("key")) print[[",         field: "idx",            hidden: true},
@@ -1029,9 +1025,9 @@ var populateHostTopAppsTable = function(host){
   showOne('historical-interface-apps', 'historical-interface-top-apps-table');
 
   if ($('#historical-interface-top-apps-table').attr("loaded") == 1) {
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
   } else {
-    disableAllDropdownsAndTabs();
+    NtopUtils.disableAllDropdownsAndTabs();
     $('#historical-interface-top-apps-table').attr("loaded", 1);
     $('#historical-interface-top-apps-table').attr("host", host);
     $('#historical-interface-top-apps-table').datatable({
@@ -1044,7 +1040,7 @@ print [[
       post: {totalRows: function(){ return $('#historical-interface-top-apps-table').attr("total_rows");} },
       showFilter: true,
       showPagination: true,
-      tableCallback: function(){$('#historical-interface-top-apps-table').attr("total_rows", this.options.totalRows);enableAllDropdownsAndTabs();},
+      tableCallback: function(){$('#historical-interface-top-apps-table').attr("total_rows", this.options.totalRows);NtopUtils.enableAllDropdownsAndTabs();},
 	rowCallback: function(row){
 	var proto_id_td = $("td:eq(0)", row[0]);
 	var proto_label_td = $("td:eq(1)", row[0]);
@@ -1107,7 +1103,7 @@ adapts the breadcrumb depending on the page.
 */
 $('a[href="#historical-top-apps"]').on('shown.bs.tab', function (e) {
   if ($('a[href="#historical-top-apps"]').attr("loaded") == 1){
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
     // do nothing if the tabs have already been computed and populated
     return;
   }
@@ -1194,7 +1190,7 @@ function historicalFlowsTab(ifId, host, epoch_begin, epoch_end, l7proto, l4proto
 <br>
 <div class="container-fluid" id="historical-flows-container">
   <ul class="nav nav-tabs" role="tablist">
-    <li class="nav-item active"> <a class="nav-link active" href="#historical-flows-summary" role="tab" data-toggle="tab"> ]] print(i18n("db_explorer.summary")) print[[ </a> </li>
+    <li class="nav-item active"> <a class="nav-link active" href="#historical-flows-summary" role="tab" data-bs-toggle="tab"> ]] print(i18n("db_explorer.summary")) print[[ </a> </li>
 ]]
 
    print '<li class="nav-item" id="tab-ipv4-li" style="display: none;"> <a class="nav-link" href="#tab-ipv4" role="tab"> ' print(i18n("ipv4")) print' </a> </li>'
@@ -1221,9 +1217,7 @@ print [[
            <tr>
              <th>&nbsp;</th>]]
 
-if not useAggregatedFlows() then -- pointless to show counters for aggregations to the user
    print[[<th>]] print(i18n("db_explorer.total_flows"))print[[</th>]]
-end
 
 print[[<th>]] print(i18n("db_explorer.traffic_volume")) print[[</th>
              <th>]] print(i18n("db_explorer.total_packets")) print[[</th><th>]] print(i18n("db_explorer.traffic_rate")) print[[</th><th>]] print(i18n("db_explorer.packet_rate")) print[[</th>
@@ -1282,7 +1276,7 @@ var abortQuery = function(){
 
 $('a[href="#historical-flows"]').on('shown.bs.tab', function (e) {
   if ($('a[href="#historical-flows"]').attr("loaded") == 1){
-    enableAllDropdownsAndTabs();
+    NtopUtils.enableAllDropdownsAndTabs();
     // do nothing if the tabs have already been computed and populated
     return;
   }
@@ -1291,7 +1285,7 @@ $('a[href="#historical-flows"]').on('shown.bs.tab', function (e) {
   $('a[href="#historical-flows"]').attr("loaded", 1);
 
   // disable all tabs
-  $("#historical-flows-container").find("li").addClass("disabled").find("a").removeAttr("data-toggle");
+  $("#historical-flows-container").find("li").addClass("disabled").find("a").removeAttr("data-bs-toggle");
 
   xhr = $.ajax({
     type: 'GET',]]
@@ -1326,7 +1320,7 @@ print[[
       }
 
       // re-enable all tabs
-      $("#historical-flows-container").find("li").removeClass("disabled").find("a").attr("data-toggle", "tab");
+      $("#historical-flows-container").find("li").removeClass("disabled").find("a").attr("data-bs-toggle", "tab");
 
       // populate the number of flows
       $("#tab-ipv4").attr("num_flows", msg.count.IPv4.tot_flows)
@@ -1349,23 +1343,18 @@ print[[
         tr += "<tr><th>" + ipvers + "</th>"
 ]]
 
-if not useAggregatedFlows() then -- only show flow counters when querying from raw flows
 print[[
         tr += "<td align='right'>" + item.tot_flows + " Flows</td>"
 ]]
-end
 
 print[[
-        tr += "<td align='right'>" + bytesToVolume(item.tot_bytes) + "</td>"
-        tr += "<td align='right'>" + formatPackets(item.tot_packets) + "</td>"
-        tr += "<td align='right'>" + fbits(item.tot_bytes * 8 / msg.timespan) + "</td>"
-        tr += "<td align='right'>" + fpackets(item.tot_packets / msg.timespan) + "</td>"
+        tr += "<td align='right'>" + NtopUtils.bytesToVolume(item.tot_bytes) + "</td>"
+        tr += "<td align='right'>" + NtopUtils.formatPackets(item.tot_packets) + "</td>"
+        tr += "<td align='right'>" + NtopUtils.fbits(item.tot_bytes * 8 / msg.timespan) + "</td>"
+        tr += "<td align='right'>" + NtopUtils.fpackets(item.tot_packets / msg.timespan) + "</td>"
         tr += "</tr>"
       });
 
-//      if(msg.aggregated_flows) {
-//        $("#results-from-aggregated-flows").html("]] print(i18n("flow_search_from_aggregated")) print[[");
-//      }
       $("#flows-summary-table").append(tr)
       $("#historical-flows-summary-body").remove()
       $("#flows-summary-table").show();
@@ -1430,12 +1419,12 @@ print [[
       $('a[href="#tab-ipv4"]').on('shown.bs.tab', function (e) {
         if ($('a[href="#tab-ipv4"]').attr("loaded") == 1){
           // do nothing if the tab has already been computed and populated
-          enableAllDropdownsAndTabs();
+          NtopUtils.enableAllDropdownsAndTabs();
           return;
         }
 
         // if here, then we actually have to load the datatable
-        disableAllDropdownsAndTabs();
+        NtopUtils.disableAllDropdownsAndTabs();
         $('a[href="#tab-ipv4"]').attr("loaded", 1);
 
    ]]
@@ -1443,7 +1432,7 @@ print [[
 print [[
   var url_update4 = "]] print(url_update) print [[&version=4]]
 
-if not useAggregatedFlows() and not allowedNetworksRestrictions() then
+if not allowedNetworksRestrictions() then
    print[[&limit=" + $("#tab-ipv4").attr("num_flows")]]
 else
    -- limit computed dynamically
@@ -1460,7 +1449,7 @@ print[[ ;
 print [[
 						    showFilter: true,
 						    showPagination: true,
-                                                    tableCallback: function(){enableAllDropdownsAndTabs();},
+                                                    tableCallback: function(){NtopUtils.enableAllDropdownsAndTabs();},
 						    sort: [ [ "BYTES","desc"] ],
 						    columns: [
 						       {
@@ -1583,18 +1572,18 @@ print [[
       $('a[href="#tab-ipv6"]').on('shown.bs.tab', function (e) {
         if ($('a[href="#tab-ipv6"]').attr("loaded") == 1){
           // do nothing if the tab has already been computed and populated
-          enableAllDropdownsAndTabs();
+          NtopUtils.enableAllDropdownsAndTabs();
           return;
         }
 
         // if here, then we actually have to load the datatable
-        disableAllDropdownsAndTabs();
+        NtopUtils.disableAllDropdownsAndTabs();
         $('a[href="#tab-ipv6"]').attr("loaded", 1);
 
 
 	    var url_update6 = "]] print(url_update) print [[&version=6]]
 
-if not useAggregatedFlows() and not allowedNetworksRestrictions() then
+if not allowedNetworksRestrictions() then
    print[[&limit=" + $("#tab-ipv6").attr("num_flows")]]
 else
    -- limit computed dynamically
@@ -1613,7 +1602,7 @@ print [[
 
 						    showFilter: true,
 						    showPagination: true,
-                                                    tableCallback: function(){enableAllDropdownsAndTabs();},
+                                                    tableCallback: function(){NtopUtils.enableAllDropdownsAndTabs();},
 						    sort: [ [ "BYTES","desc"] ],
 						    columns: [
 						       {
@@ -1723,24 +1712,3 @@ print [[
 
 end
 
--- ##########################################
-
-function historicalProtoHostHref(ifId, host, l4_proto, ndpi_proto_id, info)
-   if ntop.isPro() and ntop.getPrefs().is_dump_flows_to_mysql_enabled == true then
-      local hist_url = ntop.getHttpPrefix().."/lua/pro/db_explorer.lua?search=true&ifid="..ifId
-      local now    = os.time()
-      local ago1h  = now - 3600
-
-      hist_url = hist_url.."&epoch_end="..tostring(now)
-      if((host ~= nil) and (host ~= "")) then hist_url = hist_url.."&"..hostinfo2url(host) end
-      if((l4_proto ~= nil) and (l4_proto ~= "")) then
-	 hist_url = hist_url.."&l4proto="..l4_proto
-      end
-      if((ndpi_proto_id ~= nil) and (ndpi_proto_id ~= "")) then hist_url = hist_url.."&protocol="..ndpi_proto_id end
-      if((info ~= nil) and (info ~= "")) then hist_url = hist_url.."&info="..info end
-      print('&nbsp;')
-      -- print('<span class="badge badge-info">')
-      print('<a href="'..hist_url..'&epoch_begin='..tostring(ago1h)..'" title="'..i18n("db_explorer.last_hour_flows")..'"><i class="fas fa-history fa-lg"></i></a>')
-      -- print('</span>')
-   end
-end
