@@ -1,14 +1,16 @@
 --
--- (C) 2019-20 - ntop.org
+-- (C) 2019-21 - ntop.org
 --
 
 local lists_utils = {}
 
 local dirs = ntop.getDirs()
+package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 local os_utils = require("os_utils")
 local categories_utils = require("categories_utils")
 local json = require("dkjson")
 local alerts_api = require("alerts_api")
+local alert_severities = require "alert_severities"
 local alert_consts = require "alert_consts"
 
 -- ##############################################
@@ -68,8 +70,8 @@ local BUILTIN_LISTS = {
       format = "ip",
       enabled = true,
       update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["Snort IP Blacklist"] = {
-      url = "https://snort.org/documents/ip-blacklist",
+   }, ["Snort IP Block List"] = {
+      url = "https://snort.org/downloads/ip-block-list",
       category = CUSTOM_CATEGORY_MALWARE,
       format = "ip",
       enabled = true,
@@ -98,18 +100,18 @@ local BUILTIN_LISTS = {
       format = "hosts",
       enabled = true,
       update_interval = DEFAULT_UPDATE_INTERVAL,
+   }, ["Abuse.ch URLhaus"] = {
+      url = "https://urlhaus.abuse.ch/downloads/hostfile/",
+      category = CUSTOM_CATEGORY_MALWARE,
+      format = "hosts",
+      enabled = true,
+      update_interval = DEFAULT_UPDATE_INTERVAL,
    }, ["Disconnect.me Simple Ad List"] = {
       url = "https://s3.amazonaws.com/lists.disconnect.me/simple_ad.txt",
       category = CUSTOM_CATEGORY_ADVERTISEMENT,
       format = "domain",
       enabled = is_nedge,
       update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["hpHosts Ad and Tracking"] = {
-      url = "https://hosts-file.net/ad_servers.txt",
-      category = CUSTOM_CATEGORY_ADVERTISEMENT,
-      format = "hosts",
-      enabled = is_nedge,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
    }, ["AdAway default blocklist"] = {
       url = "https://adaway.org/hosts.txt",
       category = CUSTOM_CATEGORY_ADVERTISEMENT,
@@ -167,7 +169,7 @@ local function saveListsStatusToRedis(lists, caller)
    for list_name, list in pairs(lists or {}) do
       status[list_name] = list.status
    end
-   
+
    ntop.setPref(STATUS_KEY, json.encode(status))
 end
 
@@ -247,7 +249,7 @@ end
 -- ##############################################
 
 -- Force a single list reload
-function lists_utils.updateList(list_name)   
+function lists_utils.updateList(list_name)
    ntop.setCache("ntopng.cache.category_lists.update." .. list_name, "1")
    lists_utils.downloadLists()
 end
@@ -279,11 +281,13 @@ local function getNextListUpdate(list)
    else
       interval = list.update_interval
    end
-   
+
    local next_update
-   
+
    -- align if possible
-   if interval == 3600 then
+   if interval == 0 then
+      next_update = -1
+   elseif interval == 3600 then
       next_update = ntop.roundTime(list.status.last_update, 3600, false)
    elseif interval == 86400 then
       next_update = ntop.roundTime(list.status.last_update, 86400, true --[[ UTC align ]])
@@ -299,14 +303,19 @@ end
 function shouldUpdate(list_name, list, now)
    local list_file
    local next_update
-   
+
    if(list.enabled == false) then
       return(false)
    end
-   
+
    list_file = getListCacheFile(list_name, false)
    next_update = getNextListUpdate(list, now)
 
+   if next_update == -1 then
+      return ((not ntop.exists(list_file) and (list.status.num_errors < MAX_LIST_ERRORS)) or
+               (ntop.getCache("ntopng.cache.category_lists.update." .. list_name) == "1"))
+   end
+
    if(false) then
       tprint('---------------')
       tprint(list_file)
@@ -315,7 +324,7 @@ function shouldUpdate(list_name, list, now)
       tprint('-')
       tprint(list)
       tprint('---------------')
-      
+
       tprint(((now >= next_update) or
 	       (not ntop.exists(list_file) and (list.status.num_errors < MAX_LIST_ERRORS)) or
 	       (ntop.getCache("ntopng.cache.category_lists.update." .. list_name) == "1")))
@@ -352,7 +361,7 @@ local function checkListsUpdate(timeout)
 	 local msg = string.format("Updating list '%s' [%s]... ", list_name, list.url)
 
 	 traceError(trace_level, TRACE_INFO, string.format("Updating list '%s'... ", list_name))
-	 
+
 	 local started_at = os.time()
 	 local res = ntop.httpFetch(list.url, temp_fname, timeout)
 
@@ -363,14 +372,14 @@ local function checkListsUpdate(timeout)
 	    list.status.num_errors = 0
 	    needs_reload = true
 
-	    alerts_api.store(
-	       alerts_api.categoryListsEntity(list_name),
-	       alert_consts.alert_types.alert_list_download_succeeded.create(
-		  alert_consts.alert_severities.notice,
-		  list_name
-	       )
+	    local alert = alert_consts.alert_types.alert_list_download_succeeded.new(
+	       list_name
 	    )
 
+	    alert:set_score_notice()
+
+	    alert:store(alerts_api.systemEntity(list_name))
+
 	    msg = msg .. "OK"
 	 else
 	    -- failure
@@ -396,15 +405,15 @@ local function checkListsUpdate(timeout)
 	    list.status.last_error = last_error
 	    list.status.num_errors = list.status.num_errors + 1
 
-	    alerts_api.store(
-	       alerts_api.categoryListsEntity(list_name),
-	       alert_consts.alert_types.alert_list_download_failed.create(
-		  alert_consts.alert_severities.error,
-		  list_name,
-		  last_error
-	       )
+	    local alert = alert_consts.alert_types.alert_list_download_failed.new(
+	       list_name,
+	       last_error
 	    )
 
+	    alert:set_score_error()
+
+	    alert:store(alerts_api.systemEntity(list_name))
+
 	    msg = msg .. "ERROR ["..last_error.."]"
 	 end
 
@@ -462,7 +471,7 @@ local function loadWarning(msg)
 end
 
 --@return nil on parse error, "domain" if the loaded item is an host, "ip" otherwise
-local function loadListItem(host, category, user_custom_categories, list)
+local function loadListItem(host, category, user_custom_categories, list, num_line)
    category = tonumber(category)
 
    -- Checking for "whitelisted hosts" (Format: !<host>)
@@ -499,12 +508,8 @@ local function loadListItem(host, category, user_custom_categories, list)
       else
 	 -- Domain
 	 if((not list) or (list.format ~= "ip")) then
-	   if((string.len(host) < 4) or (string.find(host, "%.") == nil)) then
-	     traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Bad domain name '%s' in list '%s'", host, list and list.name))
-	   else
-	     ntop.loadCustomCategoryHost(host, category)
-	     return "domain"
-	   end
+	    ntop.loadCustomCategoryHost(host, category)
+	    return "domain"
 	 else
 	   loadWarning(string.format("Invalid domain '%s' in list '%s'", host, list.name))
 	 end
@@ -556,64 +561,81 @@ end
 -- Loads hosts from a list file on disk
 local function loadFromListFile(list_name, list, user_custom_categories, stats)
    local list_fname = getListCacheFile(list_name)
-   local f = io.open(list_fname, "r")
    local num_rules = 0
    local limit_exceeded = false
 
-   if f == nil then
-      if list.status.num_hosts > 0 then
-	 -- avoid generating warnings during first startup
-	 traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Could not find '%s'...", list_fname))
-      end
-
-      return(false)
-   end
-
    traceError(trace_level, TRACE_CONSOLE, string.format("Loading '%s' [%s]...", list_name, list.format))
 
-   for line in f:lines() do
-      if ntop.isShutdown() then
-	 break
+   if list.format == "ja3_suricata_csv" then
+      -- Load the signatures file in nDPI
+      local n = ntop.loadMaliciousJA3Signatures(list_fname)
+      if n >= 0 then
+
+         stats.num_ja3 = stats.num_ja3 + n
+         num_rules = num_rules + n
+
+      else -- Failure
+         if list.status.num_hosts > 0 then
+            -- Avoid generating warnings during first startup
+	    traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Could not find '%s'...", list_fname))
+         end
+
+         return(false)
       end
-      local trimmed = line:match("^%s*(.-)%s*$")
 
-      if((string.len(trimmed) > 0) and not(string.starts(trimmed, "#"))) then
-	 local host = trimmed
+   else
 
-	 if list.format == "hosts" then
-	    host = parse_hosts_line(trimmed)
-	 elseif list.format == "ja3_suricata_csv" then
-	    -- handled differently
-	    if handle_ja3_suricata_csv_line(trimmed) then
-	       stats.num_ja3 = stats.num_ja3 + 1
-	       num_rules = num_rules + 1
-	    end
-	    host = nil
-	 end
+      local f = io.open(list_fname, "r")
+      local num_line = 0
 
-	 if host then
-	    local rv = loadListItem(host, list.category, user_custom_categories, list)
+      if f == nil then
+         if list.status.num_hosts > 0 then
+            -- Avoid generating warnings during first startup
+	    traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Could not find '%s'...", list_fname))
+         end
 
-	    if(rv == "domain") then
-	       stats.num_hosts = stats.num_hosts + 1
-	       num_rules = num_rules + 1
-	    elseif(rv == "ip") then
-	       stats.num_ips = stats.num_ips + 1
-	       num_rules = num_rules + 1
-	    end
-	 end
+         return(false)
+      end
 
-	 if((stats.num_ips >= MAX_TOTAL_IP_RULES) or
-	       (stats.num_hosts >= MAX_TOTAL_DOMAIN_RULES) or
-	       (stats.num_ja3 >= MAX_TOTAL_JA3_RULES)) then
-	    limit_exceeded = true
+      for line in f:lines() do
+      	  num_line = num_line + 1
+         if ntop.isShutdown() then
 	    break
-	 end
+         end
+         local trimmed = line:match("^%s*(.-)%s*$")
+
+         if((string.len(trimmed) > 0) and not(string.starts(trimmed, "#"))) then
+	    local host = trimmed
+
+	    if list.format == "hosts" then
+	       host = parse_hosts_line(trimmed)
+            end
+
+	    if host then
+	       local rv = loadListItem(host, list.category, user_custom_categories, list, num_line)
+
+	       if(rv == "domain") then
+	          stats.num_hosts = stats.num_hosts + 1
+	          num_rules = num_rules + 1
+	       elseif(rv == "ip") then
+	          stats.num_ips = stats.num_ips + 1
+	          num_rules = num_rules + 1
+	       end
+	    end
+
+	    if((stats.num_ips >= MAX_TOTAL_IP_RULES) or
+	          (stats.num_hosts >= MAX_TOTAL_DOMAIN_RULES) or
+	          (stats.num_ja3 >= MAX_TOTAL_JA3_RULES)) then
+	       limit_exceeded = true
+	       break
+	    end
+         end
       end
+
+      f:close()
    end
 
    list.status.num_hosts = num_rules
-   f:close()
 
    traceError(trace_level, TRACE_CONSOLE, string.format("\tRead '%d' rules", num_rules))
 
@@ -633,7 +655,7 @@ local function reloadListsNow()
    local stats = {num_hosts = 0, num_ips = 0, num_ja3 = 0, begin = os.time(), duration = 0}
    local limit_reached_error = nil
 
-   if(not ntop.startCustomCategoriesReload()) then
+   if(not ntop.initnDPIReload()) then
       -- Too early, need to retry later
       traceError(trace_level, TRACE_CONSOLE, string.format("custom categories: too early reload"))
       return(false)
@@ -682,13 +704,12 @@ local function reloadListsNow()
 	 if ntop.isShutdown() then
 	    break
 	 end
-	 loadListItem(host, category_id, user_custom_categories)
+	 loadListItem(host, category_id, user_custom_categories, {} --[[ No list --]], 0 --[[ No line number --]])
       end
    end
 
    -- Reload into memory
-   ntop.reloadCustomCategories()
-   ntop.reloadJA3Hashes()
+   ntop.finalizenDPIReload()
 
    -- Calculate stats
    stats.duration = (os.time() - stats.begin)
@@ -696,7 +717,7 @@ local function reloadListsNow()
    traceError(TRACE_NORMAL, TRACE_CONSOLE,
 	      string.format("Category Lists (%u hosts, %u IPs, %u JA3) loaded in %d sec",
 			    stats.num_hosts, stats.num_ips, stats.num_ja3, stats.duration))
-   
+
    -- Save the stats
    ntop.setCache("ntopng.cache.category_lists.load_stats", json.encode(stats))
 
@@ -719,6 +740,11 @@ end
 
 -- This is run in housekeeping.lua
 function lists_utils.checkReloadLists()
+
+   if ntop.isOffline() then
+      return
+   end
+
    local forced_reload = (ntop.getCache("ntopng.cache.reload_lists_utils") == "1")
    local reload_now = false
 
@@ -734,10 +760,10 @@ function lists_utils.checkReloadLists()
    end
 
    -- print("[DEBUG]  Checking reload [") if(reload_now) then print("reload now") else print("don't reload") end print("] !!!!\n")
-   
+
    if reload_now then
       -- print("[DEBUG] **** Reloading ****\n")
-      
+
       if reloadListsNow() then
 	 -- print("[DEBUG]  Success !!!!\n")
 	 -- success
@@ -756,12 +782,12 @@ end
 
 function lists_utils.startup()
    traceError(TRACE_NORMAL, TRACE_CONSOLE, "Refreshing category lists...")
-   
+
    lists_utils.downloadLists()
    lists_utils.reloadLists()
    -- Need to do the actual reload also here as otherwise some
    -- flows may be misdetected until housekeeping.lua is executed
-   lists_utils.checkReloadLists()   
+   lists_utils.checkReloadLists()
 end
 
 -- ##############################################
