@@ -1,7 +1,6 @@
 require "lua_utils"
 
 local recording_utils = require "recording_utils"
-local remote_assistance = require "remote_assistance"
 local plugins_utils = require("plugins_utils")
 
 local prefs = ntop.getPrefs()
@@ -114,9 +113,6 @@ local menu_subpages = {
     }, toggle_dst_with_post_nat_dst = {
       title       = i18n("prefs.toggle_dst_with_post_nat_dst_title"),
       description = i18n("prefs.toggle_dst_with_post_nat_dst_description"),
-    }, toggle_src_and_dst_using_ports = {
-      title       = i18n("prefs.toggle_src_and_dst_using_ports_title"),
-      description = i18n("prefs.toggle_src_and_dst_using_ports_description"),
     },
   }}, {id="in_memory",     label=i18n("prefs.cache_settings"),             advanced=true,  pro_only=false,  hidden=false, entries={
     toggle_midnight_stats_reset = {
@@ -131,9 +127,6 @@ local menu_subpages = {
     }, flow_max_idle = {
       title       = i18n("prefs.flow_max_idle_title"),
       description = i18n("prefs.flow_max_idle_description"),
-    }, housekeeping_frequency = {
-      title       = i18n("prefs.housekeeping_frequency_title"),
-      description = i18n("prefs.housekeeping_frequency_description", {product=info["product"]}),
     }, toggle_local_host_cache_enabled = {
       title       = i18n("prefs.toggle_local_host_cache_enabled_title"),
       description = i18n("prefs.toggle_local_host_cache_enabled_description"),
@@ -195,6 +188,9 @@ local menu_subpages = {
     }, toggle_country_rrds = {
       title       = i18n("prefs.toggle_country_rrds_title"),
       description = i18n("prefs.toggle_country_rrds_description"),
+    }, toggle_os_rrds = {
+      title       = i18n("prefs.toggle_os_rrds_title"),
+      description = i18n("prefs.toggle_os_rrds_description"),
     }, toggle_ndpi_flows_rrds = {
       title       = i18n("prefs.toggle_ndpi_flows_rrds_title"),
       description = i18n("prefs.toggle_ndpi_flows_rrds_description"),
@@ -231,18 +227,51 @@ local menu_subpages = {
     disable_alerts_generation = {
       title       = i18n("prefs.disable_alerts_generation_title"),
       description = i18n("prefs.disable_alerts_generation_description"),
-    }, max_num_alerts_per_entity = {
-      title       = i18n("prefs.max_num_alerts_per_entity_title"),
-      description = i18n("prefs.max_num_alerts_per_entity_description"),
-    }, max_num_flow_alerts = {
-      title       = i18n("prefs.max_num_flow_alerts_title"),
-      description = i18n("prefs.max_num_flow_alerts_description"),
-    }, toggle_mysql_check_open_files_limit = {
+    }, max_entity_alerts = {
+      title       = i18n("prefs.max_entity_alerts_title"),
+      description = i18n("prefs.max_entity_alerts_description"),
+    }, max_num_secs_before_delete_alert = {
+      title       = i18n("prefs.max_num_secs_before_delete_alert_title"),
+      description = i18n("prefs.max_num_secs_before_delete_alert_description"),
+    },toggle_mysql_check_open_files_limit = {
       title       = i18n("prefs.toggle_mysql_check_open_files_limit_title"),
       description = i18n("prefs.toggle_mysql_check_open_files_limit_description"),
       hidden      = (prefs.is_dump_flows_to_mysql_enabled == false),
+    },toggle_emit_flow_alerts = {
+      title       = i18n("prefs.toggle_emit_flow_alerts_title"),
+      description = i18n("prefs.toggle_emit_flow_alerts_description"),
+    },toggle_emit_host_alerts = {
+      title       = i18n("prefs.toggle_emit_host_alerts_title"),
+      description = i18n("prefs.toggle_emit_host_alerts_description"),
     }
-  }}, {id="protocols",     label=i18n("prefs.protocols"),            advanced=false, pro_only=false,  hidden=false, entries={
+  }}, { id ="traffic_behaviour",
+    label=i18n("prefs.behaviour"),  
+    advanced=true, 
+    enterprise_l_only=true, 
+    hidden=false,
+    entries={
+      toggle_behaviour_analysis = {
+        title       = i18n("prefs.toggle_behaviour_analysis_title"),
+        description = i18n("prefs.toggle_behaviour_analysis_description"),
+      },
+      behaviour_analysis_learning_period = {
+        title       = i18n("prefs.behaviour_analysis_learning_period_title"),
+        description = i18n("prefs.behaviour_analysis_learning_period_description"),
+      },
+      behaviour_analysis_learning_status_during_learning = {
+        title = i18n("prefs.behaviour_analysis_status_during_learning_title"),
+        description = i18n("prefs.behaviour_analysis_status_during_learning_description")
+      },
+      behaviour_analysis_learning_status_post_learning = {
+        title = i18n("prefs.behaviour_analysis_status_post_learning_title"),
+        description = i18n("prefs.behaviour_analysis_status_post_learning_description")
+      },
+    }, 
+    iec60870_learning_period = {
+      title       = i18n("prefs.iec60870_learning_period_title"),
+      description = i18n("prefs.iec60870_learning_period_description"),
+    },
+  }, {id="protocols",     label=i18n("prefs.protocols"),            advanced=false, pro_only=false,  hidden=false, entries={
     toggle_top_sites = {
       title       = i18n("prefs.toggle_top_sites_title"),
       description = i18n("prefs.toggle_top_sites_description", {url="https://resources.sei.cmu.edu/asset_files/Presentation/2010_017_001_49763.pdf"})},
@@ -308,11 +337,6 @@ local menu_subpages = {
       title       = i18n("traffic_recording.max_extracted_pcap_bytes_title"),
       description = i18n("traffic_recording.max_extracted_pcap_bytes_description"),
     },
-  }}, {id="remote_assistance", label=i18n("remote_assistance.remote_assistance"), advanced=true,  pro_only=false, hidden=(not remote_assistance.isAvailable()), entries={
-    n2n_supernode = {
-      title       = i18n("prefs.n2n_supernode_title"),
-      description = i18n("prefs.n2n_supernode_description", {url="https://www.ntop.org/products/n2n"}),
-    },
   }}, {id="retention", label=i18n("prefs.data_retention"), advanced=true, pro_only=false, hidden=false, entries={
     data_retention = {
       title       = i18n("prefs.data_retention"),
@@ -334,7 +358,6 @@ local menu_subpages = {
        description = i18n("prefs.mgmt_acl_description", {product=info.product}),
     },
     }}, {id="misc",          label=i18n("prefs.misc"),                 advanced=false, pro_only=false,  hidden=false, entries={
-
     toggle_send_telemetry_data = {
       title       = i18n("prefs.toggle_send_telemetry_data_title"),
       description = i18n("prefs.toggle_send_telemetry_data_description", { product = info.product}),
@@ -344,13 +367,13 @@ local menu_subpages = {
     }, toggle_host_mask = {
       title       = i18n("prefs.toggle_host_mask_title"),
       description = i18n("prefs.toggle_host_mask_description"),
-    }, toggle_behaviour_analysis = {
-      title       = i18n("prefs.toggle_behaviour_analysis_title"),
-      description = i18n("prefs.toggle_behaviour_analysis_description"),
     }, topk_heuristic_precision = {
       title       = i18n("prefs.topk_heuristic_precision_title"),
       description = i18n("prefs.topk_heuristic_precision_description"),
-    }
+    }, flow_table_time = {
+      title       = i18n("prefs.flow_table_time_title"),
+      description = i18n("prefs.flow_table_time_description"),
+    },
   }}, {id="updates", label=i18n("prefs.updates"), advanced=false, pro_only=false, hidden=(is_windows or (not ntop.isPackage())), entries={
     toggle_autoupdates = {
       title       = i18n("prefs.toggle_autoupdates_title"),
