@@ -162,7 +162,8 @@ template = {
    ['NUM_PKTS_TTL_224_255'] = NTOP_BASE_ID+341,
    ['IN_SRC_OSI_SAP'] = NTOP_BASE_ID+349,
    ['OUT_DST_OSI_SAP'] = NTOP_BASE_ID+350,
-
+   ['FLOW_VERDICT'] =  NTOP_BASE_ID+522,
+   ['L7_RISK_SCORE'] = NTOP_BASE_ID+527,
    -- SIP
    ['SIP_CALL_ID'] = NTOP_BASE_ID+130,
    ['SIP_CALLING_PARTY'] = NTOP_BASE_ID+131,
