@@ -1,5 +1,5 @@
 --
--- (C) 2018 - ntop.org
+-- (C) 2021 - ntop.org
 --
 
 local driver = {}
@@ -780,7 +780,8 @@ function driver:query(schema, tstart, tend, tags, options)
 
       for k, v in pairs(series) do
         local s = ts_common.calculateStatistics(v.data, time_step, tend - tstart, data_type)
-        stats.by_serie[k] = s
+        -- Adding per timeseries min-max stats
+        stats.by_serie[k] = table.merge(s, ts_common.calculateMinMax(v.data))
 
         -- Remove the total for now as it requires a complex computation (see below)
         s.total = nil
@@ -973,17 +974,18 @@ function driver:_droppedPointsAlert()
 
    if ntop.getCache(k) ~= "1" then
       local alerts_api = require("alerts_api")
-      local alert_consts = require "alert_consts"
+      local alert_severities = require "alert_severities"
+local alert_consts = require "alert_consts"
 
-      alerts_api.store(
-        alerts_api.influxdbEntity(self.url),
-        alert_consts.alert_types.alert_influxdb_export_failure.create(
-	   alert_consts.alert_severities.error,
-	   alert_consts.alerts_granularities.min,
-	   self.url
-	)
+      local alert = alert_consts.alert_types.alert_influxdb_export_failure.new(
+        self.url
       )
 
+      alert:set_score_error()
+      alert:set_granularity(alert_consts.alerts_granularities.min)
+
+      alert:store(alerts_api.systemEntity())
+      
       -- Just to avoid doing :trigger too often
       ntop.setCache(k, "1", alert_periodicity / 2)
    end
@@ -1607,31 +1609,31 @@ local function updateCQRetentionPolicies(dbname, url, username, password)
   local res = influx_query(url .. "/query?db=".. dbname, query, username, password)
   local rp_1h_statement = "CREATE"
   local rp_1d_statement = "CREATE"
+  local create_1h, create_1d = true, true
 
   if res and res.series and res.series[1] then
     for _, rp in pairs(res.series[1].values) do
       local rp_name = rp[1]
 
       if rp_name == "1h" then
-        rp_1h_statement = "ALTER"
+	 create_1h = false
       elseif rp_name == "1d" then
-        rp_1d_statement = "ALTER"
+	 create_1d = false
       end
     end
   end
 
-  local queries = {
-    makeRetentionPolicyQuery(rp_1d_statement, "1d", dbname, get1dDatabaseRetentionDays())
-  }
+  local queries = {}
 
-  if(HOURLY_CQ_ENABLED) then
-    queries[#queries + 1] = makeRetentionPolicyQuery(rp_1h_statement, "1h", dbname, get1hDatabaseRetentionDays())
-  else
-    -- Delete existing 1h aggregated data
-    queries[#queries + 1] = "DROP RETENTION POLICY \"1h\" ON \"" .. dbname .. "\""
+  if create_1d then
+     queries[#queries + 1] = makeRetentionPolicyQuery(rp_1d_statement, "1d", dbname, get1dDatabaseRetentionDays())
+  end
+  
+  if create_1h and HOURLY_CQ_ENABLED then
+     queries[#queries + 1] = makeRetentionPolicyQuery(rp_1h_statement, "1h", dbname, get1hDatabaseRetentionDays())
   end
 
-  return multiQueryPost(queries, url, username, password)
+  return #queries > 0 and multiQueryPost(queries, url, username, password) or true
 end
 
 -- ##############################################
@@ -1654,7 +1656,7 @@ local function isCompatibleVersion(version)
   local current = toVersion(version)
   local required = toVersion(MIN_INFLUXDB_SUPPORTED_VERSION)
 
-  if (version == nil) or (required == nil) then
+  if (current == nil) or (required == nil) then
     return false
   end
 
