@@ -1,7 +1,10 @@
 --
--- (C) 2019-20 - ntop.org
+-- (C) 2019-21 - ntop.org
 --
 
+local dirs = ntop.getDirs()
+package.path = dirs.installdir .. "/scripts/lua/modules/alert_store/?.lua;" .. package.path
+
 local am_utils = {}
 local ts_utils = require "ts_utils_core"
 local format_utils = require "format_utils"
@@ -29,6 +32,14 @@ local am_hosts_key = string.format("ntopng.prefs.ifid_%d.am_hosts", getSystemInt
 
 -- ##############################################
 
+-- @brief Key used to save the result of the measurement (if requested)
+--        Used for example to save the page fetched with HTTP/HTTPs measurements
+local function am_last_result_key(key)
+  return string.format("ntopng.cache.ifid_%d.am_hosts.last_result." .. key, getSystemInterfaceId())
+end
+
+-- ##############################################
+
 local function am_last_updates_key(key)
   return string.format("ntopng.cache.ifid_%d.am_hosts.last_update_v1." .. key, getSystemInterfaceId())
 end
@@ -39,6 +50,14 @@ end
 
 -- ##############################################
 
+function am_utils.setLastResult(key, last_result)
+   if not isEmptyString(last_result) then
+      ntop.setCache(am_last_result_key(key), last_result)
+   end
+end
+
+-- ##############################################
+
 function am_utils.setLastAmUpdate(key, when, value, ipaddress, jitter, mean)
   local v = {
     when = when,
@@ -214,11 +233,15 @@ function am_utils.getAmHostKey(host, measurement)
   return(string.format("%s@%s", measurement, host))
 end
 
+-- @brief Extract the measurement and the host from the key
+--        which is the concatenation of <measurement>@<host>
 local function key2amhost(host)
-  local parts = string.split(host, "@")
+   -- Examples: http@https://maina:maina2@develv5:3003/lua/cane
+   --           https://maina:maina2@develv5:3003/lua/cane
+  local measurement, amhost = string.match(host, "^([^@]+)@(.+)")
 
-  if(parts and (#parts == 2)) then
-    return parts[2], parts[1]
+  if measurement and amhost then
+     return amhost, measurement
   end
 end
 
@@ -241,9 +264,48 @@ end
 
 -- ##############################################
 
+-- @brief Returns the possibly saved result of a measurement
+function am_utils.getLastResult(host, measurement)
+  local key = am_utils.getAmHostKey(host, measurement)
+  local val = ntop.getCache(am_last_result_key(key))
+
+  if not isEmptyString(val) then
+     return val
+  end
+
+  return nil
+end
+
+-- ##############################################
+
+-- @brief Check if this is an infrastructure active monitoring url
+local function is_infrastructure(host)
+   if not ntop.isEnterpriseM() or isEmptyString(host) then
+      return false, nil
+   end
+
+   package.path = dirs.installdir .. "/pro/scripts/lua/enterprise/modules/?.lua;" .. package.path
+   local infrastructure_utils = require("infrastructure_utils")
+
+   -- The host is considered an infrastructure host if it contains the endpoint in the name
+   if host:find(infrastructure_utils.ENDPOINT_TO_EXTRACT_DATA) or host:find(infrastructure_utils.SUFFIX_THROUGHPUT) then
+      local instance = infrastructure_utils.get_instance_by_host(host)
+
+      if not instance then
+	 return false, nil
+      else
+	 return true, instance.alias
+      end
+   end
+
+   return false, nil
+end
+
+-- ##############################################
+
 -- Only used for the formatting, don't use as a key as the "/"
 -- character is escaped in HTTP parameters
-function am_utils.formatAmHost(host, measurement)
+function am_utils.formatAmHost(host, measurement, isHtml)
   local m_info = am_utils.getMeasurementInfo(measurement)
 
   --if m_info and m_info.force_host then
@@ -251,7 +313,28 @@ function am_utils.formatAmHost(host, measurement)
     --return(host)
   --end
 
-  return(string.format("%s://%s", measurement, host))
+  if(host == nil) then
+     return(nil)
+  end
+  
+  local res = host
+  local is_infr, infr_name = is_infrastructure(host)
+
+  -- Make a smarter way to determine infrastructure labels
+  if is_infr then
+
+     -- Make a nicer label for infrastructure hosts
+     -- If the am host contains a name for the infrastructure use it
+     if isHtml then
+      res = infr_name .. " <i class='fas fa-building'></i>"
+     else 
+      res = infr_name .. " [".. i18n("infrastructure_dashboard.infrastructure") .. "]"
+    end
+  else
+    res = host
+  end
+
+  return res
 end
 
 -- ##############################################
@@ -260,7 +343,8 @@ function am_utils.key2host(host_key)
   local host, measurement = key2amhost(host_key)
 
   return {
-    label = am_utils.formatAmHost(host, measurement),
+    label = am_utils.formatAmHost(host, measurement, false),
+    is_infrastructure = is_infrastructure(host),
     measurement = measurement,
     host = host,
   }
@@ -300,8 +384,13 @@ local function deserializeAmPrefs(host_key, val, config_only)
     local v = json.decode(val)
 
     if v then
+      rv.show_iface = false
+      rv.ifname = v.ifname or ''
       rv.threshold = tonumber(v.threshold) or 500
       rv.granularity = v.granularity or "min"
+      rv.token = v.token
+      rv.save_result = v.save_result
+      rv.readonly = v.readonly
     end
   end
 
@@ -372,14 +461,31 @@ end
 
 -- ##############################################
 
-function am_utils.addHost(host, measurement, am_value, granularity, pool)
+-- @brief Add and host as part of the active monitoring
+-- @param measurement A string with the type of measurement which will be performed
+-- @param am_value A number used as threshold con consider the measurement failed
+-- @param granularity One of `supported_granularities`, indicating the granularity of the measurement
+-- @param pool The pool_id `host` will be associated to
+-- @param token A string with an ntopng `token` used to fetch data from other ntopngs in a federation [optional]
+-- @param save_result Whether the result fetched with the measure should be saved (e.g., the HTTP response) [optional]
+-- @param readonly Bool used by the GUI to know if, when true, an entry is considered read only hence it cannot be modified/deleted [optional]
+function am_utils.addHost(host, ifname, measurement, am_value, granularity, pool, token, save_result, readonly)
+  save_result = save_result or false
+  readonly = readonly or false
+
   local active_monitoring_pools = require("active_monitoring_pools")
   local am_pool = active_monitoring_pools:create()
   local host_key = am_utils.getAmHostKey(host, measurement)
+  local show_iface = ntop.isPingIfaceAvailable()
 
   ntop.setHashCache(am_hosts_key, host_key, serializeAmPrefs({
+    show_iface = show_iface,
+    ifname = ifname or '',
     threshold = tonumber(am_value) or 500,
     granularity = granularity or "min",
+    token = token, -- ntopng auth token
+    save_result = save_result, -- save the result
+    readonly = readonly,
   }))
 
   -- Bind the host from any existing pool
@@ -406,13 +512,15 @@ function am_utils.deleteHost(host, measurement)
 
   local host_key = am_utils.getAmHostKey(host, measurement)
   local am_host_entity = alerts_api.amThresholdCrossEntity(host_key)
-  local old_ifname = ifname
 
   -- Release any engaged alerts of the host
   alerts_api.releaseEntityAlerts(am_host_entity)
 
   am_utils.discardHostTimeseries(host, measurement)
 
+  -- Remove possibly saved results
+  ntop.delCache(am_last_result_key(host_key))
+
   -- Remove the redis keys of the host
   ntop.delCache(am_last_updates_key(host_key))
   am_utils.dropHourStats(host_key)
@@ -623,12 +731,10 @@ end
 -- ##############################################
 
 local function amThresholdCrossType(value, threshold, ip, granularity, entity_info)
-  local host = am_utils.key2host(entity_info.alert_entity_val)
+  local host = am_utils.key2host(entity_info.entity_val)
   local m_info = am_utils.getMeasurementInfo(host.measurement)
 
-  local alert_type = alert_consts.alert_types.alert_am_threshold_cross.create(
-     alert_consts.alert_severities.warning,
-     alert_consts.alerts_granularities[granularity],
+  local alert_type = alert_consts.alert_types.alert_am_threshold_cross.new(
      value,
      threshold,
      ip,
@@ -637,6 +743,9 @@ local function amThresholdCrossType(value, threshold, ip, granularity, entity_in
      m_info.i18n_unit
   )
 
+  alert_type:set_score_warning()
+  alert_type:set_granularity(granularity)
+
   return alert_type
 end
 
@@ -657,7 +766,7 @@ function am_utils.triggerAlert(numeric_ip, ip_label, current_value, upper_thresh
     end
   end
 
-  return alerts_api.trigger(entity_info, type_info)
+  return type_info:trigger(entity_info)
 end
 
 -- ##############################################
@@ -666,7 +775,7 @@ function am_utils.releaseAlert(numeric_ip, ip_label, current_value, upper_thresh
   local entity_info = alerts_api.amThresholdCrossEntity(ip_label)
   local type_info = amThresholdCrossType(current_value, upper_threshold, numeric_ip, granularity, entity_info)
 
-  return alerts_api.release(entity_info, type_info)
+  return type_info:release(entity_info)
 end
 
 -- ##############################################
@@ -675,7 +784,6 @@ end
 --        `am_host` is one of the hosts obtained with `am_utils.getHosts()`
 -- @return True if the host has engaged alerts, false otherwise
 function am_utils.hasAlerts(am_host)
-   local alert_utils = require "alert_utils"
    local am_key = am_utils.getAmHostKey(am_host.host, am_host.measurement)
    local entity_info = alerts_api.amThresholdCrossEntity(am_key)
 
@@ -684,13 +792,12 @@ function am_utils.hasAlerts(am_host)
    local old_ifid = interface.getId()
    interface.select(getSystemInterfaceId())
 
-   local num_engaged_alerts = alert_utils.getNumAlerts("engaged", {
-							  entity = entity_info.alert_entity.entity_id,
-							  entity_val = entity_info.alert_entity_val })
+   local am_alert_store = require "am_alert_store".new()
+   local engaged = am_alert_store:select_engaged(am_key)
 
    interface.select(tostring(old_ifid))
 
-   return num_engaged_alerts > 0
+   return engaged and #engaged > 0
 end
 
 -- ##############################################
@@ -709,35 +816,27 @@ end
 -- ##############################################
 
 -- Resolve the domain name into an IP if necessary
-function am_utils.resolveHost(domain_name, is_v6)
-   local ip_address = nil
+function am_utils.resolveHost(domain_name)
+   local ip_address = domain_name
 
-   if not isIPv4(domain_name) and not is_v6 then
-     ip_address = ntop.resolveHost(domain_name, true --[[IPv4 --]])
-
-     if not ip_address then
-	if do_trace then
-	   print("[ActiveMonitoring] Could not resolve IPv4 host: ".. domain_name .."\n")
-	end
-     end
-   elseif not isIPv6(domain_name) and is_v6 then
-      ip_address = ntop.resolveHost(domain_name, false --[[IPv6 --]])
+   if not isIPv4(domain_name) and not isIPv6(domain_name) then
+      -- Symbolic name, try first to resolve as IPv4, then as IPv6
+      ip_address = ntop.resolveHost(domain_name, true --[[IPv4 --]])
 
       if not ip_address then
-	if do_trace then
-	   print("[ActiveMonitoring] Could not resolve IPv6 host: ".. domain_name .."\n")
-	end
+	 -- IPv4 resolution failed, attempt at resolving ipv6
+	 ip_address = ntop.resolveHost(domain_name, false --[[IPv6 --]])
       end
    else
-     ip_address = domain_name
+      -- Regular IPv4 or IPv6, no need to resolve
    end
 
-  return(ip_address)
+   return ip_address
 end
 
 -- ##############################################
 
-function am_utils.editHost(host, measurement, threshold, granularity, pool)
+function am_utils.editHost(host, iface, measurement, threshold, granularity, pool, token, save_result, readonly)
   local existing = am_utils.getHost(host, measurement)
 
   if(existing == nil) then
@@ -775,7 +874,7 @@ function am_utils.editHost(host, measurement, threshold, granularity, pool)
     end
   end
 
-  am_utils.addHost(host, measurement, threshold, granularity, pool)
+  am_utils.addHost(host, iface, measurement, threshold, granularity, pool, token, save_result, readonly)
 
   return(true)
 end
