@@ -25,6 +25,8 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 
+METHOD_DEF=http
+
 usage() {
 	[ $# -gt 0 ] && echo "Missing: $@" >&2
 	cat << EOF
@@ -57,9 +59,10 @@ Options:
     -m method     -- When used with -c, overrides the default method for
                      obtaining and building the jail. See poudriere(8) for more
                      details. Can be one of:
-                       allbsd, ftp-archive, ftp, git, http, null, src=PATH, svn,
-                       svn+file, svn+http, svn+https, svn+ssh, tar=PATH
-                       url=SOMEURL.
+                       'allbsd', 'ftp-archive', 'ftp', 'freebsdci', 'http',
+		       'null', 'src=PATH', 'tar=PATH', 'url=URL', or
+		       '{git,svn}{,+http,+https,+file,+ssh}' (e.g., 'git+https').
+                     The default is '${METHOD_DEF}'.
     -P patch      -- Specify a patch to apply to the source before building.
     -S srcpath    -- Specify a path to the source tree to be used.
     -D            -- Do a full git clone without --depth (default: --depth=1)
@@ -67,8 +70,7 @@ Options:
     -U url        -- Specify a url to fetch the sources (with method git and/or svn).
     -x            -- Build and setup native-xtools cross compile tools in jail when
                      building for a different TARGET ARCH than the host.
-                     Only applies if TARGET_ARCH and HOST_ARCH are different.
-                     Will only be used if -m is svn*.
+                     Will only be used if -m does a source build.
 
 Options for -d:
     -C clean      -- Clean remaining data existing in poudriere data folder.
@@ -172,8 +174,13 @@ cleanup_new_jail() {
 update_version() {
 	local version_extra="$1"
 
-	eval `egrep "^REVISION=|^BRANCH=" ${SRC_BASE}/sys/conf/newvers.sh `
-	RELEASE=${REVISION}-${BRANCH}
+	if [ -r "${SRC_BASE}/sys/conf/newvers.sh" ]; then
+		eval `egrep "^REVISION=|^BRANCH=" ${SRC_BASE}/sys/conf/newvers.sh `
+		RELEASE=${REVISION}-${BRANCH}
+	else
+		RELEASE=$(jget ${JAILNAME} version)
+	fi
+	[ -n "${RELEASE}" ] || err 1 "updated_version: Failed to determine RELEASE"
 	[ -n "${version_extra}" ] &&
 	    RELEASE="${RELEASE} ${version_extra}"
 	jset ${JAILNAME} version "${RELEASE}"
@@ -230,7 +237,7 @@ update_jail() {
 		SRC_BASE="${JAILMNT}/usr/src"
 	fi
 	if [ -z "${METHOD}" -o "${METHOD}" = "-" ]; then
-		METHOD="ftp"
+		METHOD="${METHOD_DEF}"
 		jset ${JAILNAME} method ${METHOD}
 	fi
 	msg "Upgrading using ${METHOD}"
@@ -341,7 +348,7 @@ update_jail() {
 		make -C ${SRC_BASE} delete-old delete-old-libs DESTDIR=${JAILMNT} BATCH_DELETE_OLD_FILES=yes
 		markfs clean ${JAILMNT}
 		;;
-	allbsd|gjb|url=*)
+	allbsd|gjb|url=*|freebsdci)
 		[ -z "${VERSION}" ] && VERSION=$(jget ${JAILNAME} version)
 		[ -z "${ARCH}" ] && ARCH=$(jget ${JAILNAME} arch)
 		delete_jail
@@ -556,20 +563,27 @@ install_from_vcs() {
 		case ${METHOD} in
 		svn*)
 			msg_n "Checking out the sources with ${METHOD}..."
-			${SVN_CMD} -q co ${SVN_FULLURL}/${VERSION} ${SRC_BASE} || err 1 " fail"
+			${SVN_CMD} ${quiet} checkout \
+			    ${SVN_FULLURL}/${VERSION} ${SRC_BASE} || \
+			    err 1 " fail"
 			echo " done"
 			if [ -n "${SRCPATCHFILE}" ]; then
 				msg_n "Patching the sources with ${SRCPATCHFILE}"
-				${SVN_CMD} -q patch ${SRCPATCHFILE} ${SRC_BASE} || err 1 " fail"
+				${SVN_CMD} ${quiet} patch ${SRCPATCHFILE} \
+				    ${SRC_BASE} || err 1 " fail"
 				echo done
 			fi
 			;;
 		git*)
+			# !! Any changes here should be considered for ports.sh too.
 			if [ -n "${SRCPATCHFILE}" ]; then
 				err 1 "Patch files not supported with git, please use feature branches"
 			fi
 			msg_n "Checking out the sources with ${METHOD}..."
-			${GIT_CMD} clone ${GIT_DEPTH} -q -b ${VERSION} ${GIT_FULLURL} ${SRC_BASE} || err 1 " fail"
+			${GIT_CMD} clone ${GIT_DEPTH} ${quiet} \
+			    ${VERSION:+-b ${VERSION}} ${GIT_FULLURL} \
+			    ${SRC_BASE} || \
+			    err 1 " fail"
 			echo " done"
 			# No support for patches, using feature branches is recommanded"
 			;;
@@ -579,13 +593,17 @@ install_from_vcs() {
 		svn*)
 			msg_n "Updating the sources with ${METHOD}..."
 			${SVN_CMD} upgrade ${SRC_BASE} 2>/dev/null || :
-			${SVN_CMD} -q update -r ${TORELEASE:-head} ${SRC_BASE} || err 1 " fail"
+			${SVN_CMD} ${quiet} update -r ${TORELEASE:-head} ${SRC_BASE} || err 1 " fail"
 			echo " done"
 			;;
 		git*)
-			${GIT_CMD} -C ${SRC_BASE} pull --rebase -q || err 1 " fail"
+			# !! Any changes here should be considered for ports.sh too.
+			msg_n "Updating the sources with ${METHOD}..."
+			${GIT_CMD} -C ${SRC_BASE} pull --rebase ${quiet} || \
+			    err 1 " fail"
 			if [ -n "${TORELEASE}" ]; then
-				${GIT_CMD} checkout -q "${TORELEASE}" || err 1 " fail"
+				${GIT_CMD} -C ${SRC_BASE} checkout \
+				    ${quiet} "${TORELEASE}" || err 1 " fail"
 			fi
 			echo " done"
 			;;
@@ -625,7 +643,7 @@ install_from_ftp() {
 	esac
 
 	DISTS="${DISTS} base games"
-	[ -z "${SRCPATH}" ] && DISTS="${DISTS} src"
+	[ -z "${SRCPATH}" -a "${NO_SRC:-no}" = "no" ] && DISTS="${DISTS} src"
 	DISTS="${DISTS} ${EXTRA_DISTS}"
 
 	case "${V}" in
@@ -657,6 +675,7 @@ install_from_ftp() {
 		url=*) URL=${METHOD##url=} ;;
 		allbsd) URL="https://pub.allbsd.org/FreeBSD-snapshots/${ARCH%%.*}-${ARCH##*.}/${V}-JPSNAP/ftp" ;;
 		ftp-archive) URL="http://ftp-archive.freebsd.org/pub/FreeBSD-Archive/old-releases/${ARCH}/${V}" ;;
+		freebsdci) URL="https://artifact.ci.freebsd.org/snapshot/${V}/latest_tested/${ARCH%%.*}/${ARCH##*.}" ;;
 		esac
 		DISTS="${DISTS} dict"
 		[ "${NO_LIB32:-no}" = "no" -a "${ARCH}" = "amd64" ] &&
@@ -719,6 +738,7 @@ install_from_ftp() {
 				;;
 			allbsd) URL="https://pub.allbsd.org/FreeBSD-snapshots/${ARCH%%.*}-${ARCH##*.}/${V}-JPSNAP/ftp" ;;
 			ftp-archive) URL="http://ftp-archive.freebsd.org/pub/FreeBSD-Archive/old-releases/${ARCH%%.*}/${ARCH##*.}/${V}" ;;
+			freebsdci) URL="https://artifact.ci.freebsd.org/snapshot/${V}/latest_tested/${ARCH%%.*}/${ARCH##*.}" ;;
 			url=*) URL=${METHOD##url=} ;;
 		esac
 
@@ -805,7 +825,7 @@ create_jail() {
 	fi
 
 	case ${METHOD} in
-	ftp|http|gjb|ftp-archive|url=*)
+	ftp|http|gjb|ftp-archive|freebsdci|url=*)
 		FCT=install_from_ftp
 		;;
 	allbsd)
@@ -846,6 +866,9 @@ create_jail() {
 		FCT=install_from_vcs
 		;;
 	git*)
+		if [ ! -x "${GIT_CMD}" ]; then
+			err 1 "Git is not installed. Perhaps you need to 'pkg install git'"
+		fi
 		# Do not check valid version given one can have a specific branch
 		FCT=install_from_vcs
 		;;
@@ -873,10 +896,15 @@ create_jail() {
 
 	# Some methods determine VERSION from newvers.sh if possible
 	# but need to have -v specified otherwise.
-	if [ -z "${VERSION}" ] && \
-	    [ ! -r "${SRC_BASE}/sys/conf/newvers.sh" ]; then
-		usage VERSION
-	fi
+	case "${FCT}" in
+	install_from_vcs) ;;	# Checkout is done in $FCT
+	*)
+		if [ -z "${VERSION}" ] && \
+		    [ ! -r "${SRC_BASE:?}/sys/conf/newvers.sh" ]; then
+			usage VERSION
+		fi
+		;;
+	esac
 
 	if [ "${JAILFS}" != "none" ]; then
 		[ -d "${JAILMNT}" ] && \
@@ -903,11 +931,12 @@ create_jail() {
 	jset ${JAILNAME} method ${METHOD}
 	[ -n "${FCT}" ] && ${FCT} version_extra
 
-	if [ -r "${SRC_BASE}/sys/conf/newvers.sh" ]; then
+	if [ -r "${SRC_BASE:?}/sys/conf/newvers.sh" ]; then
 		RELEASE=$(update_version "${version_extra}")
 	else
 		RELEASE="${VERSION}"
 	fi
+	[ -n "${RELEASE}" ] || err 1 "Failed to determine RELEASE"
 
 	[ "${METHOD}" = "null" ] && \
 	    [ ! -f "${JAILMNT}/etc/login.conf" ] && \
@@ -1136,8 +1165,9 @@ done
 saved_argv="$@"
 shift $((OPTIND-1))
 post_getopts
+[ ${VERBOSE} -gt 0 ] || quiet="-q"
 
-METHOD=${METHOD:-ftp}
+METHOD=${METHOD:-${METHOD_DEF}}
 CLEANJAIL=${CLEAN:-none}
 if [ -n "${JAILNAME}" -a ${CREATE} -eq 0 ]; then
 	_jget ARCH ${JAILNAME} arch || :
@@ -1145,48 +1175,31 @@ if [ -n "${JAILNAME}" -a ${CREATE} -eq 0 ]; then
 	_jget JAILMNT ${JAILNAME} mnt || :
 fi
 
-if [ -n "${SOURCES_URL}" ]; then
+# Handle common (jail+ports) git/svn methods and then fallback to
+# methods only supported by jail.
+if ! svn_git_checkout_method "${SOURCES_URL}" "${METHOD}" \
+    "${SVN_HOST}/base" "${GIT_BASEURL}" \
+    METHOD SVN_FULLURL GIT_FULLURL; then
+	if [ -n "${SOURCES_URL}" ]; then
+		usage
+	fi
 	case "${METHOD}" in
-	svn*)
-		case "${SOURCES_URL}" in
-		http://*) METHOD="svn+http" ;;
-		https://*) METHOD="svn+https" ;;
-		file://*) METHOD="svn+file" ;;
-		svn+ssh://*) METHOD="svn+ssh" ;;
-		svn://*) METHOD="svn" ;;
-		*) err 1 "Invalid svn url" ;;
-		esac
-		;;
-	git*)
-		case "${SOURCES_URL}" in
-		ssh://*) METHOD="git+ssh" ;;
-		http://*) METHOD="git+http" ;;
-		https://*) METHOD="git+https" ;;
-		git://*) METHOD="git" ;;
-		file://*) METHOD="git" ;;
-		*) err 1 "Invalid git url" ;;
-		esac
-		;;
+	allbsd) ;;
+	csup) ;;
+	freebsdci) ;;
+	ftp) ;;
+	ftp-archive) ;;
+	gjb) ;;
+	http) ;;
+	null) ;;
+	src=*) ;;
+	tar=*) ;;
+	url=*) ;;
 	*)
-		err 1 "-U only valid with git and svn methods"
+		msg_error "Unknown method ${METHOD}"
+		usage
 		;;
 	esac
-	SVN_FULLURL=${SOURCES_URL}
-	GIT_FULLURL=${SOURCES_URL}
-else
-	case ${METHOD} in
-	svn+http) proto="http" ;;
-	svn+https) proto="https" ;;
-	svn+ssh) proto="svn+ssh" ;;
-	svn+file) proto="file" ;;
-	svn) proto="svn" ;;
-	git+ssh) proto="ssh" ;;
-	git+http) proto="http" ;;
-	git+https) proto="https" ;;
-	git) proto="git" ;;
-	esac
-	SVN_FULLURL=${proto}://${SVN_HOST}/base
-	GIT_FULLURL=${proto}://${GIT_BASEURL}
 fi
 
 
@@ -1194,7 +1207,7 @@ case "${CREATE}${INFO}${LIST}${STOP}${START}${DELETE}${UPDATE}${RENAME}" in
 	10000000)
 		[ -z "${JAILNAME}" ] && usage JAILNAME
 		case ${METHOD} in
-			src=*|null|tar) ;;
+			src=*|null|git*) ;;
 			*) [ -z "${VERSION}" ] && usage VERSION ;;
 		esac
 		jail_exists ${JAILNAME} && \
