@@ -29,7 +29,6 @@
 
 require_once("guiconfig.inc");
 require_once("interfaces.inc");
-require_once("services.inc");
 require_once("system.inc");
 require_once("plugins.inc.d/rfc2136.inc");
 
@@ -66,10 +65,6 @@ include("head.inc");
 
 legacy_html_escape_form_data($a_rfc2136);
 
-$main_buttons = array(
-    array('label' => gettext('Add'), 'href' => 'services_rfc2136_edit.php'),
-);
-
 ?>
 <body>
   <script>
@@ -125,7 +120,11 @@ $main_buttons = array(
                       <th><?=gettext("Hostname");?></th>
                       <th><?=gettext("Cached IP");?></th>
                       <th><?=gettext("Description");?></th>
-                      <th class"text-nowrap"></th>
+                      <th class"text-nowrap">
+                        <a href="services_rfc2136_edit.php" class="btn btn-primary btn-xs" data-toggle="tooltip" title="<?= html_safe(gettext('Add')) ?>">
+                          <i class="fa fa-plus fa-fw"></i>
+                        </a>
+                      </th>
                     </tr>
                   </thead>
                   <tbody>
@@ -147,7 +146,7 @@ $main_buttons = array(
                         if (file_exists($filename) && !empty($rfc2136['enable']) && (empty($rfc2136['recordtype']) || $rfc2136['recordtype'] == 'A')) {
                             echo "IPv4: ";
                             if (isset($rfc2136['usepublicip'])) {
-                                $ipaddr = get_dyndns_ip($rfc2136['interface'], 4);
+                                $ipaddr = get_rfc2136_ip_address($rfc2136['interface'], 4);
                             } else {
                                 $ipaddr = get_interface_ip($rfc2136['interface']);
                             }
@@ -168,7 +167,7 @@ $main_buttons = array(
                         if (file_exists($filename6) && !empty($rfc2136['enable']) && (empty($rfc2136['recordtype']) || $rfc2136['recordtype'] == 'AAAA')) {
                             echo "IPv6: ";
                             if (isset($rfc2136['usepublicip'])) {
-                                $ipaddr = get_dyndns_ip($rfc2136['interface'], 6);
+                                $ipaddr = get_rfc2136_ip_address($rfc2136['interface'], 6);
                             } else {
                                 $ipaddr = get_interface_ipv6($rfc2136['interface']);
                             }
