@@ -59,18 +59,11 @@ function remove_duplicate($array, $field)
 }
 
 $interfaces = legacy_config_get_interfaces(array('virtual' => false));
-$leasesfile = dhcpd_dhcpv4_leasesfile();
 
 if ($_SERVER['REQUEST_METHOD'] === 'GET') {
-    $awk = "/usr/bin/awk";
-    /* this pattern sticks comments into a single array item */
-    $cleanpattern = "'{ gsub(\"#.*\", \"\");} { gsub(\";\", \"\"); print;}'";
-    /* We then split the leases file by } */
-    $splitpattern = "'BEGIN { RS=\"}\";} {for (i=1; i<=NF; i++) printf \"%s \", \$i; printf \"}\\n\";}'";
-
-    /* stuff the leases file in a proper format into an array by line */
-    exec("/bin/cat {$leasesfile} | {$awk} {$cleanpattern} | {$awk} {$splitpattern}", $leases_content);
+    $leases_content = dhcpd_leases(4);
     $leases_count = count($leases_content);
+
     exec("/usr/sbin/arp -an", $rawdata);
     $arpdata_ip = array();
     $arpdata_mac = array();
@@ -213,16 +206,15 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $macs = [];
     foreach ($leases as $i => $this_lease) {
         if (!empty($this_lease['mac'])) {
-            $macs[$this_lease['mac']] = $i;
+            if (!isset($macs[$this_lease['mac']])) {
+                $macs[$this_lease['mac']] = [];
+            }
+            $macs[$this_lease['mac']][] = $i;
         }
     }
 
-    foreach (dhcpd_staticmap() as $static) {
-        if (!isset($static['ipaddr'])) {
-            continue;
-        }
-
-        $slease = array();
+    foreach (dhcpd_staticmap("not.found", legacy_interfaces_details(), false, 4) as $static) {
+        $slease = [];
         $slease['ip'] = $static['ipaddr'];
         $slease['type'] = 'static';
         $slease['mac'] = $static['mac'];
@@ -237,7 +229,9 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             /* update lease with static data */
             foreach ($slease as $key => $value) {
                 if (!empty($value)) {
-                    $leases[$macs[$slease['mac']]][$key] = $slease[$key];
+                    foreach ($macs[$slease['mac']] as $idx) {
+                        $leases[$idx][$key] = $value;
+                    }
                 }
             }
         } else {
@@ -263,6 +257,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 } elseif ($_SERVER['REQUEST_METHOD'] === 'POST') {
     if (!empty($_POST['deleteip']) && is_ipaddr($_POST['deleteip'])) {
         killbypid('/var/dhcpd/var/run/dhcpd.pid', 'TERM', true);
+        $leasesfile = '/var/dhcpd/var/db/dhcpd.leases'; /* XXX needs wrapper */
         $fin = @fopen($leasesfile, "r");
         $fout = @fopen($leasesfile.".new", "w");
         if ($fin) {
@@ -433,7 +428,7 @@ legacy_html_escape_form_data($leases);
                   <td class="text-nowrap">
 <?php if (!empty($data['if'])): ?>
 <?php if ($data['type'] == 'dynamic'): ?>
-                      <a class="btn btn-default btn-xs" href="services_dhcp_edit.php?if=<?=$data['if'];?>&amp;mac=<?=$data['mac'];?>&amp;hostname=<?=$data['hostname'];?>">
+                      <a class="btn btn-default btn-xs" href="services_dhcp_edit.php?if=<?=$data['if'];?>&amp;mac=<?=$data['mac'];?>&amp;hostname=<?=$data['hostname'];?>&amp;ipaddr=<?=$data['ip'];?>">
                         <i class="fa fa-plus fa-fw" data-toggle="tooltip" title="<?=gettext("add a static mapping for this MAC address");?>"></i>
                       </a>
 <?php if ($data['online'] != 'online'):?>
