@@ -1,6 +1,7 @@
 <?php
 
 /*
+ * Copyright (C) 2019 Pascal Mathis <mail@pascalmathis.com>
  * Copyright (C) 2014-2015 Deciso B.V.
  * Copyright (C) 2008 Shrew Soft Inc. <mgrooms@shrew.net>
  * Copyright (C) 2003-2005 Manuel Kasper <mk@neon1.net>
@@ -32,9 +33,8 @@
 require_once("guiconfig.inc");
 require_once("system.inc");
 require_once("filter.inc");
-require_once("plugins.inc.d/ipsec.inc");
-require_once("services.inc");
 require_once("interfaces.inc");
+require_once("plugins.inc.d/ipsec.inc");
 
 /*
  * ikeid management functions
@@ -62,6 +62,14 @@ function ipsec_ikeid_next() {
     return $ikeid;
 }
 
+function ipsec_keypairs()
+{
+    $mdl = new \OPNsense\IPsec\IPsec();
+    $node = $mdl->getNodeByReference('keyPairs.keyPair');
+
+    return $node ? $node->getNodes() : [];
+}
+
 config_read_array('ipsec', 'phase1');
 config_read_array('ipsec', 'phase2');
 
@@ -78,9 +86,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig['interface'] = "wan";
     $pconfig['iketype'] = "ikev2";
     $phase1_fields = "mode,protocol,myid_type,myid_data,peerid_type,peerid_data
-    ,encryption-algorithm,lifetime,authentication_method,descr,nat_traversal
-    ,interface,iketype,dpd_delay,dpd_maxfail,remote-gateway,pre-shared-key,certref
-    ,caref,reauth_enable,rekey_enable,auto,tunnel_isolation,authservers,mobike";
+    ,encryption-algorithm,lifetime,authentication_method,descr,nat_traversal,rightallowany,inactivity_timeout
+    ,interface,iketype,dpd_delay,dpd_maxfail,dpd_action,remote-gateway,pre-shared-key,certref,margintime,rekeyfuzz
+    ,caref,local-kpref,peer-kpref,reauth_enable,rekey_enable,auto,tunnel_isolation,authservers,mobike,keyingtries
+    ,closeaction";
     if (isset($p1index) && isset($config['ipsec']['phase1'][$p1index])) {
         // 1-on-1 copy
         foreach (explode(",", $phase1_fields) as $fieldname) {
@@ -99,6 +108,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $pconfig['ikeid'] = $config['ipsec']['phase1'][$p1index]['ikeid'];
         }
         $pconfig['disabled'] = isset($config['ipsec']['phase1'][$p1index]['disabled']);
+        $pconfig['sha256_96'] = !empty($config['ipsec']['phase1'][$p1index]['sha256_96']);
         $pconfig['installpolicy'] = empty($config['ipsec']['phase1'][$p1index]['noinstallpolicy']); // XXX: reversed
 
         foreach (array('authservers', 'dhgroup', 'hash-algorithm') as $fieldname) {
@@ -130,16 +140,17 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $pconfig['myid_type'] = "myaddress";
         $pconfig['peerid_type'] = "peeraddress";
         $pconfig['authentication_method'] = "pre_shared_key";
-        $pconfig['encryption-algorithm'] = array("name" => "aes", "keylen" => "128");
-        $pconfig['hash-algorithm'] = array('sha256');
+        $pconfig['encryption-algorithm'] = ["name" => "aes256gcm16"];
+        $pconfig['hash-algorithm'] = ['sha256'];
         $pconfig['dhgroup'] = array('14');
         $pconfig['lifetime'] = "28800";
         $pconfig['nat_traversal'] = "on";
+        $pconfig['installpolicy'] = true;
         $pconfig['authservers'] = array();
 
         /* mobile client */
         if (isset($_GET['mobile'])) {
-            $pconfig['mobile']=true;
+            $pconfig['mobile'] = true;
         }
         // init empty
         foreach (explode(",", $phase1_fields) as $fieldname) {
@@ -162,6 +173,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     if (!isset($pconfig['dpd_enable'])) {
         unset($pconfig['dpd_delay']);
         unset($pconfig['dpd_maxfail']);
+        unset($pconfig['dpd_action']);
     }
 
     /* My identity */
@@ -210,6 +222,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $reqdfields = explode(" ", "caref certref");
             $reqdfieldsn = array(gettext("Certificate Authority"),gettext("Certificate"));
             break;
+        case "pubkey":
+            $reqdfields = explode(" ", "local-kpref peer-kpref");
+            $reqdfieldsn = array(gettext("Local Key Pair"),gettext("Peer Key Pair"));
+            break;
     }
 
     if (empty($pconfig['mobile'])) {
@@ -219,9 +235,29 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 
     do_input_validation($pconfig, $reqdfields, $reqdfieldsn, $input_errors);
 
+    if (!empty($pconfig['inactivity_timeout']) && !is_numericint($pconfig['inactivity_timeout'])) {
+        $input_errors[] = gettext("The inactivity timeout must be an integer.");
+    }
+    if (!empty($pconfig['keyingtries']) && !is_numericint($pconfig['keyingtries']) && $pconfig['keyingtries'] != "-1") {
+        $input_errors[] = gettext("The keyingtries must be an integer.");
+    }
+
     if ((!empty($pconfig['lifetime']) && !is_numeric($pconfig['lifetime']))) {
         $input_errors[] = gettext("The P1 lifetime must be an integer.");
     }
+    if (!empty($pconfig['margintime'])) {
+        if (!is_numericint($pconfig['margintime'])) {
+            $input_errors[] = gettext("The margintime must be an integer.");
+        } else {
+            $rekeyfuzz = empty($pconfig['rekeyfuzz']) || !is_numeric($pconfig['rekeyfuzz']) ? 100 : $pconfig['rekeyfuzz'];
+            if (((int)$pconfig['margintime'] * 2) * ($rekeyfuzz / 100.0) > (int)$pconfig['lifetime']) {
+                $input_errors[] = gettext("The value margin... + margin... * rekeyfuzz must not exceed the original lifetime limit.");
+            }
+        }
+    }
+    if (!empty($pconfig['rekeyfuzz']) && !is_numericint($pconfig['rekeyfuzz'])) {
+        $input_errors[] = gettext("Rekeyfuzz must be an integer.");
+    }
 
     if (!empty($pconfig['remote-gateway'])) {
         if (!is_ipaddr($pconfig['remote-gateway']) && !is_domain($pconfig['remote-gateway'])) {
@@ -308,6 +344,10 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         }
     }
 
+    if (!empty($pconfig['closeaction']) && !in_array($pconfig['closeaction'], ['clear', 'hold', 'restart'])) {
+        $input_errors[] = gettext('Invalid argument for close action.');
+    }
+
     if (!empty($pconfig['dpd_enable'])) {
         if (!is_numeric($pconfig['dpd_delay'])) {
             $input_errors[] = gettext("A numeric value must be specified for DPD delay.");
@@ -315,6 +355,9 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if (!is_numeric($pconfig['dpd_maxfail'])) {
             $input_errors[] = gettext("A numeric value must be specified for DPD retries.");
         }
+        if (!empty($pconfig['dpd_action']) && !in_array($pconfig['dpd_action'], array("restart", "clear"))) {
+            $input_errors[] = gettext('Invalid argument for DPD action.');
+        }
     }
 
     if (!empty($pconfig['iketype']) && !in_array($pconfig['iketype'], array("ike", "ikev1", "ikev2"))) {
@@ -339,18 +382,29 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $pconfig['dhgroup'] = array();
     }
 
-    foreach ($p1_ealgos as $algo => $algodata) {
+    foreach (ipsec_p1_ealgos() as $algo => $algodata) {
         if (!empty($pconfig['iketype']) && !empty($pconfig['encryption-algorithm']['name']) && !empty($algodata['iketype'])
           && $pconfig['iketype'] != $algodata['iketype'] && $pconfig['encryption-algorithm']['name'] == $algo) {
             $input_errors[] = sprintf(gettext("%s can only be used with IKEv2 type VPNs."), $algodata['name']);
         }
     }
 
+    if (!empty($pconfig['ikeid']) && !empty($pconfig['installpolicy'])) {
+        foreach ($config['ipsec']['phase2'] as $phase2ent) {
+            if ($phase2ent['ikeid'] == $pconfig['ikeid'] && $phase2ent['mode'] == 'route-based') {
+                $input_errors[] = gettext(
+                    "Install policy on phase1 is not a valid option when using Route-based phase 2 entries."
+                );
+                break;
+            }
+        }
+    }
+
     if (count($input_errors) == 0) {
         $copy_fields = "ikeid,iketype,interface,mode,protocol,myid_type,myid_data
-        ,peerid_type,peerid_data,encryption-algorithm,
-        ,lifetime,pre-shared-key,certref,caref,authentication_method,descr
-        ,nat_traversal,auto,mobike";
+        ,peerid_type,peerid_data,encryption-algorithm,margintime,rekeyfuzz,inactivity_timeout,keyingtries
+        ,lifetime,pre-shared-key,certref,caref,authentication_method,descr,local-kpref,peer-kpref
+        ,nat_traversal,auto,mobike,closeaction";
 
         foreach (explode(",",$copy_fields) as $fieldname) {
             $fieldname = trim($fieldname);
@@ -366,6 +420,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         }
 
         $ph1ent['disabled'] = !empty($pconfig['disabled']);
+        $ph1ent['sha256_96'] = !empty($pconfig['sha256_96']);
         $ph1ent['noinstallpolicy'] = empty($pconfig['installpolicy']); // XXX: reversed
         $ph1ent['private-key'] =isset($pconfig['privatekey']) ? base64_encode($pconfig['privatekey']) : null;
         if (!empty($pconfig['mobile'])) {
@@ -384,9 +439,14 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $ph1ent['tunnel_isolation'] = true;
         }
 
+        if (isset($pconfig['rightallowany'])) {
+            $ph1ent['rightallowany'] = true;
+        }
+
         if (isset($pconfig['dpd_enable'])) {
             $ph1ent['dpd_delay'] = $pconfig['dpd_delay'];
             $ph1ent['dpd_maxfail'] = $pconfig['dpd_maxfail'];
+            $ph1ent['dpd_action'] = $pconfig['dpd_action'];
         }
 
         /* generate unique phase1 ikeid */
@@ -424,7 +484,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 
         write_config();
         mark_subsystem_dirty('ipsec');
-        header(url_safe('Location: /vpn_ipsec.php'));
+        header(url_safe('Location: /ui/ipsec/tunnels'));
         exit;
     }
 }
@@ -507,6 +567,10 @@ include("head.inc");
                     $(".auth_eap_tls").show();
                     $(".auth_eap_tls :input").prop( "disabled", false );
                     break;
+                case "pubkey":
+                    $(".auth_pubkey").show();
+                    $(".auth_pubkey :input").prop("disabled", false);
+                    break;
                 default: /* psk modes*/
                     $(".auth_psk").show();
                     $(".auth_psk :input").prop( "disabled", false );
@@ -663,18 +727,6 @@ include("head.inc");
                       foreach ($aliaslist as $aliasip => $aliasif) {
                           $interfaces[$aliasip] = $aliasip." (".get_vip_descr($aliasip).")";
                       }
-
-                      $grouplist = return_gateway_groups_array();
-                      foreach ($grouplist as $name => $group) {
-                          if ($group[0]['vip'] != '') {
-                              $vipif = $group[0]['vip'];
-                          } else {
-                              $vipif = $group[0]['int'];
-                          }
-                          $interfaces[$name] = "GW Group {$name}";
-                      }
-
-
                       foreach ($interfaces as $iface => $ifacename) :
 ?>
                         <option value="<?=$iface;?>" <?= $iface == $pconfig['interface'] ? "selected=\"selected\"" : "" ?> >
@@ -691,27 +743,33 @@ include("head.inc");
                       </div>
                     </td>
                   </tr>
-                  <?php if (empty($pconfig['mobile'])) :
-?>
-
+<?php if (empty($pconfig['mobile'])): ?>
                   <tr>
                     <td><a id="help_for_remotegw" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Remote gateway"); ?></td>
                     <td>
-                      <input name="remote-gateway" type="text" class="formfld unknown" id="remotegw" size="28" value="<?=$pconfig['remote-gateway'];?>" />
+                      <input name="remote-gateway" type="text" id="remotegw" size="28" value="<?=$pconfig['remote-gateway'];?>" />
                       <div class="hidden" data-for="help_for_remotegw">
-                        <?=gettext("Enter the public IP address or host name of the remote gateway"); ?>
+                        <?= gettext('Enter the public IP address or host name of the remote gateway.') ?>
                       </div>
                     </td>
                   </tr>
-<?php            endif;
-?>
+                  <tr>
+                    <td><a id="help_for_rightallowany" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('Dynamic gateway') ?></td>
+                    <td>
+                      <input name="rightallowany" type="checkbox" id="rightallowany" value="yes" <?= !empty($pconfig['rightallowany']) ? 'checked="checked"' : '' ?>/>
+                      <?= gettext('Allow any remote gateway to connect') ?>
+                      <div class="hidden" data-for="help_for_rightallowany">
+                        <?= gettext('Recommended for dynamic IP addresses that can be resolved by DynDNS at IPsec startup or update time.') ?>
+                      </div>
+                    </td>
+                  </tr>
+<?php endif ?>
                   <tr>
                     <td><a id="help_for_descr" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Description"); ?></td>
                     <td>
                       <input name="descr" type="text" id="descr" size="40" value="<?=$pconfig['descr'];?>" />
                       <div class="hidden" data-for="help_for_descr">
-                        <?=gettext("You may enter a description here " .
-                                                "for your reference (not parsed)."); ?>
+                        <?=gettext("You may enter a description here for your reference (not parsed)."); ?>
                       </div>
                     </td>
                   </tr>
@@ -726,7 +784,7 @@ include("head.inc");
                     <td>
                       <select name="authentication_method" id="authentication_method">
 <?php
-                      foreach ($p1_authentication_methods as $method_type => $method_params) :
+                      foreach (ipsec_p1_authentication_methods() as $method_type => $method_params) :
                           if (empty($pconfig['mobile']) && $method_params['mobile']) {
                               continue;
                           }
@@ -774,7 +832,8 @@ include("head.inc");
                         'user_fqdn' => array( 'desc' => gettext('User distinguished name'), 'mobile' => true ),
                         'asn1dn' => array( 'desc' => gettext('ASN.1 distinguished Name'), 'mobile' => true ),
                         'keyid tag' => array( 'desc' => gettext('KeyID tag'), 'mobile' => true ),
-                        'dyn_dns' => array( 'desc' => gettext('Dynamic DNS'), 'mobile' => true ));
+                        'dyn_dns' => array( 'desc' => gettext('Dynamic DNS'), 'mobile' => true ),
+                        'auto' => array( 'desc' => gettext('Automatic'), 'mobile' => true ));
                       foreach ($my_identifier_list as $id_type => $id_params) :
 ?>
                         <option value="<?=$id_type;?>" <?php if ($id_type == $pconfig['myid_type']) {
@@ -792,7 +851,7 @@ endforeach; ?>
                   </tr>
 <?php
                   if (empty($pconfig['mobile'])):?>
-                  <tr class="auth_opt auth_eap_tls auth_psk">
+                  <tr class="auth_opt auth_eap_tls auth_psk auth_pubkey">
                     <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Peer identifier"); ?></td>
                     <td>
                       <select name="peerid_type" id="peerid_type">
@@ -803,7 +862,8 @@ endforeach; ?>
                         'fqdn' => array( 'desc' => gettext('Distinguished name'), 'mobile' => true ),
                         'user_fqdn' => array( 'desc' => gettext('User distinguished name'), 'mobile' => true ),
                         'asn1dn' => array( 'desc' => gettext('ASN.1 distinguished Name'), 'mobile' => true ),
-                        'keyid tag' => array( 'desc' =>gettext('KeyID tag'), 'mobile' => true ));
+                        'keyid tag' => array( 'desc' =>gettext('KeyID tag'), 'mobile' => true ),
+                        'auto' => array( 'desc' => gettext('Automatic'), 'mobile' => true ));
                       foreach ($peer_identifier_list as $id_type => $id_params) :
                         if (!empty($pconfig['mobile']) && !$id_params['mobile']) {
                           continue;
@@ -828,7 +888,7 @@ endforeach; ?>
                   <tr class="auth_opt auth_psk">
                     <td><a id="help_for_psk" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Pre-Shared Key"); ?></td>
                     <td>
-                      <input name="pre-shared-key" type="text" class="formfld unknown" id="pskey" size="40"
+                      <input name="pre-shared-key" type="text" id="pskey" size="40"
                              value="<?= $pconfig['authentication_method'] == "pre_shared_key" || $pconfig['authentication_method'] == "xauth_psk_server" ? $pconfig['pre-shared-key'] : "";?>" />
                       <div class="hidden" data-for="help_for_psk">
                         <?=gettext("Input your Pre-Shared Key string."); ?>
@@ -878,6 +938,52 @@ endforeach; ?>
                       </div>
                     </td>
                   </tr>
+                  <tr class="auth_opt auth_pubkey">
+                      <td><a id="help_for_pubkey_local" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Local Key Pair"); ?></td>
+                      <td>
+                          <select name="local-kpref">
+                              <?php
+                              foreach (ipsec_keypairs() as $keypair_uuid => $keypair) :
+                                  if ($keypair['publicKey'] and $keypair['privateKey']) :
+                                      ?>
+                                      <option value="<?= $keypair_uuid; ?>" <?= isset($pconfig['local-kpref']) && $pconfig['local-kpref'] == $keypair_uuid ? "selected=\"selected\"" : "" ?>>
+                                          <?= $keypair['name']; ?>
+                                      </option>
+                                  <?php
+                                  endif;
+                              endforeach;
+                              ?>
+                          </select>
+                          <div class="hidden" data-for="help_for_pubkey_local">
+                              <?= gettext("Select a local key pair previously configured at IPsec \\ Key Pairs."); ?>
+                              <br />
+                              <?= gettext("This selection will only display key pairs which have both a public and private key."); ?>
+                          </div>
+                      </td>
+                  </tr>
+                  <tr class="auth_opt auth_pubkey">
+                      <td><a id="help_for_pubkey_peer" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Peer Key Pair"); ?></td>
+                      <td>
+                          <select name="peer-kpref">
+                              <?php
+                              foreach (ipsec_keypairs() as $keypair_uuid => $keypair) :
+                                  if ($keypair['publicKey']) :
+                                      ?>
+                                      <option value="<?= $keypair_uuid; ?>" <?= isset($pconfig['peer-kpref']) && $pconfig['peer-kpref'] == $keypair_uuid ? "selected=\"selected\"" : "" ?>>
+                                          <?= $keypair['name']; ?>
+                                      </option>
+                                  <?php
+                                  endif;
+                              endforeach;
+                              ?>
+                          </select>
+                          <div class="hidden" data-for="help_for_pubkey_peer">
+                              <?=gettext("Select a peer key pair previously configured at IPsec \\ Key Pairs."); ?>
+                              <br />
+                              <?= gettext("This selection will only display key pairs which have a public key."); ?>
+                          </div>
+                      </td>
+                  </tr>
                   <tr class="auth_opt auth_eap_radius">
                     <td><a id="help_for_authservers" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Radius servers"); ?></td>
                     <td>
@@ -901,11 +1007,11 @@ endforeach; ?>
                     <td colspan="2"><b><?=gettext("Phase 1 proposal (Algorithms)"); ?></b></td>
                   </tr>
                   <tr>
-                    <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Encryption algorithm"); ?></td>
+                    <td><a id="help_for_encalg" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Encryption algorithm"); ?></td>
                     <td>
                       <select name="ealgo" id="ealgo" data-default-keylen="<?=$pconfig['encryption-algorithm']['keylen'];?>">
 <?php
-                      foreach ($p1_ealgos as $algo => $algodata) :
+                      foreach (ipsec_p1_ealgos() as $algo => $algodata) :
                       ?>
                         <option value="<?=$algo;?>" <?= $algo == $pconfig['encryption-algorithm']['name'] ? "selected=\"selected\"" : "" ;?>
                                 data-hi="<?=$algodata['keysel']['hi'];?>"
@@ -921,6 +1027,9 @@ endforeach; ?>
 
                       <select name="ealgo_keylen" id="ealgo_keylen" width="30">
                       </select>
+                      <div class="hidden" data-for="help_for_encalg">
+                          <?=gettext("Note: For security reasons avoid the use of DES,3DES,CAST and BLOWFISH protocols"); ?>.
+                      </div>
                     </td>
                   </tr>
                   <tr>
@@ -945,7 +1054,7 @@ endforeach; ?>
 ?>
                       </select>
                       <div class="hidden" data-for="help_for_halgo">
-                        <?=gettext("Must match the setting chosen on the remote side."); ?>
+                        <?=gettext("Note: For security reasons avoid the use of MD5 and SHA1 algorithms."); ?>
                       </div>
                     </td>
                   </tr>
@@ -972,6 +1081,7 @@ endforeach; ?>
                            28 => '28 (Brainpool EC 256 bits)',
                            29 => '29 (Brainpool EC 384 bits)',
                            30 => '30 (Brainpool EC 512 bits)',
+                           31 => '31 (Elliptic Curve 25519)',
                       );
                       foreach ($p1_dhgroups as $keygroup => $keygroupname):
 ?>
@@ -1035,10 +1145,23 @@ endforeach; ?>
                       </div>
                     </td>
                   </tr>
+                  <tr>
+                    <td><a id="help_for_sha256_96" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('SHA256 96 Bit Truncation') ?></td>
+                    <td>
+                      <input name="sha256_96" type="checkbox" id="sha256_96" value="yes" <?= !empty($pconfig['sha256_96']) ? 'checked="checked"' : '' ?>/>
+                      <div class="hidden" data-for="help_for_sha256_96">
+                        <?= gettext(
+                          "For compatibility with implementations that incorrectly use 96-bit (instead of 128-bit) truncation this ".
+                          "option may be enabled to configure the shorter truncation length. This is not negotiated, so this only works ".
+                          "with peers that use the incorrect truncation length (or have this option enabled), e.g. Forcepoint Sidewinder."
+                        ) ?>
+                      </div>
+                    </td>
+                  </tr>
                   <tr>
                     <td><a id="help_for_nat_traversal" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("NAT Traversal"); ?></td>
                     <td>
-                      <select name="nat_traversal">
+                      <select name="nat_traversal" class="selectpicker">
                         <option value="off" <?= isset($pconfig['nat_traversal']) && $pconfig['nat_traversal'] == "off" ? "selected=\"selected\"" :"" ;?> >
                           <?=gettext("Disable"); ?>
                         </option>
@@ -1064,6 +1187,39 @@ endforeach; ?>
                       </div>
                     </td>
                   </tr>
+                  <tr>
+                    <td><a id="help_for_closeaction" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("Close Action"); ?></td>
+                    <td>
+                      <select name="closeaction" class="selectpicker">
+                        <option value="" <?= empty($pconfig['closeaction']) ? "selected=\"selected\"" :"" ;?> >
+                          <?=gettext("None"); ?>
+                        </option>
+                        <option value="clear" <?= $pconfig['closeaction'] == "clear" ? "selected=\"selected\"" :"" ;?> >
+                          <?=gettext("Clear"); ?>
+                        </option>
+                        <option value="hold" <?= $pconfig['closeaction'] == "hold" ? "selected=\"selected\"" :"" ;?> >
+                          <?=gettext("Hold"); ?>
+                        </option>
+                        <option value="restart" <?= $pconfig['closeaction'] == "restart" ? "selected=\"selected\"" :"" ;?> >
+                          <?=gettext("Restart"); ?>
+                        </option>
+                      </select>
+                      <div class="hidden" data-for="help_for_closeaction">
+                          <?=gettext(
+                            "Defines the action to take if the remote peer unexpectedly closes a CHILD_SA. ".
+                            "A closeaction should not be used if the peer uses reauthentication or uniqueids checking, ".
+                            "as these events might trigger the defined action when not desired. "
+                          )?>
+                          <br/></br>
+                          <?=gettext(
+                            "With clear the connection is closed with no further actions taken. ".
+                            "hold installs a trap policy, which will catch matching traffic and tries to re-negotiate ".
+                            "the connection on demand. restart will immediately trigger an attempt ".
+                            "to re-negotiate the connection. The default is none and disables the close action."
+                          )?>
+                      </div>
+                    </td>
+                  </tr>
                   <tr>
                     <td><a id="help_for_dpd_enable" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("Dead Peer Detection"); ?></td>
                     <td>
@@ -1073,17 +1229,66 @@ endforeach; ?>
                       </div>
                       <div id="opt_dpd">
                         <br />
-                        <input name="dpd_delay" type="text" class="formfld unknown" id="dpd_delay" size="5" value="<?=$pconfig['dpd_delay'];?>" />
+                        <input name="dpd_delay" type="text" id="dpd_delay" size="5" value="<?=$pconfig['dpd_delay'];?>" />
                         <?=gettext("seconds"); ?>
                         <div class="hidden" data-for="help_for_dpd_enable">
                           <?=gettext("Delay between requesting peer acknowledgement."); ?>
                         </div>
                         <br />
-                        <input name="dpd_maxfail" type="text" class="formfld unknown" id="dpd_maxfail" size="5" value="<?=$pconfig['dpd_maxfail'];?>" />
+                        <input name="dpd_maxfail" type="text" id="dpd_maxfail" size="5" value="<?=$pconfig['dpd_maxfail'];?>" />
                         <?=gettext("retries"); ?>
                         <div class="hidden" data-for="help_for_dpd_enable">
                           <?=gettext("Number of consecutive failures allowed before disconnect."); ?>
                         </div>
+                        <br />
+                        <select name="dpd_action" class="selectpicker">
+                          <option value="" <?=empty($pconfig['dpd_action']) ?  "selected=\"selected\"" : ""; ?>><?=gettext("default");?></option>
+                          <option value="restart" <?=$pconfig['dpd_action'] == "restart" ?  "selected=\"selected\"" : ""; ?>><?=gettext("Restart the tunnel");?></option>
+                          <option value="clear" <?=$pconfig['dpd_action'] == "clear" ?  "selected=\"selected\"" : ""; ?>><?=gettext("Stop the tunnel");?></option>
+                        </select>
+                        <?=gettext("DPD action"); ?>
+                        <div class="hidden" data-for="help_for_dpd_enable">
+                          <?=gettext("Choose the behavior here what to do if a peer is detected to be unresponsive to DPD requests."); ?>
+                        </div>
+                      </div>
+                    </td>
+                  </tr>
+                  <tr>
+                    <td><a id="help_for_inactivity_timeout" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("Inactivity timeout"); ?></td>
+                    <td>
+                      <input name="inactivity_timeout" type="text" id="inactivity_timeout" value="<?=$pconfig['inactivity_timeout'];?>" />
+                      <div class="hidden" data-for="help_for_inactivity_timeout">
+                        <?=gettext("Time before closing inactive tunnels if they don't handle any traffic. (seconds)"); ?>
+                      </div>
+                    </td>
+                  </tr>
+                  <tr>
+                    <td><a id="help_for_keyingtries" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("Keyingtries"); ?></td>
+                    <td>
+                      <input name="keyingtries" type="text" id="keyingtries" value="<?=$pconfig['keyingtries'];?>" />
+                      <div class="hidden" data-for="help_for_keyingtries">
+                        <?=gettext(
+                          "How many attempts should be made to negotiate a connection, or a replacement for one, before giving up (default 3). ".
+                          "Leave empty for default, -1 for forever or any positive integer for the number of tries"
+                        ); ?>
+                      </div>
+                    </td>
+                  </tr>
+                  <tr>
+                    <td><a id="help_for_margintime" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("Margintime"); ?></td>
+                    <td>
+                      <input name="margintime" type="text" id="margintime" value="<?=$pconfig['margintime'];?>" />
+                      <div class="hidden" data-for="help_for_margintime">
+                        <?=gettext("Time before SA expiry the rekeying should start. (seconds)"); ?>
+                      </div>
+                    </td>
+                  </tr>
+                  <tr>
+                    <td><a id="help_for_rekeyfuzz" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a>  <?=gettext("Rekeyfuzz"); ?></td>
+                    <td>
+                      <input name="rekeyfuzz" type="text" id="rekeyfuzz" value="<?=$pconfig['rekeyfuzz'];?>" />
+                      <div class="hidden" data-for="help_for_rekeyfuzz">
+                        <?=gettext("Percentage by which margintime is randomly increased (may exceed 100%). Randomization may be disabled by setting rekeyfuzz=0%."); ?>
                       </div>
                     </td>
                   </tr>
